//------------------------------------------------------------------------------
// <Copyright From='2004' To='2006' Company='Microsoft Corporation'> 
//		Copyright (c) Microsoft Corporation. All Rights Reserved. 
//		Information Contained Herein is Proprietary and Confidential. 
// </Copyright>
//
// Owner: MSyed
//------------------------------------------------------------------------------

using System;
using System.Web;
using System.Web.UI;

using Microsoft.Passport.RPS;

namespace Microsoft.MapPoint.VirtualEarth.Website
{
	public class BaseRPSPage : Page
	{
		protected RPS rpsSession;
		
		protected RPSHttpAuth rpsHttpAuth;
		protected RPSDomainMap rpsDomainMap;
		protected RPSPropBag rpsAuthenticatePropBag;
		protected RPSPropBag rpsValidatePropBag;
		protected RPSPropBag rpsConstructPropBag;

		protected RPSTicket rpsTicket;

		protected bool hasTicket = false;
		protected bool validated = false;
		
		protected uint authState = 0;

		protected string siteName = string.Empty;
		protected string rpsHeaders = string.Empty;
		protected string silentUrl = string.Empty;

        public BaseRPSPage()
        {
            try
            {
                HttpContext.Current.Response.Cache.SetCacheability(HttpCacheability.NoCache);

                if (HttpContext.Current.Application["globalRPS"] != null && HttpContext.Current.Application["globalRPS"] is RPS)
                {
                    rpsSession = HttpContext.Current.Application["globalRPS"] as RPS;
                }
                else
                {
                    rpsSession = new RPS();
                    rpsSession.Initialize(null);
                }

                // Initialize RPS objects
                rpsHttpAuth = rpsSession.GetObject("rps.http.auth") as RPSHttpAuth;
                rpsDomainMap = rpsSession.GetObject("rps.net.domainmap") as RPSDomainMap;
                rpsAuthenticatePropBag = rpsSession.GetObject("rps.property.bag") as RPSPropBag;
                rpsValidatePropBag = rpsSession.GetObject("rps.property.bag") as RPSPropBag;
                rpsConstructPropBag = rpsSession.GetObject("rps.property.bag") as RPSPropBag;

                // Set the site context variable (this should match a Site node's SiteName attribute value in RPSServer.xml
                siteName = HttpContext.Current.Request.ServerVariables["SERVER_NAME"];

                // Call Authenticate to parse any RPS token
                rpsTicket = rpsHttpAuth.Authenticate(siteName, HttpContext.Current.Request, rpsAuthenticatePropBag);
                hasTicket = (rpsTicket != null);
                authState = (uint)rpsAuthenticatePropBag["RPSAuthState"];
                rpsHeaders = rpsAuthenticatePropBag["RPSRespHeaders"].ToString();

                rpsConstructPropBag["ReturnURL"] = HttpContext.Current.Request.Url.ToString();
                silentUrl = rpsDomainMap.ConstructURL("silentauth", siteName, null /*domain*/, rpsConstructPropBag);

                if (hasTicket)
                {
                    validated = rpsTicket.Validate(rpsValidatePropBag);
					if (validated)
					{
						if (HttpContext.Current.Response != null && rpsHeaders != null)
						{
							rpsHttpAuth.WriteHeaders(HttpContext.Current.Response, rpsHeaders);
						}
					}
                }
            }
            catch (Exception ex)
            {
                //TODO: Log Passport failure
            }
            finally
            {
                //Do anything commmon?
            }
        }
	}
}
