//------------------------------------------------------------------------------
// API/GraphicsManager.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$GraphicsManager$</header>
//------------------------------------------------------------------------------

//class VEGraphicsManager
//public interface:
//   VEGraphicsManager(vemap)
//      constructor
//      parameters:
//         vemap: VEMap instance this graphics manager is associated with
//
//   Initialize()
//   Draw()
//   DrawLine(vePolyline)
//   RemoveLine(vePolyline)
//   RemoveLinebyId(vePolylineId)
//   Clear()
//   Dispose()
//
function VEGraphicsManager(vemap)
{
	VEValidator.ValidateObject(vemap, "vemap", VEMap, "VEMap");
	var self = this;      //so event callbacks can access "this"
	this.m_vemap = vemap;
	this.m_vemapcontrol = this.m_vemap.vemapcontrol;
	var m_veLatLongDecoder = new VELatLongDecoder();
	
	//Initialize
	this.Initialize = function()
	{
		if(this.m_vegraphiccanvas == null || this.m_vegraphiccanvas == "undefined")
		{
			try
			{
				this.m_vegraphicspolylines = new Array();
				this.m_vegraphiccanvas = Msn.Drawing.Graphic.CreateGraphic(this.m_vemap.mapelement);
				this.m_vegraphiccanvas.SetZIndex(17);
				this.m_vemapcontrol.AttachEvent("onstartzoom", self.Clear);
				this.m_vemapcontrol.AttachEvent("onendzoom", self.Draw);
				this.m_vemapcontrol.AttachEvent("onstartcontinuouspan", self.Clear);
				this.m_vemapcontrol.AttachEvent("onmouseup", self.Draw);
				//make sure the polylines get redrawn when changes happen in birdseye mode
				this.m_vemapcontrol.AttachEvent("onobliquechange", self.Update);
			}
			catch(ex)
			{
				throw ex;
				return;
			}
		}
	}
	
	//Update
	//   event callback which repaints the polylines
	this.Update = function()
	{
		self.Clear();
		self.Draw();
	}

	//Clear
	this.Clear = function()
	{
		//Clear is also used as an event callback, so it must use self to access "this"
		if(self.m_vegraphiccanvas != null &&
			self.m_vegraphiccanvas != "undefined")
		{
			self.m_vegraphiccanvas.Clear();
		}
	}

	//Draw
	this.Draw = function()
	{
		//Draw is also used as an event callback, so it must use self to access "this"
		//TODO: Do a better job of keeping track of showing lines
		if(self.m_vegraphicspolylines == null || 
			self.m_vegraphicspolylines.length <= 0)
		{
			return;
		}

		var lineslength = self.m_vegraphicspolylines.length;
		for(var j = 0;j<lineslength;j++)
		{
			var vepolyline = self.m_vegraphicspolylines[j];
			var points = new Array();
			var ll = null;
			var pix = null;
			var vepoints = vepolyline.GetLatLongs();
			for(var i=0;i<vepoints.length;i++)
			{
				var veLatLongDecoded = m_veLatLongDecoder.Decode(vepoints[i]);
				ll = new Msn.VE.LatLong(veLatLongDecoded.Latitude, veLatLongDecoded.Longitude);
				pix = self.m_vemapcontrol.LatLongToPixel(ll);
				points.push(new Msn.Drawing.Point(pix.x, pix.y));
			}
			
			var polyline = new Msn.Drawing.PolyLine(points);
			self.m_vegraphiccanvas.SetZIndex(17);
			self.m_vegraphiccanvas.SetStroke(vepolyline.Stroke);
			self.m_vegraphiccanvas.DrawPolyline(polyline);
		}
	}
}

//RemoveLine
VEGraphicsManager.prototype.RemoveLine = function(vePolyline)
{
	this.RemoveLinebyId(vePolyline.ID);
}

//RemoveLinebyId
VEGraphicsManager.prototype.RemoveLinebyId = function(vePolylineId)
{
    if(this.m_vegraphiccanvas == null || this.m_vegraphiccanvas == "undefined")
	{
		throw new VEException("VEMap:DrawLine", "err_GraphicsInitError", L_GraphicsInitError_Text);
		return;
	}
	try
	{
		var linelength = this.m_vegraphicspolylines.length;
		var i=0;

		while (i < linelength && vePolylineId != this.m_vegraphicspolylines[i].ID)
		{
			i++;
		}
		if (i < linelength)
		{
			this.m_vegraphicspolylines.splice(i,1);
		}
		else
		{
			throw new VEException("VEMap:DrawLine", "err_GraphicsInitError", L_invalidpolylineid_text);
			return;
		}		
	}
	catch(ex)
	{
		throw ex;
		return;
	}

	this.Clear();
	this.Draw();
}

//RemoveAllLines
VEGraphicsManager.prototype.RemoveAllLines = function()
{
	//Clear the polylines array and repaint
	this.m_vegraphicspolylines = new Array();
	this.Clear();
	this.Draw();
}

//DrawLine
VEGraphicsManager.prototype.DrawLine = function(vePolyline)
{
	if(this.m_vegraphiccanvas == null || this.m_vegraphiccanvas == "undefined")
	{
		throw new VEException("VEMap:DrawLine", "err_GraphicsInitError", L_GraphicsInitError_Text);
		return;
	}

	try
	{
		VEValidator.ValidateObject(vePolyline, "vePolyline", VEPolyline, "VEPolyline");
	
		//See if it already exists
		var linelength = this.m_vegraphicspolylines.length;
		for(var i=0;i<linelength;i++)
		{
			if(vePolyline.ID == this.m_vegraphicspolylines[i].ID)
			{
				throw new VEException("VEMap:DrawLine", "err_invalidpolylineid", L_invalidpolylineid_text);
			}
		}

		this.m_vegraphicspolylines.push(vePolyline);
	}
	catch(ex)
	{
		throw ex;
		return;		
	}
	
	this.Draw();
}

//Dispose
VEGraphicsManager.prototype.Dispose = function()
{
	if(this.m_vegraphiccanvas != null)
	{
		this.m_vegraphiccanvas.Clear();
		this.m_vegraphiccanvas.Destroy();
		this.m_vegraphiccanvas = null;
	}
	
	//Clear cache
	if(this.m_vegraphicspolylines != null && 
		this.m_vegraphicspolylines != "undefined")
	{
		var lineslength = this.m_vegraphicspolylines.length;
		for(var j = 0;j<lineslength;j++)
		{
			this.m_vegraphicspolylines.pop();
		}
	}
	
	//Detach events
	if(this.m_vemap != null)
	{
		this.m_vemapcontrol.DetachEvent("onstartzoom",this.Clear);
		this.m_vemapcontrol.DetachEvent("onendzoom",this.Draw);
		this.m_vemapcontrol.DetachEvent("onstartcontinuouspan",this.Clear );
		this.m_vemapcontrol.DetachEvent("onmouseup",this.Draw);
	}
}
