//------------------------------------------------------------------------------
// API/LatLon.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$LatLon$</header>
//------------------------------------------------------------------------------

//VELatLong
//   NOTE: VELatLongs that are returned from APIs need to be instatiated from the
//   VELatLongFactory so the oblique encoding is done properly.
//   If the VELatLong is not encoded, then Latitude and Longitude will be valid and
//   _reserved will be null.  If the VELatLong is encoded, then Latitude and Longitude
//   will be null and the _reserved field will contain a hash string representation
//   of the LatLong.  Use of the VELatLong whether it is encoded or not should be
//   transparent to the user when passing them/receiving them from API calls.
function VELatLong(latitude, longitude)
{
	try
	{
		this.Latitude = null;
		this.Longitude = null;
		this._reserved = null;

		if ((latitude != null) && (latitude != "undefined"))
		{
			VEValidator.ValidateFloat(latitude, "latitude");
			this.Latitude = latitude;
		}
		
		if ((longitude != null) && (longitude != "undefined"))
		{
			VEValidator.ValidateFloat(longitude, "longitude");
			this.Longitude = longitude;
		}
	}
	catch(ex)
	{
		throw ex;
		return;
	}
}

//Clones a copy of the VELatLong
function Clone()
{
	var veLatLong = new VELatLong();
	veLatLong.Latitude = this.Latitude;
	veLatLong.Longitude = this.Longitude;
	veLatLong._reserved = this._reserved;
	return veLatLong;
}

function toString()
{
	var s = "";
	if ((this.Latitude != null) && (this.Longitude != null))
	{
		s = this.Latitude + ", " + this.Longitude;
	}
	return s;
}

//Prototype assignment
VELatLong.prototype.toString = toString;
VELatLong.prototype.Clone = Clone;

//VELatLongRectangle
//   TopLeftLatLong: VELatLong of top left rectangle coordinate
//   BottomRightLatLong: VELatLong of bottom right rectangle coordinate
function VELatLongRectangle(topLeftLatLong, bottomRightLatLong)
{
	VEValidator.ValidateObject(topLeftLatLong, "topLeftLatLong", VELatLong, "VELatLong");
	VEValidator.ValidateObject(bottomRightLatLong, "bottomRightLatLong", VELatLong, "VELatLong");
	this.TopLeftLatLong = topLeftLatLong;
	this.BottomRightLatLong = bottomRightLatLong;
}

