//------------------------------------------------------------------------------
// API/LatLongFactory.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: keithkin
//
// <header>	$LatLong$</header>
//------------------------------------------------------------------------------

//VELatLongFactoryAlwaysEncodeSpec
//   A speification which can be passed to a VELatLongFactory which 
//   tells the factory to always encode the VELatLongs.
function VELatLongFactoryAlwaysEncodeSpec()
{
	this.IsEncode = function()
	{
		return true;
	}
}

//VELatLongFactorySpecFromMap
//   A specification which can be passed to a VELatLongFactory which 
//   tells the factory how to create VELatLongs based on the map settings.
function VELatLongFactorySpecFromMap(vemap)
{
	VEValidator.ValidateObject(vemap, "vemap", VEMap, "VEMap");
	var m_vemap = vemap;
	
	this.IsEncode = function()
	{
		//Encode the VELatLong if the map style is Birdseye
		return (m_vemap.GetMapStyle() == VEMapStyle.Birdseye);
	}
}

//VELatLongFactorySpecFromMapView
//   A specification which can be passed to a VELatLongFactory which 
//   tells the factory how to create VELatLongs based on the MapView settings.
function VELatLongFactorySpecFromMapView(mapView)
{
	VEValidator.ValidateNonNull(mapView, "mapView");
	var m_mapView = mapView;
	
	this.IsEncode = function()
	{
		//Encode the VELatLong if the map view is Oblique (aka Birdseye)
		return (m_mapView.mapStyle == Msn.VE.MapStyle.Oblique);
	}
}

///VELatLongFactory
//   A factory that creates VELatLongs based on a passed in specification.
//   The specification must implement the "IsEncode" method which tells the
//   factory whether or not to encode the created VELatLong.
//   See VELatLong for details on what fields will be populated for encoding/decoding.
//   It should be seamless to the external user of the API that they can pass
//   VELatLongs anywhere regardless of whether they are encoded or not.
//
function VELatLongFactory(veLatLongFactorySpec)
{
	VEValidator.ValidateNonNull(veLatLongFactorySpec, "veLatLongFactorySpec");
	var m_spec = veLatLongFactorySpec;
	var m_latLongEncoder = new VELatLongEncoding();
	
	//CreateVELatLong
	this.CreateVELatLong=function(latitude, longitude)
	{
		var veLatLong = null;
		//If the spec says to encode the VELatLong, then encode it.
		if  (m_spec.IsEncode())
		{
			veLatLong = new VELatLong();
			veLatLong._reserved = m_latLongEncoder.Encode(latitude, longitude);
		}
		else
		{
			veLatLong = new VELatLong(latitude, longitude);
		}

		return veLatLong;
	}
}	