//------------------------------------------------------------------------------
// API/Polyline.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Cthota, keithkin
//
// <header>	$Polyline$</header>
//------------------------------------------------------------------------------

//VEPolyline constructor
//   parameters:
//      id:           id for the polyline
//      arrVELatLong: array of VELatLongs
//      color:        optional VEColor object. Default is VEColor(17,221,17,0.7) = #11DD11
//      width:        optional width. Default is 6.
function VEPolyline(id, arrVELatLong, color, width )
{
	VEValidator.ValidateNonNull(id, "id");	
	VEValidator.ValidateNonNull(arrVELatLong, "arrVELatLong");	
	this.ID=id; 
	this.LatLongs=arrVELatLong;
	this.Stroke = new Msn.Drawing.Stroke();
	if(color == null || color == "undefined")
	{
		//color not specified. Use default color.
		color = new VEColor(17, 221, 17, 0.7);
	}
	this.SetColor(color);

	if(width == null || width == "undefined")
	{
		//width not specified. Use default width.
		width=6;
	}
	this.SetWidth(width);
}

//SetColor
//   parameters:
//      color: VEColor object
function SetColor(color)
{
	VE_Analytics.LogAPI('VE | APIs','VEPolyline - SetColor') ;
	VEValidator.ValidateNonNull(color);
	this.Stroke.color = new Msn.Drawing.Color(color.R, color.G, color.B, color.A);
}

function SetWidth(width)
{
	VE_Analytics.LogAPI('VE | APIs','VEPolyline - SetWidth') ;
	VEValidator.ValidateInt(width, "width");
	this.Stroke.width = width;
}

function GetLatLongs()
{
	return this.LatLongs;
}

//Prototypes
VEPolyline.prototype.SetColor=SetColor;
VEPolyline.prototype.SetWidth=SetWidth;
VEPolyline.prototype.GetLatLongs=GetLatLongs;