//------------------------------------------------------------------------------
// Annotation/AnnotationDatastrucutres.js - Datastructures, manipulators,
//                                          serializers, deserializers
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, shankak, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//
// Upgraded these files to include new fields in the schema.
//
//------------------------------------------------------------------------------
//
// hash of annotation data keyed on pinID
VE_AnnotationData.DataHash = new Array ();

//
// the annotation data object constructor
function VE_AnnotationData (pinId, lat, lon, title, notes,infoUrl,photoUrl)
{
	// TODO: make this a REAL object with sets/gets
	this.pinId = pinId;
	this.lat = lat;
	this.lon = lon;
	this.title = title;
	this.notes = notes;
	this.infoUrl = infoUrl;
	this.photoUrl = photoUrl;
}

//
// creates an annotation data object if it doesn't exist
// stores it in the datahash
// also modifies an existing annotation data object if one exists
VE_AnnotationData.StoreAnnotation = function (pinId, lat, lon, title, notes, infoUrl,photoUrl)
{
	var adh = VE_AnnotationData.DataHash;
	var ad = adh[ pinId ];

	if ( !ad )
	{
		// create a new instance
		title = (title) ? unescape(title) : "";
		notes = (notes) ? unescape(notes) : "";
		infoUrl = (infoUrl) ? unescape(infoUrl) : "";
		photoUrl = (photoUrl) ? unescape(photoUrl)  : "";
		ad = new VE_AnnotationData (pinId, lat, lon, title, notes, infoUrl,photoUrl);
		//Store it in the hash
		adh[ pinId ] = ad;
		return ad;
	}
	

	if (title)
		ad.title = unescape(title);

	if(notes)
		ad.notes = unescape(notes);

	if(infoUrl)
		ad.infoUrl = unescape(infoUrl);

	if(photoUrl)
		ad.photoUrl = unescape(photoUrl);

		
	if (lat && lon)
	{
		ad.lat = lat;
		ad.lon = lon;
	}
	
	return ad;
}

//
// removes the annotation from the hash
VE_AnnotationData.RemoveAnnotation = function (pinId)
{
	var adh = VE_AnnotationData.DataHash;
	var ad = adh[ pinId ];
	
	if ( !ad || !ad.valid )
	{
		return false;
	}
	
	adh[ pinId ] = null;
	return true;
}

//
// get an annotation by its pinID
// if the annotation doesn't exist, one is created with default values
// defined in AnnotationConf.js
VE_AnnotationData.GetAnnotation = function (pinId)
{
	var ad = VE_AnnotationData.DataHash[ pinId ];
	if ( !ad )
	{
		// TODO: assess the validity of this behavior
		// TODO: remove use of 'pin' and 'vePushpin' from annotations
		var cpin = document.getElementById ('context_pin');
		if (!cpin)
		{
			throw "VE_AnnotationData.GetAnnotation: Annotation could not be found.";
		}
		else
		{
			var lat = cpin.vePushpin.GetLatitude ();
			var lon = cpin.vePushpin.GetLongitude ();
			ad = VE_AnnotationData.StoreAnnotation (pinId, lat, lon, VE_Annotations.DefaultTitle, VE_Annotations.DefaultNote, VE_Annotations.DefaultInfoUrl,VE_Annotations.DefaultPhotoUrl);
		}
	}
	return ad;
}

//
// serialize an annotation
// look it up by its pinId
// return the serialized form:
//   lat_lon_title_notes
// any internal _ are url escaped
VE_AnnotationData.SerializeAnnotation = function ( pinId)
{
	var ad = VE_AnnotationData.GetAnnotation ( pinId );
	
	var lat = ad.lat;
	var lon = ad.lon;

	var primarySeparator = VE_SerializationConstants.primarySeparator[state.GetCurrentUrlApiVersion()];
	var secondarySeparator = VE_SerializationConstants.secondarySeparator[state.GetCurrentUrlApiVersion()];
	
	var serial = "";
	serial += VE_Annotations.PinType;
	serial += ".";

	//Encode the LatLong coordinate into a hash for the serialization
	var encoder = new VELatLongEncoding();
	latLongEncoding = encoder.Encode(lat.toFixed(6), lon.toFixed(6));
	serial += latLongEncoding;

	// escape the delimiter and replace any any primary separators with spaces.
	// It is unfortunate that we are stripping out primary separators, but it must be done because browsers
	// will convert primary separators into url encoded version and if we don't decode them, 
	// deserializing more important things (such center point) will not work.
	var removePrimarySeparatorRegExp = new RegExp("(" + primarySeparator + ")|(" + IOSec.EncodeUrl(primarySeparator) + ")", "g");
	serial += secondarySeparator;
	//
	// truncate the annotation: 100 chars for title, 200 for notes, 500 for info and photo url
	var trunc = IOSec.EncodeUrlDelims ( secondarySeparator, ad.title.replace(removePrimarySeparatorRegExp, " ") );
	serial += trunc.substring (0, 100);
	serial += secondarySeparator;
	trunc = IOSec.EncodeUrlDelims ( secondarySeparator, ad.notes.replace(removePrimarySeparatorRegExp, " ") );
	serial += trunc.substring (0, 200);
	serial += secondarySeparator;
	trunc = IOSec.EncodeUrlDelims ( secondarySeparator, ad.infoUrl.replace(removePrimarySeparatorRegExp, " ") );
	serial += trunc.substring (0, 500);
	serial += secondarySeparator;
	trunc = IOSec.EncodeUrlDelims ( secondarySeparator, ad.photoUrl.replace(removePrimarySeparatorRegExp, " ") );
	serial += trunc.substring (0, 500);
	
	return IOSec.EncodeUrl(serial);
}

//
// deserialize a list (array) of annotations
// returns an array containing the non-annotations pushpins
VE_AnnotationData.DeserializeAnnotationList = function ( arySerializedPins )
{
	// length decreases with splicing (ii used for elements that are not annotations and should not contribute to length)
	for ( var ii=0; 0 < arySerializedPins.length - ii; )
	{
		// see if first unknown element is an annotation
		if ( VE_AnnotationData.DeserializeAnnotation ( arySerializedPins[ ii ] ) )
		{
			// cut the first annotation element out of the list
			arySerializedPins.splice ( ii + 0, 1 );
		}
		else
		{
			// not an annotation, subtract it from arySerializedPins.length
			ii++;
		}
	}
	
	if ( 0 < arySerializedPins.length )
	{
		return arySerializedPins;
	}
	else
	{
		return null;
	}
}

//
// deserialize an annotation
// parameter is a serialized pushpin representation
// returns false if the passed-in string is not an annotation
VE_AnnotationData.DeserializeAnnotation = function ( strSerializedAnno )
{
	if ( !VE_Annotations.SerializedRepIsAnno ( strSerializedAnno ) )
	{
		return false;
	}

	var secondarySeparator = VE_SerializationConstants.secondarySeparator[state._config.serializationVersion];
	
	// actual serialized annotation starts after the '.', ends at the end of the string
	var serialIdLen = VE_Annotations.PinType.length + 1;
	strSerializedAnno = strSerializedAnno.substring ( serialIdLen, strSerializedAnno.length );
	
	var aryAnnoParts = strSerializedAnno.split ( secondarySeparator );
	var ii = 0;

	var lat = "undefined";
	var lon = "undefined";

	var isValidEncoding = false;
	if (aryAnnoParts.length > ii)
	{
		try
		{
			//try to decode the latlong from an encoded string first
			var encoder = new VELatLongEncoding();
			var decodedLatLong = encoder.Decode(aryAnnoParts[ii]);
			//If the decoder throws an exception, the following lines will not get executed
			lat=decodedLatLong[0];
			lon=decodedLatLong[1];
			isValidEncoding = true;
			++ii;
		}
		catch(ex)
		{
			isValidEncoding = false;
		}
	}

	//If the decoding failed, it may be an unencoded lat/lon. Try to read unencoded lat/lon coordinates from the serialization.
	if (!isValidEncoding)
	{
		if (aryAnnoParts.length > ii)
		{
			lat = parseFloat(aryAnnoParts[ii++]);
		}
		
		if (aryAnnoParts.length > ii)
		{
			lon = parseFloat(aryAnnoParts[ii++]);
		}
	} 

	// if either of these aren't numbers, discard the annotation
	if (isNaN (lat) || isNaN (lon))
	{
		return false;
	}
	//
	// title, notes may contain escaped '_', so unescape them
	// if a null title or notes are given, use the defaults
	var title = (aryAnnoParts.length > ii) ? unescape ( aryAnnoParts[ii++] ) : VE_Annotations.DefaultTitle;
	var notes = (aryAnnoParts.length > ii) ? unescape ( aryAnnoParts[ii++] ) : VE_Annotations.DefaultNotes;
	var infoUrl = (aryAnnoParts.length > ii) ? unescape ( aryAnnoParts[ii++] ) : VE_Annotations.DefaultInfoUrl;
	var photoUrl = (aryAnnoParts.length > ii) ? unescape ( aryAnnoParts[ii++] ) : VE_Annotations.DefaultPhotoUrl;
	//
	// create a location to use to generate a pushpin id
	// TODO: move this code into the scratchpad
	var id = VE_Scratchpad.locationCounter;
	var location = new VE_SearchResult(id, title, notes, "", "", "aN", lat, lon, infoUrl,photoUrl);
	var pinId = VE_Scratchpad.GeneratePinId (location);
	delete location;
	
	VE_AnnotationData.StoreAnnotation ( pinId, lat, lon, title, notes, infoUrl, photoUrl );
	VE_Scratchpad.AddLocation ( title, lat, lon, notes, "aN", infoUrl, photoUrl );	
	
	return true;
}
