//------------------------------------------------------------------------------
// Annotation/AnnotationPopup.js - Creation of annotation popup
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, sstchur, shankak
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//
// keeps track of the Popup's open state
VE_Annotations.PopupOpen = false;
//
// holds the annotations Popup
VE_Annotations.Popup = null;

//
// opens the annotation Popup
// given an event (e), it will try to fill in pin/lat/lon
// given a pin, it will fill in lat/lon
// default color for popup is black
// default title is annotation title
VE_Annotations.OpenPopup = function (e, pin, latitude, longitude, color, title)
{
	if (VE_ContextMenu.MenuOpen) 
	{
		VE_ContextMenu.RemoveContextPin();
		VE_ContextMenu.CloseMenu();
	}
	
	//Fix: Bug 3909
	//Do not close the Annotations Panel because it is annoying if the 
	//user clicks on Add Pushpin and then hovers over a pin on the way to the panel
	//VE_ContextMenu.RemoveContextPin ();
	//VE_Annotations.ClosePanel ();
	
	// close the driving directions popup if it's open
	VE_Directions.ClosePopup();
 	// close the search popup if it's open
	VE_SearchManager.CloseSearchPopup ();
	// close the annotations popup if it's open
	VE_Annotations.ClosePopup ();
	
	if (!pin)
	{
		var t = GetTarget (e);		
		if (!t)
			return false;
		
		pin = t;
	}
	
	if (!latitude)
		latitude = pin.vePushpin.GetLatitude();

	if (!longitude)	
		longitude = pin.vePushpin.GetLongitude();

	var content = VE_Annotations.CreatePopupContent(pin);
	ero.setContent(content);	
	ero.dockToElement(pin);
	return;
}

//
// close the Popup
VE_Annotations.ClosePopup = function (immediately)
{
	ero.hide(immediately);
}

//
// builds the content of the Popup from a pin
VE_Annotations.CreatePopupContent = function (pin)
{
	var fieldWidth = VE_Annotations.PopupWidth - 15; // 15 for padding
	var textAreaHeight = VE_Annotations.PopupHeight - 90 - 34; // 90 for other elements, 34 for title bar/close box
	var pinAnnotationData = VE_AnnotationData.GetAnnotation ( pin.id );
  
	if(pinAnnotationData == null)
		return;

	var title = IOSec.EncodeHtml(pinAnnotationData.title);
		
	var notes = IOSec.EncodeHtml(pinAnnotationData.notes);

	if(notes != null && notes != "undefined")
		notes = notes.replace(/^\s+/g, "").replace(/\s+$/g, ""); // trim it.
	else 
		notes = "";
		
	//shankak : new fields for schema change
	var infoUrl = IOSec.EncodeUrl(pinAnnotationData.infoUrl);
	var photoUrl = IOSec.EncodeUrl(pinAnnotationData.photoUrl);
	  
	var text =	'<p><div class = "title">%IMG%' + title + '</div>' + notes + '<br /></p>'; 
  

	ero.clearActions();
		
	var editLink = VE_Annotations._CreateAnnotationPopupLink ('edit', pin);
	var moveLink = VE_Annotations._CreateAnnotationPopupLink('move', pin);  
	var delLink = VE_Annotations._CreateAnnotationPopupLink ('delete', pin);
	var driveFromLink = VE_Annotations._CreateAnnotationPopupLink ('driveFrom', pin);
	var driveToLink = VE_Annotations._CreateAnnotationPopupLink ('driveTo', pin);
	var emailLink = VE_Annotations._CreateAnnotationPopupLink ('email', pin);

	var editMoveDelLink =
	'<a href = "' + editLink + '">' + L_AnnotationPopupEdit_Text + '</a>' +
	', ' +
	'<a href = "' + moveLink + '">' + L_AnnotationPopupMove_Text + '</a>' +
	', ' +
	L_Or_Text + ' ' +
	'<a href = "' + delLink + '">' + L_AnnotationPopupDelete_Text + '</a>';
  
	ero.addAction(editMoveDelLink);

	//Set the LatLong for the current Search Manager instance
	//Note: this LatLong storage mechanism is required for pictometry contractual purposes
	//We legally cannot show lat/long coordinates on the status bar in birdseye mode.
	VE_SearchManager.SetLatLong(pin.vePushpin.GetLatitude(), pin.vePushpin.GetLongitude());
	
	ero.addAction(VE_SearchManager.PopupZoomContent());
  
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) )    
	{
			ero.addAction('<a href = "' + driveFromLink + '">'+ L_AnnotationPopupDriveFrom_Text +'</a>' + ' ' + L_Or_Text + ' <a href = "' + driveToLink + '">' + L_AnnotationPopupDriveTo_Text + '</a>');
	}
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.SendEmailOnPushpinItem ) )    
	{
			ero.addAction('<a href = "' + emailLink + '" class = "VE_Panel_sublink" onclick = "VE_Analytics.Log(\'VE | User PushPin Pane\',\'Email\')">' + L_AnnotationPopupEmailAFriend_Text + '</a>');	
	}  
	  
	var windowW = 775;
	  
	if(infoUrl && infoUrl.length > 0 && infoUrl != 'undefined')
	{
		var iurl = ""
		try
		{
			iurl = VE_CollectionsManager.GetValidatedUrl(infoUrl);
		}
		catch(errorx)
		{
			iurl = "";
		}
		
		ero.addAction('<a href = "javascript:OpenNewWindow(\'' + IOSec.EncodeUrl(iurl) + '\', \'Information'  + '\', \'all\', ' + windowW + ', 500, 200, 200); VE_Analytics.Log(\'VE | Push Pin Popup\', \'More Information\');" class = "VE_Panel_sublink" oncontextmenu = "return false;">' + L_AnnotationPopupMoreInfo_Text + '</a>');
	}
  
  
	if(photoUrl && photoUrl.length > 1 && photoUrl != 'undefined')
	{		
		var purl = photoUrl;	
		try
		{
			purl = VE_CollectionsManager.GetValidatedUrl(photoUrl);
		}
		catch(errorx)
		{
			purl = "";
		}
	
		var purl1 = IOSec.EncodeUrl(purl);															
		//start html content 							
		var htmlContent = "<a href=\"javascript:OpenNewWindow('%P1%', '_blank' , 'menubar=0,resizable=1,scrollbars=0,status=0,titlebar=0,toolbar=0', 800, 600, 200, 200);\">";
		htmlContent = htmlContent.replace(/%P1%/g,purl1);		
		//Create image content and encode it
		var imgContent = "<IMG border=0 id=eroImg_" + pin.id + " style=\"position: relative; display: block; padding-right: 10px; padding-left: 0px; padding-bottom: 5px; padding-top: 3px; width: 80px; height: 80px; float: left;\" src=\"%P2%\" />";
		imgContent = imgContent.replace(/%P2%/g,IOSec.EncodeHtml(purl));
		//Append the img content to the html content
		htmlContent = htmlContent + imgContent + "</a>";		
		text = text.replace(/%IMG%/g,htmlContent);	
		
	}

	text = text.replace(/%IMG%/g,' ');
  
	return text;
}

