//------------------------------------------------------------------------------
// ContextMenu/ContextPanel.js - Presentation generation for Context Panel
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, sstchur, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//
// menu open state
VE_ContextMenu.MenuOpen = false;
//
// holds the context menu panel
VE_ContextMenu.MenuPanel = null;
//
// the context menu width
VE_ContextMenu.MenuWidth = 165;
//
// the context menu base height (with minimum # of lines)
VE_ContextMenu.MenuBaseHeight = 115;
//
// the height of each extra line of text
VE_ContextMenu.LineHeight = 14;
//
// the context menu height (with actual # of lines)
VE_ContextMenu.MenuHeight = 115;

// holds the context menu LatLong coordinate
// Note: this LatLong storage mechanism is required for pictometry contractual purposes
// We legally cannot show lat/long coordinates on the status bar in birdseye mode.
VE_ContextMenu.LatLong = null;

//
// create a context menu object
function VE_ContextMenu()
{
}

//Sets the Latitude/Longitude coordinate for the Context menu
VE_ContextMenu.SetLatLong = function(latitude, longitude)
{
	VE_ContextMenu.LatLong = new Msn.VE.LatLong(latitude, longitude);
}

//
// creates a new context menu panel if it doesn't exist and displays it
// at a given lon/lat and in a given color.
VE_ContextMenu.OpenContextMenu = function (latitude, longitude, bOblique, color)
{
	// close the search popup if it's open
	VE_SearchManager.CloseSearchPopup ();
	// close the context menu if it's open
	VE_ContextMenu.CloseMenu ();
	// close the annotations popup if it's open
	VE_Annotations.ClosePopup ();

	//Set the context menu LatLong
	// Note: this LatLong storage mechanism is required for pictometry contractual purposes
	// We legally cannot show lat/long coordinates on the status bar in birdseye mode.
	VE_ContextMenu.SetLatLong(latitude, longitude);
	
	// If the menu has not been created, create it
	if(!VE_ContextMenu.MenuPanel)
	{
		VE_ContextMenu.MenuPanel = new VE_Panel(
			"contextMenu",					// id
			5,								// x
			80,								// y
			VE_ContextMenu.MenuWidth,							// width
			VE_ContextMenu.MenuHeight,								// height
			color,							// color
			VE_SearchManager.popupZIndex,	// z index
			"",								// title
			"",								// body
			"");							// footer
			
		var m = VE_ContextMenu.MenuPanel;
		m.SetFooterSize(0);
		m.SetToolbarSize(0);
		m.DisableTitle ();
		//
		// close handler removes temp pushpin
		m.onCloseClick = VE_ContextMenu._ContextMenuCloseHandler;
	}
	
	var m = VE_ContextMenu.MenuPanel;
		
			
	//
	// this can alter the height of the context menu
	m.SetBody( VE_ContextMenu._ContextMenuContent(bOblique) );
	
	var cmw = VE_ContextMenu.MenuWidth;
	var cmh = VE_ContextMenu.MenuHeight;
	
	//
	// Adjust the effective width of the screen for the scratchpad
	// and driving directions panel
	var w = windowWidth;
	if(VE_Scratchpad.panel && VE_Scratchpad.panel.visible ||
	   VE_Directions.panel && VE_Directions.panel.visible)
	{
		w -= 200;
	}
	
	// Convert the latlong to pixel
	var pixel = map.LatLongToPixel(VE_ContextMenu.LatLong);
	
	// Make sure the popup isn't too far to the right
	// If it is, show it on the left side of the pushpin
	var x = pixel.x + 5;
	if(x > w-cmw)
	{
		x -= cmw + 10;
	}
	
	m.SetColor(color);
	m.Show();
	
	// Make sure the popup isn't too far down
	// If it is, show it above the pushpin
	var y = pixel.y - 11;
	if(y > windowHeight - cmh)
	{
		y -= cmh - 22;
	}
	
	m.height = cmh;
	m.SetPosition(x, y, cmw, cmh);
	m.Resize();
	
	/* sstchur:  added a class for styling the right-click, context menu */
	m.el.className = 'VE_contextMenu';
	
	VE_ContextMenu.MenuOpen = true;	
}


VE_ContextMenu.CloseMenu = function ()
{
  var m = VE_ContextMenu.MenuPanel;
  if (m)
  {
    m.Hide ();
  }
  
  VE_ContextMenu.MenuOpen = false;
}

//
// this method should just generate the contextmenu content.
// any handling should be done by generating a link with:
// VE_ContextMenu._ContextMenuHandlerLink (typeOfAction)
VE_ContextMenu._ContextMenuContent = function (bOblique)
{
var cmh = VE_ContextMenu.MenuBaseHeight;
  // begin table
	var text = '<table cellpadding="0" cellspacing="0" width="100%">';
	
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.Collections.AddPushpin ) )    	
	{
		var annotatePinLink = VE_ContextMenu._ContextMenuHandlerLink("createAnnotation");
		text += '<tr><td><a class="VE_Context_Link" href="' + annotatePinLink + '">'+ L_ContextPanelAddPushpin_Text +'</a></td></tr>';
	}
	else
	{
		//re-calculate the menuHeight
		cmh -= VE_ContextMenu.LineHeight*2; 
	}
	if ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) )    	
	{	
		var driveFromLink = VE_ContextMenu._ContextMenuHandlerLink("driveFrom");
		text += '<tr><td><a class="VE_Context_Link" href="' + driveFromLink + '">'+ L_DriveFrom_Text +'</a></td></tr>';
		
		var driveToLink = VE_ContextMenu._ContextMenuHandlerLink("driveTo");
		text += '<tr><td><a class="VE_Context_Link" href="' + driveToLink + '">'+ L_DriveTo_Text +'</a></td></tr>';
		
	}
	else
	{ 
		//re-calculate the menuHeight
		cmh -= VE_ContextMenu.LineHeight*3; 
	}
		
	// hr	
	if ( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.Collections.AddPushpin ) ||  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MapAction.DrivingDirections ) ) 
		&& (  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.CenterMapHere ) 
			||  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.ZoomToRegion ) 
			||  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.ZoomToCity ) 
			||  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.ZoomToStreet ) 
		   ) 
		)
		{
			text += '<tr><td><hr /></td></tr>';
		}
	else
	{ 
		//re-calculate the menuHeight
		cmh -= VE_ContextMenu.LineHeight*2; 
	}		
	
	if ( map.GetMapStyle() != 'o' )
	{
		//
		// add a line of text to the menu
		var contextMenuZoomContent= VE_ContextMenu._ContextMenuZoomContent();
		text += contextMenuZoomContent;
		
		//re-calculate the menuHeight
		cmh -= VE_ContextMenu.LineHeight * (3 - contextMenuZoomContent.split("<tr>").length);
	}
	else
	{
		cmh -= VE_ContextMenu.LineHeight * 3;
	}
	
	if (  Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.CenterMapHere ) )
	{
		var centerLink = VE_ContextMenu._ContextMenuHandlerLink("center");
		text += '<tr><td><a class="VE_Context_Link" href="' + centerLink + ';VE_Analytics.Log(\'VE | Right Click Options\',\'CenterMap\',\'RightClick\');">'+ L_ContextPanelCenterMapHere_Text +'</a></td></tr>';
	}
	else
	{ 
		cmh -= VE_ContextMenu.LineHeight*2; 
	}	
	// end table
	text += '</table>';
	
	//
	// now we know the height of the menu
	VE_ContextMenu.MenuHeight = cmh;
	return text;
}
