//------------------------------------------------------------------------------
// ContextMenu/EventHandlers.js - Handlers for context menu and context menu
//                                delegates
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: BayardB, a-pratin, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// 
var startScene = null ; // holds reference to the scene/tile that has the Start pushpin
var endScene = null ; // holds reference to the scene/tile that has the End pushpin


// place a pushpin and open the context menu
VE_ContextMenu.OpenContextMenuAtLocation = function (e)
{
	// Ignore this context-menu request if the user is in "move pushpin" mode
	if (VE_Annotations.PinBeingMoved)
		return;

	var lat = e.view.latlong.latitude;
	var lon = e.view.latlong.longitude;
	var bOblique = e.oblique != null;
	VE_ContextMenu.PlaceContextPin (lat, lon);
	VE_ContextMenu.OpenContextMenu (lat, lon, bOblique);
}
// place the context pin
VE_ContextMenu.PlaceContextPin = function (latitude, longitude, pinclass)
{
	if ( !pinclass )
	{
		pinclass = "context";
	}

	// remove the temp pin if it exists
	VE_ContextMenu.RemoveContextPin ( );
	
	// add a new temp pin
	var pin = map.AddPushpin(
		'context_pin',              // pushpinID
		latitude,                                  // latitude
		longitude,                                 // longitude
		7,                                               // width
		7,                                               // height
		"VE_Pushpin VE_Pushpin_" + pinclass,                 // className
		'',                                    // innerHTML
		VE_Scratchpad.pinZIndex);					      // zIndex, should be highest, except for when things are hovered over.
}

// remove the context pin
VE_ContextMenu.RemoveContextPin = function ()
{
	map.RemovePushpin('context_pin');
}

VE_ContextMenu._ContextMenuHandlerLink = function (action, zoom)
{
	var link = '';
	if ("center" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'center\')';
	}
	else if ("zoom" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'zoom\',' + zoom + ')';
	}
	else if ("driveFrom" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'driveFrom\')';
	}
	else if ("driveTo" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'driveTo\')';
	}
	else if ("createAnnotation" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'createAnnotation\')';
	}
	else if ("showEagleEye" == action)
	{
		link += 'javascript:VE_ContextMenu._ContextMenuActionHandler(\'showEagleEye\')';
	}	
	
	return link;
}

VE_ContextMenu._ContextMenuCloseHandler = function ()
{
	VE_ContextMenu._ContextMenuActionHandler ('close');
}

VE_ContextMenu._ContextMenuActionHandler = function (action, zoom)
{
// Note: this LatLong storage mechanism is required for pictometry contractual purposes
// We legally cannot show lat/long coordinates on the status bar in birdseye mode.
	var latitude = VE_ContextMenu.LatLong.latitude;
	var longitude = VE_ContextMenu.LatLong.longitude;
	
	VE_ContextMenu.CloseMenu ();
	if ("center" == action)
	{
		map.SetCenter (latitude, longitude);
	}
	else if ("zoom" == action)
	{
		map.SetCenterAndZoom(latitude, longitude, zoom);
		VE_Analytics.Log("VE | Right Click Options","ZoomTo","RightClick");
	}
	else if ("close" == action)
	{
  	VE_ContextMenu.RemoveContextPin ();
	}
	else if ("driveFrom" == action)
	{
		// isOrigin == true
		VE_ContextMenu.RemoveContextPin ();
		VE_Directions.SetStart (L_Origin_Text, latitude, longitude, true);
		
		// if the map style is in ObliqueMode , record the StartScene object
		if ( map.GetMapStyle() == 'o' )
			startScene = map.GetObliqueScene();
		
		VE_Analytics.Log("VE | Right Click Options","Get Directions","RightClick");
	}
	else if ("driveTo" == action)
	{
		// isOrigin == false
		VE_ContextMenu.RemoveContextPin ();
		VE_Directions.SetEnd (L_Destination_Text, latitude, longitude, true);
		
		// if the map style is in ObliqueMode , record the StartScene object
		if ( map.GetMapStyle() == 'o' )
			endScene = map.GetObliqueScene();
		
		VE_Analytics.Log("VE | Right Click Options","Get Directions","RightClick");
	}
	else if ("createAnnotation" == action)
	{
		VE_Annotations.AnnotateLocation ();
		VE_Analytics.Log("VE | Right Click Options","CreatePushPin","RightClick");
	}
	else if ("showEagleEye" == action)
	{
		//
		// animated pan breaks SetMapStyle ()
		var panAnimated = map.IsAnimationEnabled ();
		if ( panAnimated )
		{
			map.SetAnimationEnabled (false);
		}
		
		map.SetCenter (latitude, longitude);
		map.SetMapStyle('o');
		
		if ( panAnimated )
		{
			map.SetAnimationEnabled (true);
		}
		VE_Analytics.Log("VE | Right Click Options","EagleEye","RightClick");
	}
}
