//------------------------------------------------------------------------------
// Directions/Directions.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, EShinn, sstchur, cthota, shankak, a-pratin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------
// Class that handles all directions related code.
function VE_Directions()
{
	VE_Directions.maxInputLength = 100;
}

//Enums for Direction Print option
VE_Directions.PrintMapsAndDirections = 1;
VE_Directions.PrintDirectionsOnly = 2;
VE_Directions.PrintMapsOnly = 3;

// Performs necessary clean-up work before page exit.
VE_Directions.Destroy = function()
{
	// Clear out directions
	VE_Directions.Clear();
	
	// Set all non-simple references to null.
	VE_Directions.panel = VE_Directions.displayedRoute = VE_Directions.startAmbiguousSelect = VE_Directions.endAmbiguousSelect = VE_Directions.xmlHttp = null;
}

VE_Directions._initialize = function()
{
	// The driving directions panel
	VE_Directions.panel = null;
	
	VE_Directions.startAmbiguousSelect = new VE_Select("startDisambiguate","","","disambiguateListOver");
	VE_Directions.endAmbiguousSelect = new VE_Select("endDisambiguate","","","disambiguateListOver");	
}

// Directions push pin z-index.
VE_Directions.pinZIndex = 19;

// This is the default contents of the directions panel

VE_Directions.initialBody = 
	'<form id = "dd" name = "dd" target = "hiddenframe" action = "about:blank" method = "get" onsubmit = "VE_Directions.GetDirections();">' +
		'<fieldset class = "firstChild">' +
			'<legend></legend>' +
			'<label>' + L_Start_Text + '</label>' + '<span id = "startNotFoundMessage" class="VE_DrivingDirections_NotFoundMessage" ></span>' +
			'<textarea id = "start" name = "start" onpaste=\"return VE_Directions._ValidateInputOnPaste(this);\" onkeypress=\"return VE_Directions._ValidateInputOnKeyPress(this,event);\" onchange = "VE_Directions._ClearSetStartValues();" tabindex = "10"></textarea>' +
			'<div id = "startMessage"></div>' +
			'<div id = "startAmbiguous" tabindex = "1700"></div>' +
			'<div id = "swapStartAndEndPoints" title = "' + L_DirectionsSwapStartAndEndPoints_Text + '" onclick = "VE_Directions.ReverseStartAndEnd()"></div>' +
		'</fieldset>' +
	
		'<fieldset class = "secondChild">' +
			'<legend></legend>' +
			'<label>' + L_End_Text + '</label>' + '<span id = "endNotFoundMessage" class="VE_DrivingDirections_NotFoundMessage" ></span>' +
			'<textarea id = "end" name = "end" onpaste=\"return VE_Directions._ValidateInputOnPaste(this);\"  onkeypress=\"return VE_Directions._ValidateInputOnKeyPress(this,event);\" onchange = "VE_Directions._ClearSetEndValues();" tabindex = "11"></textarea>' +
			'<div id = "endMessage" ></div>' +	
			'<div id = "endAmbiguous" tabindex ="1701"></div>' +					
		'</fieldset>' +
		
		'<fieldset>' +
			'<legend></legend>' +
			'<input id = "routeType_quickest" name = "routeType" type = "radio" checked = "checked" tabindex = "12" /><label for = "routeType_quickest">' + L_Quickest_Text + '</label>' +
			'<input id = "routeType_shortest" name = "routeType" type = "radio" tabindex = "12" /><label for = "routeType_shortest">' + L_Shortest_Text + '</label>' +						
		'</fieldset>' +
		
		'<fieldset>' +			
			'<ul class = "wll_GetDirectionsButton"><li id = "GetDirectionsButton" onclick = "VE_Directions.GetDirections()" onkeypress="if ( event.keyCode == 13 ) VE_Directions.GetDirections();" tabindex = "13" >' +  L_DirectionsGetDirections_Text + '</li></ul>' +
		'</fieldset>'	+
	'</form>' +	

	'<div id = "directionsResults" class = "VE_DrivingDirections_Results"></div>' +
	'<div id = "VE_DrivingDirections_Reverse"></div>';

// Information about the route (to be) displayed
VE_Directions.startAddress = null;
VE_Directions.startLat = null;
VE_Directions.startLon = null;
VE_Directions.startPin = null;
VE_Directions.endAddress = null;
VE_Directions.endLat = null;
VE_Directions.endLon = null;
VE_Directions.endPin = null;
VE_Directions.CurrentUnit = 'mi';

// Directions will display temporary route start/end pushpins if true.
// This is used in scenarios where the user has set start & end points using context-menu or via URL & a successful route calculation was done.
VE_Directions.showTemporaryPins = true; 

// Create the scratchpad panel
VE_Directions.CreatePanel=function()
{		
	// sstchur:  x and y are 0; height and width are 'auto', and this panel is a child of the taskArea
	VE_Directions._initialize();
	var p = new VE_Panel(
		"directions",                		// id
		0,							        // x
		0,									// y
		'auto',								// width
		'auto',								// height 
		'directions',						// color (sstchur:  in this case it's actualy now a 'type' rather than a color)
		VE_Scratchpad.panelZIndex,			// z index
		L_Directions_Text,					// title 
		VE_Directions.initialBody,			// body
		'',									// footer 
		"Directions",						// mouseovertitle
		true,								// opaque
		taskArea_data);
		
	taskArea.addItem(p, true);
	
	
	// Make IE obey the pseudo class :hover for the Direction Panel's TitleBar
	pseudoHover(p.title);
	
	//
	// these are transparent text entry elements
	var oStart = document.getElementById ('start');
	var oEnd = document.getElementById ('end');
	if ( oStart.attachEvent )
	{
		oStart.attachEvent ( 'onkeydown', VE_Directions._FixScroll );
		oEnd.attachEvent ( 'onkeydown', VE_Directions._FixScroll );
	}
	oStart = oEnd = null;
	
	//set toolbar
	p.SetToolbar(VE_Directions._GetToolbar());
	
	//set styles on Directions toolbar menu items
	var Directions_menu = document.getElementById('directions_tb');
	pseudoHoverForChildren(Directions_menu);
	
	//shankak : Footer has to be visible to enable display of Clear and Reverse Directions buttons.
	//and hence > 0
	p.SetFooterSize(1);
	p.Hide();
	
	p.onTitleClick = function(e)
	{
		taskArea.setCurrent(VE_Directions.panel);
	}
		
	p.onCloseClick = function(e)
	{	
		// TODO: look into cleaning up this function
		VE_SearchManager.CloseSearchPopup();		
		VE_Directions.Destroy();
		
		// Now we'll physically remove the elements from the DOM
		var parent = this.el.parentElement;
		if (parent)
			parent.removeChild(this.el);		
		
		// Housekeeping for the TaskArea as well
		taskArea.removeItem(this);		
	}
	
	p.onMaximize = function(e)
	{
		resetWhatBox();		
	}
	
	VE_Directions.panel = p;
}

//
// TODO: make this a global utility method
VE_Directions._FixScroll=function()
{
	//
	// assigning a 'new' filter to an element (even if it is the same filter)
	// causes the element to update its state. this is a workaround for text
	// input elements which don't update correctly when filtered in IE.
	var css = event.srcElement.style;
	css.filter = css.filter;
}

// Scratchpad toolbar markup
VE_Directions._GetToolbar=function()
{
	var markup =
	'<ul class = "wll_dropDownMenu">' +
		'<li onclick = "VE_Directions.Clear()" oncontextmenu = "return false">' + L_Clear_Text + '</li>' +
		( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.ShareDrivingDirections.SendEmail ) )? '<li onclick = "VE_Directions.Email()" oncontextmenu = "return false">' + L_Email_Text + '</li>' : '' ) +
		( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.PrintDrivingDirections.DisplayNone ) )? '' : 
		'<li oncontextmenu = "return false" onmouseover="UpdatePrintMenuItems()">' + L_DirectionsPrint_Text +
			'<ul>' +
				( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.PrintDrivingDirections.MapsAndDirectionsOnly ) )? '<li id="DDMapsPlus" onclick="VE_Analytics.Log(\'VE | Directions Pane\', \'PrintTotal\'); VE_Directions.InvokePrint(' + VE_Directions.PrintMapsAndDirections + '); VE_Analytics.Log(\'VE | Directions Pane\',\'Print MapAndText on directions pane\');" oncontextmenu = "return false" >' + L_DirectionsPrintMapsAndDirections_Text  + '</li>' : '' ) +
				( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.PrintDrivingDirections.MapsOnly ) )? '<li id="DDMapsOnly" onclick="VE_Analytics.Log(\'VE | Directions Pane\', \'PrintTotal\'); VE_Directions.InvokePrint(' + VE_Directions.PrintMapsOnly + '); VE_Analytics.Log(\'VE | Directions Pane\',\'Print Map on directions pane\');" oncontextmenu = "return false" >' + L_DirectionsPrintMapOnly_Text  + '</li>' : '' ) +
				( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.PrintDrivingDirections.DirectionsOnly ) )? '<li id="DDTextOnly" onclick="VE_Analytics.Log(\'VE | Directions Pane\', \'PrintTotal\'); VE_Directions.InvokePrint(' + VE_Directions.PrintDirectionsOnly + '); VE_Analytics.Log(\'VE | Directions Pane\',\'Print Text on directions pane\');" oncontextmenu = "return false" >' + L_DirectionsPrintTextDirections_Text + '</li>' : '' ) +
				( ( Msn.VE.Marketization.IsEnabled( Msn.VE.Features.MenuOption.PrintDrivingDirections.Customized ) )? '<li onclick="VE_Analytics.Log(\'VE | Directions Pane\', \'PrintTotal\'); VE_Directions.InvokeCustomizePrint(); VE_Analytics.Log(\'VE | Directions Pane\',\'Print Custom on direction pane\');" oncontextmenu = "return false" >' + L_DirectionsCustomizePrinting_Text + '</li>' : '' ) +	
			'</ul>'+		
		 '</li>' 
		 ) +		 
	'</ul>';
	
	return markup;
}

// Show driving directions
VE_Directions.Show=function()
{
	// sstchur:  If the taskArea isn't visible, make it visible (required in order to show the introPanel
	toggleTaskArea(true);
			
	if (!VE_Directions.panel)
	{
		VE_Directions.CreatePanel();
		SetupDirections();
	}
	
	VE_Directions.panel.Show();	
	taskArea.setCurrent(VE_Directions.panel);
	document.getElementById('start').focus();	
}

// Removes all temporary pins
VE_Directions._RemoveTemporaryPins=function()
{
	VE_Directions._RemoveTemporaryStartPin();
	VE_Directions._RemoveTemporaryEndPin();
}

// REmoves the temporary start pin if it is defined.
VE_Directions._RemoveTemporaryStartPin=function()
{
	if (VE_Directions.startPin != null)
	{
		map.RemovePushpin(VE_Directions.startPin.id);
		VE_Directions.startPin = null;
	}
}

// Removes the temporary end pin if it is defined.
VE_Directions._RemoveTemporaryEndPin=function()
{
	if (VE_Directions.endPin != null)
	{
		map.RemovePushpin(VE_Directions.endPin.id);
		VE_Directions.endPin = null;
	}
}

// Will display temporary pushpins if VE_Directions.showTemporaryPins = true;
VE_Directions.DisplayTemporaryPins=function()
{
	if (VE_Directions.showTemporaryPins)
	{
		VE_Directions._CreateTemporaryPins();
	}
	else
	{
		VE_Directions._RemoveTemporaryPins();
	}
}

// Create temporary pins for start & end if available.
VE_Directions._CreateTemporaryPins=function()
{
	VE_Directions._CreateTemporaryStartPin();
	VE_Directions._CreateTemporaryEndPin();
}

// Create a start temporary pin if we have a latitude & longitude for the start location
VE_Directions._CreateTemporaryStartPin=function()
{
	// Remove the pin if it already exists.
	VE_Directions._RemoveTemporaryStartPin();

	// only create if we have a latitude & longitude
	if (VE_Directions.startLat != null && VE_Directions.startLon != null)
	{
		VE_Directions.startPin = VE_Directions.PlaceTemporaryStartOrEndPushpin("veDD_tempStart", "S", VE_Directions.startLat, VE_Directions.startLon, "VE_Pushpin VE_Pushpin_dd_start");
	}
}

// Create a start temporary pin if we have a latitude & longitude for the end location
VE_Directions._CreateTemporaryEndPin=function()
{
	// Remove the pin if it already exists.
	VE_Directions._RemoveTemporaryEndPin();
	
	// only create if we have a latitude & longitude
	if (VE_Directions.endLat != null && VE_Directions.endLon != null)
	{
		VE_Directions.endPin = VE_Directions.PlaceTemporaryStartOrEndPushpin("veDD_tempEnd", "E", VE_Directions.endLat, VE_Directions.endLon, "VE_Pushpin VE_Pushpin_dd_end");
	}
}

// Sets the "display" style on the temporary pushpins.
VE_Directions._SetTemporaryPinDisplay=function(display)
{
	if (VE_Directions.startPin != null)
	{
		VE_Directions.startPin.style.display = display;
	}
	if (VE_Directions.endPin != null)
	{
		VE_Directions.endPin.style.display = display;
	}
}

//calculate the maxlength and control the paste functionality. Note : Onpaste event is not triggered in Fire fox browser(TODO)
VE_Directions._ValidateInputOnPaste=function(element)
{
	// called only by IE
	var inputLength = element.value.length;    	
	var selectedTextObj = element.document.selection.createRange();
	if ( selectedTextObj.text.length == 0 )
		selectedTextObj.text  = clipboardData.getData('text').substr(0, VE_Directions.maxInputLength - inputLength );		
	else
	{			
		selectedTextObj.text  = clipboardData.getData('text').substr(0, selectedTextObj.text.length + ( VE_Directions.maxInputLength - inputLength ));		
	}
	return false;
}

//calculate the maxlength and control the Keypress event. 
//Note: keys like "backspace","delete", "left",etc do not fire OnKeyPress in IE.It does in FireFox(hence explicit handling).
VE_Directions._ValidateInputOnKeyPress=function(element,event)
{	
	switch(event.keyCode) 
	{ 
		case 36: // Home			
		case 35: // End			
		case 37: // left 			
		case 38: // up 			
		case 39: // right 			
		case 40: // down 			
		case 8: // backspace 			
		case 46: // delete 
			return true; 				
	}
	
	if( SelectedTextLength(element) > 0 )
		return true;
	else
		return ( element.value.length > VE_Directions.maxInputLength - 1  )? false:true;		
}

// Resets the driving directions panel
VE_Directions.Clear=function()
{
	// sstchur:  Only do this if the Directions Panel is visible
	if (!VE_Directions.panel)
		return;
		
	VE_Directions.CancelGetDirections();
	VE_Directions.SetStart("", null, null);
	VE_Directions.SetEnd("", null, null);
	
	// reset the Directions Result section 
	ResetDirectionsResult();
	
	VE_Directions.ClosePopup();
	if(VE_Directions.displayedRoute)
	{
		var distanceObj = new Object();
		distanceObj.distance = VE_Directions.displayedRoute.distance;
		distanceObj.distanceUnit = VE_Directions.displayedRoute.distanceUnit;
		VE_Analytics.Log("VE | Directions Pane","Clear", "",distanceObj);
		VE_Directions.routeDisplayManager.Destroy();
		VE_Directions.routeDisplayManager = null;
	}
	else 
	{
		VE_Analytics.Log("VE | Directions Pane","Clear");
	}
	
}

// Resets the stored values for the route start
VE_Directions._ClearSetStartValues=function()
{
	VE_Directions.startAddress = null;
	VE_Directions.startLat = null;
	VE_Directions.startLon = null;
	VE_Directions._RemoveTemporaryStartPin();
	// Don't show the temporary pushpins if there's a route already present.
	if (VE_Directions.displayedRoute == null)
	{
		VE_Directions.showTemporaryPins = true;
		VE_Directions.DisplayTemporaryPins();
	}
}

// Resets the stored values for the route end
VE_Directions._ClearSetEndValues=function()
{
	VE_Directions.endAddress = null;
	VE_Directions.endLat = null;
	VE_Directions.endLon = null;
	VE_Directions._RemoveTemporaryEndPin();
	// Don't show the temporary pushpins if there's a route already present.
	if (VE_Directions.displayedRoute == null)
	{
		VE_Directions.showTemporaryPins = true;
		VE_Directions.DisplayTemporaryPins();
	}
}

// Sets the start point to a specific address and/or latitude/longitudee
VE_Directions.SetStart=function(address, latitude, longitude)
{
	VE_Directions.Show();
	document.getElementById("end").focus();
	
	// unescape url-encoded character
	address = unescape(address);
	document.dd.start.value = address;	
	VE_Directions.startAddress = address;
	VE_Directions.startLat = latitude;
	VE_Directions.startLon = longitude;

	VE_Directions._CreateTemporaryStartPin();
	
	// Make the temporary pins visible again.
	VE_Directions.showTemporaryPins = true;
	
	VE_Directions.Disambiguate("start", null);
	VE_Directions.ShowMessage("start", null);	
	VE_Directions.ShowNotFoundMessage("start",null);		
	
	VE_Directions.ClearRoute();	
	// display the temporary pins
	VE_Directions.DisplayTemporaryPins();	
	// reset the Directions Result section 
	var eleStartNotFoundMessage = document.getElementById("startNotFoundMessage");
	var eleEndNotFoundMessage = document.getElementById("endNotFoundMessage");
	if ( eleStartNotFoundMessage.innerHTML == "" && eleEndNotFoundMessage.innerHTML == "" )
	{
		ResetDirectionsResult();	
	}
	eleStartNotFoundMessage = eleEndNotFoundMessage = null;
}

// Sets the end point to a specific address and/or latitude/longitudee
VE_Directions.SetEnd=function(address, latitude, longitude)
{
	VE_Directions.Show();
	
	// unescape url-encoded character
	address = unescape(address);
	document.dd.end.value = address;
	VE_Directions.endAddress = address;
	VE_Directions.endLat = latitude;
	VE_Directions.endLon = longitude;

	VE_Directions._CreateTemporaryEndPin();
	
	// Make the temporary pins visible again.
	VE_Directions.showTemporaryPins = true;	
	
	VE_Directions.Disambiguate("end", null);
	VE_Directions.ShowMessage("end", null);
	VE_Directions.ShowNotFoundMessage("end",null);

	VE_Directions.ClearRoute();	
	// display the temporary pins
	VE_Directions.DisplayTemporaryPins();
	// reset the Directions Result section 
	var eleStartNotFoundMessage = document.getElementById("startNotFoundMessage");
	var eleEndNotFoundMessage = document.getElementById("endNotFoundMessage");	
	if ( eleStartNotFoundMessage.innerHTML == "" && eleEndNotFoundMessage.innerHTML == "" )
	{
		ResetDirectionsResult();
	}
	eleStartNotFoundMessage = eleEndNotFoundMessage = null;
	
}

function ResetDirectionsResult()
{
	var element = document.getElementById("directionsResults");	
	element.innerHTML = "";
	element.className = "VE_DrivingDirections_Results";		
	element = null;	
}

// Swaps the start & end points for the route.
VE_Directions.ReverseStartAndEnd=function()
{
	// You can't switch in the middle of a route calculation.
	if (VE_Directions.routing)
	{
		return;
	}
	
	// Swap the start & end.	
	var tempStartAddress = document.dd.start.value;	
	var tempStartLat = VE_Directions.startLat;
	var tempStartLon = VE_Directions.startLon;
	
	document.dd.start.value = document.dd.end.value;
	VE_Directions.startAddress = document.dd.start.value;
	VE_Directions.startLat = VE_Directions.endLat;
	VE_Directions.startLon = VE_Directions.endLon;

	document.dd.end.value = tempStartAddress;
	VE_Directions.endAddress = tempStartAddress;
	VE_Directions.endLat = tempStartLat;
	VE_Directions.endLon = tempStartLon;

	// Remove the old pins (if any)
	VE_Directions._RemoveTemporaryPins();
	
	// Display new ones if there isn't a route already showing.
	if (VE_Directions.displayedRoute == null)
	{
		VE_Directions.showTemporaryPins = true;
		VE_Directions.DisplayTemporaryPins();
	}
	
	// Remove the ambiguous select & any message
	VE_Directions.startAmbiguousSelect.Hide();
	VE_Directions.ShowMessage("start", null);
	VE_Directions.ShowNotFoundMessage("start",null);

	VE_Directions.endAmbiguousSelect.Hide();
	VE_Directions.ShowMessage("end", null);
	VE_Directions.ShowNotFoundMessage("end",null);
	
	// reset the Directions Result section
	var element = document.getElementById("directionsResults");
	if ( element.className == "VE_DrivingDirections_Results_Error" )
	{
		// reset the Directions Result section 
		ResetDirectionsResult();
	}	
	element = null;

}

// Creates a temporary pushpin for the start or end point.
VE_Directions.PlaceTemporaryStartOrEndPushpin=function(id, text, latitude, longitude, className)
{	
	// add a new temp pin
	var imageName = "<img src=\"i/pins/mapicon_end.gif\" id=\""+id+"\"/>";
	if(text == "S")
	{
		imageName = "<img src=\"i/pins/mapicon_start.gif\" id=\""+id+"\"/>";
	}
	
	var pin = map.AddPushpin(
		id+"_pinDiv",                                     // pushpinID
		latitude,                               // latitude
		longitude,                              // longitude
		29,                                     // width
		35,                                     // height
		"PushPin",                              // className
		imageName,                              // innerHTML
		VE_Directions.pinZIndex,				// zIndex, should be highest, except for when things are hovered over.
		Msn.VE.PushPinTypes.DirectionTemp);     // pinType      
	
	return pin;
}

// Display a message related to one of the end points.
VE_Directions.ShowMessage=function(field, message)
{
	var box = document.getElementById(field + "Message");
	if(message != null)
	{
		box.innerHTML = message;
		box.style.display = "block";
	}
	else
	{
		box.style.display = "none";
	}
}

VE_Directions.ShowNotFoundMessage = function (field, message )
{
	var box = document.getElementById( field + "NotFoundMessage" );	
	if ( message != null )
	{
		box.innerHTML = IOSec.EncodeHtml(message);		
	}
	else
	{	
		box.innerHTML = "";		
	}
}

// In the future, this will open up a mail client with a URL to recreate the driving directions
VE_Directions.Email=function()
{
	var route = VE_Directions.displayedRoute;
	
	// If we have no route, we can't very well email it, can we?
	if (route == null)
	{
		VE_Directions.HandleFailedGetDirections( L_DirectionsEmailNoDirection_Text );							
		return;
	}

	var primarySeparator = VE_SerializationConstants.primarySeparator[state.GetCurrentUrlApiVersion()];
	
	// Create the URL to get driving directions.
	// (Addresses must be doubly encoded because programs escape the body at least once)
	var ddUrl = GetUrlPrefix() + "?v=" + state.GetCurrentUrlApiVersion() + 
		"&rtp=" + route.GetSerializedForm().join(primarySeparator);

	// Tell the browser to go to a mailto: link with that information.
	document.location.href = 		
		"mailto:?subject=" + IOSec.EncodeUrl(L_DirectionsAppendToSubject_Text.replace(/%1/g,route.startAddress).replace(/%2/g,route.endAddress).replace(/%3/g,Msn.VE.Features.ProductName)) +		
		"&body=" + IOSec.EncodeUrl(ddUrl);
		
	var distanceObj = new Object();
	distanceObj.distance = VE_Directions.displayedRoute.distance;
	distanceObj.distanceUnit = VE_Directions.displayedRoute.distanceUnit;
	distanceObj.numberOfWaypoints = VE_Directions.displayedRoute.instructions.length;
	VE_Analytics.Log("VE | Directions Pane","Email", "",distanceObj);
}

// Displays printable version of driving directions if a route has been calculated
VE_Directions.InvokeCustomizePrint = function()
{
	var printWin = OpenNewWindow(printableDirectionsPage ,"printableMap","toolbar, scrollbars, resizable menubar",685,screen.availHeight-50,0,0);
}
VE_Directions.InvokePrint = function( option )
{	
	// display notification
	ShowMessage( L_DirectionsFormatingForPrint_Text.replace(/%1/g,Msn.VE.Features.ProductName) );
	
	//set printDirectionOption
	printDirectionOption= option;
	
	//set source file for the Iframe or if already loaded call the PrintAction method directly	
	document.getElementById( "hiddenIframeForPrintDirections" ).contentWindow.document.location.href = printUrl;	
}
