//------------------------------------------------------------------------------
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: a-bwrigh, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------
//JS/Graphics/graphics.js
//Note: This string is used as an exception for developers and does not need to be localized
var L_GraphicsInitError_Text = "Your Web browser does not support SVG or VML. Some graphics features may not function properly.";

RegisterNamespaces("Msn.Drawing");

Msn.Drawing.Exception=function(msg)
{
  this.message = msg;
  this.name = "Msn.Drawing.Exception";
};

Msn.Drawing.Exception.prototype.toString=function()
{
  return this.name+": "+this.message;
};

Msn.Drawing.Graphic=function(){};
/**
  * Graphic factory method that is responsible for creating
  * either a VML or SVG Graphic object based on the browser.
  */
Msn.Drawing.Graphic.CreateGraphic=function(pElm)
{
	if(document.all)//IE
	{         
		return new Msn.Drawing.VMLGraphic(pElm);
	}
	else
	{   //SVG support for Firefox 1.5 only. 
		var major = 0;
		var minor = 0;
		var versionRegex =  new RegExp("Firefox/(.*)");
	    
		var match = versionRegex.exec(navigator.userAgent);
	    
		if(match[1])
		{
			var versionNumbers = match[1].split(".");
			if(versionNumbers)
			{
				major = versionNumbers[0];            
				minor = versionNumbers[1];             
				if(parseInt(major) > 0 && parseInt(minor) >= 5)
				{
					return new Msn.Drawing.SVGGraphic(pElm); 	        
				}         
			}                      
		}
		throw new Msn.Drawing.Exception(L_GraphicsInitError_Text);             
	}
};
//------------------------------------------------------------------------------
// Geometry
//------------------------------------------------------------------------------

Msn.Drawing.Point=function(x,y)
{
  this.x = x ? x : 0;
  this.y = y ? y : 0;
};
//This is only useful for SVG
Msn.Drawing.Point.prototype.toString=function()
{
  return this.x+","+this.y;
};

//------------------------------------------------------------------------------
// Shapes
//------------------------------------------------------------------------------
/**
  * Polyline primitive. 
  * @param points in an array of Msn.Drawing.Point 
  */
Msn.Drawing.PolyLine=function(points)
{
    this.id = 0;    
    this.points = points ? points : new Array();
    
    this.AddPoint=function(aPoint)
    {
      this.points.push(aPoint);
    }

};
// This is only useful for VML
Msn.Drawing.PolyLine.prototype.toString=function()
{
  return this.points.join(" ");
};

Msn.Drawing.Stroke=function()
{
  this.width = 1; //VML = weight
  this.linecap = "round";//TODO:make enum, VML = endcap
  this.opacity = 1.0;
  this.linejoin = "miter";//TODO:make enum, VML = joinstyle
  this.color = new Msn.Drawing.Color(255,255,255,1.0);  
};

//TODO: lots of validation on values passed in 
Msn.Drawing.Color=function(r,g,b,a)
{
  this.R = r ? r : 0;
  this.G = g ? g : 0;
  this.B = b ? b : 0;
  this.A = a ? a : 1.0;
  //Return the hex representation of this color.
  //Note that the alpha is not valid and is ignored on this operation
  //Alpha is however used when rendering shapes and setting their color values. 
  this.ToHexString=function()
  {
    return "#"+Number(this.R).toString(16)+(this.R < 16?"0":"")
    +Number(this.G).toString(16)+(this.G < 16?"0":"")+Number(this.B).toString(16)
    +(this.B < 16?"0":"");
  };

};

//VML implementation of the Graphic interface.
//@param parentElm is the parent DOM element that
//all shapes will be a child of.
Msn.Drawing.VMLGraphic=function(parentElm)
{   
    var graphicsElm = parentElm;
    var color = new Msn.Drawing.Color(255,0,0,1.0);
    var stroke = new Msn.Drawing.Stroke();

    var zIndex = 1;
    
    var parentLeft = parentElm.offsetLeft;
    var parentTop  = parentElm.offsetTop;
    
    parentElm.unselectable = "on";
    
    var currentShapes = new Array();   
    
    this.DrawPolyline=function(polyLine)
    {
		var element = null;

		element = document.createElement("v:polyline"); 		
		element.id =  polyLine.id;
		
		element.unselectable = "on";
		element.style.position = "absolute";
		element.points = polyLine.points.join(" ");
		element.style.zIndex = zIndex;
		element.filled = "false";
		graphicsElm.appendChild(element);   

		if(stroke)
		{
			var strokeElm = document.createElement("v:stroke");
			strokeElm.unselectable = "on";
			strokeElm.setAttribute("weight", stroke.width);
			strokeElm.setAttribute("joinstyle",stroke.linejoin);
			strokeElm.setAttribute("color", stroke.color.ToHexString());
			strokeElm.setAttribute("endcap",stroke.linecap);  		
			strokeElm.setAttribute("opacity",(stroke.color.A).toString());
			element.appendChild(strokeElm);
		}    	
		currentShapes.push(element);
    };
    
    this.SetColor=function(aColor)
    {     
      color = aColor;
    };
    
    this.SetStroke=function(aStroke)
    {
      stroke = aStroke;
    };

    this.SetZIndex=function(index) 
    {
      zIndex = index;
    };
    
    this.Clear = function()
    {
      var currentShape = null;
      while(currentShape = currentShapes.pop())
      {
        graphicsElm.removeChild(currentShape);
        currentShape = null;
      }
    };    

    this.Destroy = function()
	{
	   this.Clear();
	   graphicsElm = null;
	};
};

//SVG implementation of the Graphic interface.
//@param parentElm is the parent DOM element that
//all shapes will be a child of.
Msn.Drawing.SVGGraphic=function(parentElm)
{        
    var divElm = document.createElement("div");    
    divElm.id = "svgCanvas";
    divElm.style.position = "absolute";
    //divElm.style.cursor = "pointer";
    var graphicsElm = null;
    var zIndex = 1;
        
    divElm.style.top    = parentElm.offsetTop+"px";
    divElm.style.left   = parentElm.offsetLeft+"px";
   
    parentElm.appendChild(divElm);
    
    
    var currentShapes = new Array();
    
    var color = new Msn.Drawing.Color(255,0,0,1.0);
    var stroke = new Msn.Drawing.Stroke();
   
    //SetDimensions();

    //set the initial width and height of divElm to zero.
	divElm.style.width  = "0px";
	divElm.style.height = "0px";
      
    graphicsElm = document.createElementNS("http://www.w3.org/2000/svg","svg");      

    graphicsElm.setAttribute("height","100%");
    graphicsElm.setAttribute("width","100%");
    divElm.appendChild(graphicsElm); 
    /**
      * if the container element has a height / width of 0 then
      * walk up the DOM until we find a width and height. 
      */
    function SetDimensions()
    {
      var startSearchNode = parentElm;
       //need to compute new width and height for all points      
      //travel up the tree and find a valid width and height
      while(startSearchNode && startSearchNode.offsetWidth == 0)
      {
		startSearchNode = startSearchNode.parentNode;
      }
      divElm.style.position = "absolute";
      divElm.style.width  = startSearchNode.offsetWidth+"px";
      divElm.style.height = startSearchNode.offsetHeight+"px";      
    }
    
    /**
      * Because SVG does not draw negative points we adjust the svg element
      * and translate the points to be positive. This has the same effect but you
      * need to keep in mind that your containing element should have overflow:hidden.
      */
    function ReComputeDimensions(aPolyline)
    { //find max width and height
      if(!aPolyline || !(aPolyline.points) || (aPolyline.points.length < 2))
      {
        return;
      }
      var points = aPolyline.points;
      var pointsLen = points.length;
      var minX = 0;
      var maxX = 0;
      var minY = 0;
      var maxY = 0;
      var maxWidth = 0;
      var maxHeight = 0;
     
      minX =  Math.min(points[0].x,points[1].x);
      maxX = Math.max(points[0].x,points[1].x);
      minY = Math.min(points[0].y,points[1].y);
      maxY = Math.max(points[0].y,points[1].y);
      maxWidth = Math.max(maxX - minX,maxWidth);
      maxHeight = Math.max(maxY - minX,maxHeight);
      for(var i = 2;i < pointsLen; ++i)
      {
        minX = Math.min(points[i].x,minX);
        maxX = Math.max(points[i].x,maxX);
        minY = Math.min(points[i].y,minY);
        maxY = Math.max(points[i].y,maxY);
        maxWidth = Math.max(maxX - minX,maxWidth);
        maxHeight = Math.max(maxY - minY,maxHeight);
      } 
      divElm.style.top =   minY+"px";
      divElm.style.left = minX+"px";
      divElm.style.width  = maxWidth+"px";
      divElm.style.height = maxHeight+"px";   
      //make sure the svg element has the same width and height as container
      graphicsElm.setAttribute("width",maxWidth+"");
      graphicsElm.setAttribute("height",maxHeight+"");
      //translate points, because we have moved the parent containers origin
      for(var i = 0;i < pointsLen ; ++i)
      {
        points[i].x-= minX;
        points[i].y-= minY;
      }
    }
    
    this.DrawPolyline=function(aPolyLine)
    {
      ReComputeDimensions(aPolyLine);      
      var line = document.createElementNS("http://www.w3.org/2000/svg","polyline");
      line.setAttribute("points",aPolyLine.toString());
      line.setAttribute("fill","none");
      line.setAttribute("id",aPolyLine.id);
      
      if(stroke)
  	  {
        line.setAttribute("stroke",stroke.color.ToHexString());
        line.setAttribute("stroke-width",stroke.width);
        line.setAttribute("stroke-opacity",stroke.color.A);
      }
     
      graphicsElm.appendChild(line);
      
      currentShapes.push(line);
    };
    
    this.SetColor=function(aColor)
    {     
      color = aColor;
    };
    this.SetStroke=function(aStroke)
    {
      stroke = aStroke;
    };
    
    this.SetZIndex=function(index) 
    {
      zIndex = index;
      divElm.style.zIndex = zIndex;
    };
    
    this.Destroy = function()
	{
	   this.Clear();
	};
	  
	this.Clear = function()
    {
       var currentShape = null;
       while(currentShape = currentShapes.pop())
       {
        graphicsElm.removeChild(currentShape);
        currentShape = null;
       }    
    };
};

