//------------------------------------------------------------------------------
// MapControl/Map.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur, keithkin, cthota
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------
#include ..\..\js\window.js
#include Browser.js
#include Math.js
#include Matrix.js
#include Namespaces.js
#include ../Graphics/graphics.js
#include ../Marketization.js
#include ../util/microsoft.web.ui.ieglyph.js


RegisterNamespaces('Msn.VE');

Msn.VE.PushPinTypes =
{
	Default: 0,
	SearchResult: 1,
	Annotation: 2,
	Direction: 3,	
	DirectionTemp: 4,
	TrafficLight: 5,
	TrafficOthers: 6,
	YouAreHere: 7
};

// Constructor - creates and clears all the properties
Msn.VE.MapControl = function(p_elSource, p_htParams)
{

#include Global.js

	// The p_this reference is a way to get a reference to the this object from
	// within anonymous functions.
	var p_this = this;

	var roadStyle = 'r';
	var hybridStyle = 'h';
	var aerialStyle = 'a';
	var obliqueStyle = 'o';
	
	// This represents the current view of the map.
	var currentView = new Msn.VE.MapView();
	
	// The preferred view is used when changing zoom levels or other events that may 
	// change the current restriction on the viewport.  
	// For example, a user may have established that they prefer a latlong just outside
	// of a bounding box.  When the user zooms out, that point might be avaiable in a new
	// bounding box.
	var preferredView = new Msn.VE.MapView();
	
	// The previous zoom level is used after a view transition.  If the zoom levels have changed,
	// then the onendzoom event must be fired.
	var previousZoomLevel = 1;
	
	// The previous map style is used after a view transition.  If the map style has changed,
	// then the onchangemapstyle event must be fired.
	var previousMapStyle = null;
	
	// The last ortho zoom level is used when transitioning from oblique back to ortho.  The oblique
	// and ortho zoom levels are not 100% compatible, so we must go back to the last zoom level.
	var lastOrthoZoomLevel = 15;
	
	// The last ortho map style is used in situations when we need to kick back into ortho, for example
	// when the user moves to an area without oblique or the user needs to zoom out to see multiple points.
	var lastOrthoMapStyle = roadStyle;
	
	// The absolute position and size of the map control.  These values should match the values of
	// the container element in the DOM.  Changing them directly will only mess things up.
	var x = 0, y = 0, width = 0, height = 0;
	
	// The current tiles list is a 1-dimensional array of tile objects.  The order of the tiles
	// array is very important and must be preserved for UpdateView to function properly.
	// The order starts at the top left, moves to the right across the top row, then starts
	// again at the left most tile on the next row down.
	var currentTilesList = new Array();
	
	// The old tiles list is only used during animated zooming where it holds the tiles from the
	// originating view.  During the animated zoom they are synchronously updated with the new tiles
	// coming in to create the animation effect.
	var oldTilesList = null;
	
	var trafficAvailable = false;
	
	var layerTileList = new Array();
	
	// The origin x/y is like an anchor for a view "session".  It represents the absolute distance
	// from the top left corner of the world in pixel space at the current zoom level to an anchor 
	// point that is declared at zoom or a completely new view.
	var originX = 0, originY = 0;
	
	// The offset x/y is the delta between the anchor and the current viewport.
	var offsetX = 0, offsetY = 0;
	
	// The tile viewport definition.  These values represent the boundaries of the map in terms of tile
	// indexes (inclusive).  They are computed in StartView and UpdateView. 
	var tileViewportX1 = 0, tileViewportY1 = 0;
	var tileViewportX2 = 0, tileViewportY2 = 0;
	var tileViewportWidth = 0, tileViewportHeight = 0;
	
	// Tracks whether or not the mouse is currently down and dragged.
	var dragging = false;

	// Tracks if, while panning, the keyboard is being used.
	var keyboardPan = false;
	
	// Tracks the last mouse position for mouse drag panning.
	var lastMouseX = 0, lastMouseY = 0;
	
	// Tracks whether or not the control is in the middle of an animated zoom.
	var zooming = false;
	
	// The zoom counter tracks the progress of an animated zoom.  Animated tiles
	// use the zoomCounter to determine the correct size and position.
	var zoomCounter = 0;
	
	// Tracks whether or not the control is panning (either automatic or manual).
	var panning = false;
	
	// The pan counter tracks the progress of an animated pan.  
	var panCounter = 0;
	
	// The panning deltas keep track of the direction and speed of the pan.
	var panningX = 0, panningY = 0;
	
	// The panning lat/long keeps track of the desired destination of a PanToLatLong call.
	// TODO:  Could this be replaced with preferredView somehow?
	var panLatitude = null, panLongitude = null;
	
	// The pushpins array holds references to all of the pushpins on the map.
	// They are in no particular order.
	var pushpins = new Array();
	
	// The pushpins array holds references to all of the lines on the map.
	// They are in no particular order.
	var lines = new Array();
	
	var map = document.createElement("div");
	var keyboard = document.createElement("input");
	keyboard.id = "wl_ve_mapInput";

	// UI elements
	var logo = null;
	var scaleBar = null;
	var copyright = null;
	
	// Dashboard
	var dashboardContainer = null;
	var dashboard = null;
	
	// Tools
	var boxTool = null;
	var panTool = null;
	var currentTool = null;
	
	// Modes
	var orthoMode = null;
	var obliqueMode = null;
	var currentMode = null;
	
	// The current bounds tracks what is currently being viewed.  This is good to have for panning
	// so it doesn't dig through 100 sets of bounds every time you pan the map.
	var currentBounds = null;
	
	// Event table.  This keeps track of all the other elements out there that have
	// subscribed to the map controls events.  The event table is 2-dimensional -
	// each event can have multiple subscribers which enables multicasting.
	var eventTable = new Array();
	
	// Debug mode overlays image filenames on top of the tiles
	var debug = false;
	
	//SVG/VML Graphic object for drawing driving direction polylines
	var graphicCanvas = null;
	
	// Initialize the control
	// Applies parameters if provided
	// Attaches many events, which are cleaned up in Destroy()
	this.Init = function()
	{
		// Load the generations
		generations[roadStyle] = __roadGeneration__;
		generations[aerialStyle] = __aerialGeneration__;
		generations[hybridStyle] = __hybridGeneration__;
		generations[obliqueStyle] = __obliqueGeneration__;
		
		// Create the ortho view mode
		orthoMode = new OrthoMode();
		orthoMode.Init();
		
		// Create the oblique view mode
		if (p_htParams.obliqueEnabled && p_htParams.obliqueUrl)
		{
			obliqueMode = new ObliqueMode();
			obliqueMode.Init(p_htParams.obliqueUrl);
		}
		
		// Extract location and size info from the parent element
		UpdateFromParent();

		// Prepare the map element
		map.className = "Map";
		map.style.zIndex = mapZIndex;
		p_elSource.appendChild(map);
		
		// Add the keyboard element
		keyboard.className = "KeyboardInput";
		p_elSource.appendChild(keyboard);
		
		// The user can specify that they want a fixed view.  The fixed view does not allow
		// keyboard or mouse interaction.  This can reduce memory usage per control.
		if (!p_htParams.fixedView)
		{
			// The box tool is used for drawing boxes that are converted into new views.
			boxTool = new BoxTool();
			boxTool.Init();
			
			// The pan tool is the most commonly used tool, that allows the user to drag and pan.
			panTool = new PanTool();
			panTool.Init();
			
			// The default tool is the pan tool.
			currentTool = panTool;
		
			// Subscribe to all of the events needed for map nav via mouse and keyboard
			p_elSource.attachEvent("onmousedown", MouseDown);
			p_elSource.attachEvent("onmouseup", MouseUp);
			p_elSource.attachEvent("onmousemove", MouseMove);
			p_elSource.attachEvent("onmousewheel", MouseWheel);
			p_elSource.attachEvent("ondblclick", MouseDoubleClick);
			p_elSource.attachEvent("oncontextmenu", ContextMenu);			
			keyboard.attachEvent("onkeydown", KeyDown);
			keyboard.attachEvent("onkeyup", KeyUp);
			keyboard.attachEvent('onblur', StopKeyboardPan);
			
			// Add the mouse scroll wheel event for Firefox and other gecko based browsers.
			if (window.addEventListener && navigator.product && navigator.product == "Gecko")
			{
				p_elSource.addEventListener("DOMMouseScroll", MouseWheel, false);
			}
			
			// If the view is dynamic, make the buffer the size of 1 tile.
			// If the view is not dynamic, the buffer size will be 0 (no buffer should be needed, right?)
			buffer = tileSize;
		}
		
		// The user can override the buffer if they want.  This is especially useful for fixed views.
		if (p_htParams.buffer != undefined && p_htParams.buffer != null)
		{
			buffer = p_htParams.buffer;
		}
		
		// If the developer specifies a default view, use that.  If it causes an exception or doesn't exist,
		// we can always resort back to the default view (picture of the whole world).
		if (p_htParams.latitude && p_htParams.longitude && p_htParams.zoomlevel && p_htParams.mapstyle)
		{
			try
			{
				var initialView = new Msn.VE.MapView();
				initialView.SetMapStyle(ValidateMapStyle(p_htParams.mapstyle), p_htParams.obliqueSceneId);
				initialView.SetZoomLevel(eval(p_htParams.zoomlevel));
				initialView.SetCenterLatLong(new Msn.VE.LatLong(eval(p_htParams.latitude), eval(p_htParams.longitude)));
				SetView(initialView);
			}
			catch (ex)
			{
				SetDefaultView();
			}
		}
		else
		{
			SetDefaultView();
		}
		
		// Unless explicitly disabled, show the logo
		if (!p_htParams.disableLogo)
		{
			logo = new Logo(p_elSource);
			logo.Init();
		}
		
		// Build the copyright
		copyright = new Copyright(p_elSource);
		copyright.Init();
		copyright.Update();
		
		// If requested, build and display the scale bar
		if (p_htParams.showScaleBar)
		{
			scaleBar = new ScaleBar(p_elSource);
			scaleBar.Init();
			AttachEvent("onendzoom", scaleBar.Update);
			AttachEvent("onendcontinuouspan", scaleBar.Update);
			AttachEvent("onresize", scaleBar.Reposition);
		}
		
		// If requested, build and display the dashboard
		// TODO:  Perhaps the dashboard should be created separately, and manually
		
		if (p_htParams.showDashboard)
		{
			CreateDashboard(p_htParams.dashboardX, p_htParams.dashboardY, p_htParams.dashboardSize, p_htParams.dashboardId);
		}
		
		// Request an oblique update
		if (obliqueMode && currentMode != obliqueMode)
		{
			obliqueMode.UpdateAvailability();
		}
		try 
		{
			graphicCanvas = Msn.Drawing.Graphic.CreateGraphic(map);	
			graphicCanvas.SetZIndex(17);	
		}
		catch(e)
		{
			//ShowMessage(e.message);
		}
	};
	
	// Returns the dashboard.  The dashboard may be null if the user did not request it.
	this.GetDashboard = function()
	{
		return dashboard;
	};
	
	// Deconstructor, detaches events and attempts to clean up memory references
	this.Destroy = function()
	{
		if (!p_htParams.fixedView)
		{
			p_elSource.detachEvent("onmousedown", MouseDown);
			p_elSource.detachEvent("onmouseup", MouseUp);
			p_elSource.detachEvent("onmousemove", MouseMove);
			p_elSource.detachEvent("onmousewheel", MouseWheel);
			p_elSource.detachEvent("ondblclick", MouseDoubleClick);
			p_elSource.detachEvent("oncontextmenu", ContextMenu);
			keyboard.detachEvent("onkeydown", KeyDown);
			keyboard.detachEvent("onkeyup", KeyUp);
			keyboard.detachEvent('onblur', StopKeyboardPan);
		}
		
		//Remove original tile layer
		if(currentTilesList != null)
		{
			ClearTiles(currentTilesList);
			currentTilesList = null;
		}

		//Remove traffic tile layer		
		this.RemoveTrafficLayer();
		
		while (pushpins.length)
		{
			pushpins.pop().Destroy();
		}
		
		pushpins = null;
		
		ClearLines();
		
		if (dashboard)
		{
			dashboard.Destroy();
			dashboard = null;
		}
		
		if (dashboardContainer)
		{
			dashboardContainer.detachEvent("onmousedown", IgnoreEvent);
			dashboardContainer.detachEvent("onmouseup", IgnoreEvent);
			dashboardContainer.detachEvent("onmousemove", IgnoreEvent);
			dashboardContainer.detachEvent("onmousewheel", IgnoreEvent);
			dashboardContainer.detachEvent("ondblclick", IgnoreEvent);
			dashboardContainer.detachEvent("oncontextmenu", IgnoreEvent);
			dashboardContainer.detachEvent("onkeydown", IgnoreEvent);
			dashboardContainer.detachEvent("onkeyup", IgnoreEvent);			
			p_elSource.removeChild(dashboardContainer);			
			dashboardContainer = null;
		}
		
		if (scaleBar)
		{
			DetachEvent("onendzoom", scaleBar.Update);
			DetachEvent("onendcontinuouspan", scaleBar.Update);
			DetachEvent("onresize", scaleBar.Reposition);
			scaleBar.Destroy();
			scaleBar = null;
		}
		
		if (logo)
		{
			logo.Destroy();
			logo = null;
		}
		
		if (copyright)
		{
			copyright.Destroy();
			copyright = null;
		}
		
		if (orthoMode)
		{
			orthoMode.Destroy();
			orthoMode = null;
		}
		
		if (obliqueMode)
		{
			obliqueMode.Destroy();
			obliqueMode = null;
		}
		
		if (boxTool)
		{
			boxTool.Destroy();
			boxTool = null;
		}
		
		if (panTool)
		{
			panTool.Destroy();
			panTool = null;
		}
		
		if(graphicCanvas)
		{
			graphicCanvas.Destroy();
		}
		
		// Clears out all the event handlers that have been attached to the map control.
		DestroyEventTable();
		
		p_elSource = p_this = null;
	};
	
	// Updates the position from the parent element
	function UpdateFromParent()
	{
		x = GetLeftPosition(p_elSource);
		y = GetTopPosition(p_elSource);
		width = p_elSource.offsetWidth;
		height = p_elSource.offsetHeight;
	}
	
	// Optionally, a developer can request the dashboard be created at the same time as the control.
	// The built in dashboard is no different than the developer creating the dashboard manually
	// outside of the control.
	function CreateDashboard(x, y, size, dashboardId)
	{
		if (!size)
		{
			size = Msn.VE.DashboardSize.Normal;
		}
		
		if(size == Msn.VE.DashboardSize.Tiny)
		{
			size = Msn.VE.DashboardSize.Small;
		}
		
		dashboardContainer = document.createElement("div");
		if ((dashboardId == null) || (typeof(dashboardId)=="undefined"))
		{
			dashboardId = "dashboard";
		}
		dashboardContainer.id = dashboardId;
		p_elSource.appendChild(dashboardContainer);		

		dashboardContainer.className = "Dashboard Dashboard_" + size;		
		dashboardContainer.style.top = y + "px";
		dashboardContainer.style.left = x + "px";
		
		// The base dashboard element itself needs to capture and ignore all mouse and key events.
		// Otherwise, they will bubble up to the map control and adjust the map.
		// We don't want a double click on the dashboard to accidentally zoom in the map view.
		
		dashboardContainer.attachEvent("onmousedown", IgnoreEvent);
		dashboardContainer.attachEvent("onmouseup", IgnoreEvent);
		dashboardContainer.attachEvent("onmousemove", IgnoreEvent);
		dashboardContainer.attachEvent("onmousewheel", IgnoreEvent);
		dashboardContainer.attachEvent("ondblclick", IgnoreEvent);
		dashboardContainer.attachEvent("oncontextmenu", IgnoreEvent);
		dashboardContainer.attachEvent("onkeydown", IgnoreEvent);
		dashboardContainer.attachEvent("onkeyup", IgnoreEvent);
		
		dashboard = new Msn.VE.Dashboard(dashboardContainer, p_this, size);
		dashboard.Init();
	}
	
	// Sets the default view and captures it as the preferred zoom.  The default view
	// is an overview of the whole world centered at lat/long 0, 0 with the road style.
	function SetDefaultView()
	{
		var view = new Msn.VE.MapView();
		view.SetCenterLatLong(new Msn.VE.LatLong(0.0, 0.0));
		view.SetZoomLevel(1);
		view.SetMapStyle(roadStyle);
		SetView(view);
	}

	// Set the center point of the map.
	// This will ultimately clear all tiles and start a new set.
	// If you want a smooth transition, you should call PanMap or ContinuousPan
	function SetCenter(latitude, longitude)
	{
		var view = preferredView.MakeCopy();
		view.SetCenterLatLong(new Msn.VE.LatLong(latitude, longitude));
		SetView(view);
	}

	// Sets the map style
	// This will ultimately clear all the tiles and start a new set.
	function SetMapStyle(mapStyle, sceneId, sceneOrientation)
	{
		// Note, this is one of the few methods that uses currentView instead of preferredView
		var view = currentView.MakeCopy();
		view.SetMapStyle(ValidateMapStyle(mapStyle), sceneId, sceneOrientation);
		
		// If we're changing map styles between ortho and oblique
		// then we need to preserve the zoom level
		if (currentView.mapStyle != mapStyle)
		{
			if (mapStyle == obliqueStyle)
			{
				view.SetZoomLevel(1);					
				lastOrthoZoomLevel = currentView.zoomLevel;
				lastOrthoMapStyle = currentView.mapStyle;
			}
			else if (currentView.mapStyle == obliqueStyle)
			{
				view.SetZoomLevel(lastOrthoZoomLevel);
			}
		}
		
		view.latlong.latitude=currentView.latlong.latitude;
		view.latlong.longitude=currentView.latlong.longitude;	
			
		SetView(view);
	}

	// Validates a map style.
	// Accepts aerial, otherwise converts to vector.
	function ValidateMapStyle(mapStyle)
	{
		if (mapStyle == aerialStyle || mapStyle == hybridStyle || mapStyle == obliqueStyle)
		{
			return mapStyle;
		}
		else
		{
			return roadStyle;
		}
	}

	// Returns the center latitude
	function GetCenterLatitude()
	{
		if ((currentView != null) && (currentView != "undefined") && 
			(currentView.latlong != null) && (currentView.latlong != "undefined") &&
			(currentView.latlong.latitude != null) && (currentView.latlong.latitude != "undefined"))
		{
			return currentView.latlong.latitude;
		}
		return null;
	}

	// Returns the center longitude
	function GetCenterLongitude()
	{
		if ((currentView != null) && (currentView != "undefined") && 
			(currentView.latlong != null) && (currentView.latlong != "undefined") &&
			(currentView.latlong.longitude != null) && (currentView.latlong.longitude != "undefined"))
		{
			return currentView.latlong.longitude;
		}
		return null;
	}
	
	// Computes the center point
	function ComputeCenterPoint(capture)
	{
		currentView.latlong = currentMode.PixelToLatLong(currentView.center, currentView.zoomLevel);
		
		if (capture)
		{
			preferredView.Copy(currentView);
		}
	
		// Update oblique availability
		if (obliqueMode)
		{
			obliqueMode.UpdateAvailability();
		}
	}

	// Returns the latitude at a y-coordinate on the display
	function GetLatitude(y)
	{
		var pixel = new Msn.VE.Pixel(originX + offsetX + width/2, originY + offsetY + y);
		var latlong = currentMode.PixelToLatLong(pixel, currentView.zoomLevel);
		if (!latlong)
		{
			return null;
		}
		
		return latlong.latitude;
	}

	// Returns the longitude at an x-coordinate on the display
	function GetLongitude(x)
	{
		var pixel = new Msn.VE.Pixel(originX + offsetX + x, originY + offsetY + height/2);
		var latlong = currentMode.PixelToLatLong(pixel, currentView.zoomLevel);
		if (!latlong)
		{
			return null;
		}
		
		return latlong.longitude;
	}

	// Returns the y-coordinate for a latitude
	function GetY(latitude)
	{
		var latlong = new Msn.VE.LatLong(latitude, currentView.center.longitude);
		var pixel = currentMode.LatLongToPixel(latlong, currentView.zoomLevel);
		if (!pixel)
		{
			return null;
		}
		
		return MathRound(pixel.y - originY - offsetY);
	}

	// Returns the x-coordinate for a longitude
	function GetX(longitude)
	{
		var latlong = new Msn.VE.LatLong(currentView.center.latitude, longitude);
		var pixel = currentMode.LatLongToPixel(latlong, currentView.zoomLevel);
		if (!pixel)
		{
			return null;
		}
		
		return MathRound(pixel.x - originX - offsetX);
	}
	
	// Returns the pixel location of the requested latlong
	function LatLongToPixel(latlong)
	{
		var pixel = currentMode.LatLongToPixel(latlong, currentView.zoomLevel);
		pixel.x -= originX + offsetX;
		pixel.y -= originY + offsetY;
		return pixel;
	}
	
	// Returns the pixel location of the requested latlong
	function PixelToLatLong(pixel)
	{
		var adjPixel = new Msn.VE.Pixel(pixel.x + originX + offsetX, pixel.y + originY + offsetY);
		var latlong = currentMode.PixelToLatLong(adjPixel, currentView.zoomLevel);
		return latlong;
	}

	// Returns the current zoom level
	function GetZoomLevel()
	{
		return currentView.zoomLevel;
	}

	// Returns the current map style
	function GetMapStyle()
	{
		return currentView.mapStyle;
	}

	// Returns the approximate meters per pixel at a latitude and zoom level
	// Both parameters are optional, and will default to the current settings if omitted.
	function GetMetersPerPixel(latitude, zoomLevel)
	{
		if (!latitude)
		{
			latitude = currentView.latlong.latitude;
		}
		
		if (!zoomLevel)
		{
			zoomLevel = currentView.zoomLevel;
		}
		
		return Math.cos(DegToRad(latitude)) * currentMode.MetersPerPixel(zoomLevel);
	}

	// Resize the control.
	function Resize(w, h)
	{
		/*	If w and/or h passed into this function are valid, they will be honored.
				However, if w and/or h are null or invalid, they will default to '100%'				
		*/
			
		var width, height;
		
		if (!w || w <= 0)
			{ width = '100%'; }
		else
			{ width = w + 'px'; }
			 
		if (!h || h <= 0)
			{ height = '100%'; }
		else
			{ height = h + 'px'; }

		p_elSource.style.width = width;
		p_elSource.style.height = height;
		
		UpdateFromParent();
		PanToView(currentView);	
				
		copyright.Reposition();
		scaleBar.Reposition();
		if(logo)
		{
			logo.Reposition();
		}
		Fire("onresize");
	}
	
	// Tests for oblique availability.
	function IsObliqueAvailable()
	{
		return obliqueMode ? obliqueMode.IsAvailable() : false;
	}
	
	// Gets the oblique scene
	function GetObliqueScene()
	{
		return obliqueMode ? obliqueMode.GetScene() : null;
	}
	
	// Turns on or off animated zoom and pan
	function SetAnimationEnabled(enabled)
	{
		animatedMovementEnabled = enabled;
	}
	
	// Checks whether or not animation is enabled
	function IsAnimationEnabled()
	{
		return animatedMovementEnabled;
	}
	
	// Sets the oblique scene
	function SetObliqueScene(id)
	{
		if (obliqueMode)
		{
			SetMapStyle(obliqueStyle, id, null);
		}
	}
	
	// Sets the oblique orientation
	function SetObliqueOrientation(orientation)
	{
		if (obliqueMode)
		{
			SetMapStyle(obliqueStyle, null, orientation);
		}
	}
	
	// Sets the debug status for the control.  When in debug mode, 
	// the map displays filenames over the tiles
	function Debug(d)
	{
		debug = d;
		
		for (var i=0; i<currentTilesList.length; i++)
		{
			currentTilesList[i].Debug(d);
		}
	}
	
	function SetCopyrightOffset(xOffset, yOffset)
	{
		copyright.SetOffset(xOffset, yOffset);
		scaleBar.SetOffset(xOffset, yOffset);
	}
	
	function SetFocus()
	{
		keyboard.focus();
	}
	
	function StopKeyboardPan()
	{
		if (panning && keyboardPan)
		{
			StopContinuousPan();
		}		
	}

#include Clipping.js
#include Copyright.js
#include Events.js
#include Keyboard.js
#include Line.js
#include Logo.js
#include Mouse.js
#include Pan.js
#include Pushpin.js
#include ScaleBar.js
#include Tile.js
#include View.js
#include Viewport.js
#include XmlHttp.js
#include Zoom.js

#include Modes/ObliqueMode.js
#include Modes/OrthoMode.js

#include Tools/BoxTool.js
#include Tools/PanTool.js
#include Traffic.js
#include Layer.js

	// Publish the public methods.  With the javascript closure pattern, functions are, by default, not visible
	// to external code.  Therefore, we must publish the methods by making them publicly accessible to the "this"
	// object.
	this.SetCenter = SetCenter;
	this.SetMapStyle = SetMapStyle;
	this.GetCenterLatitude = GetCenterLatitude;
	this.GetCenterLongitude = GetCenterLongitude;
	this.GetLatitude = GetLatitude;
	this.GetLongitude = GetLongitude;
	this.GetY = GetY;
	this.GetX = GetX;
	this.LatLongToPixel = LatLongToPixel;
	this.PixelToLatLong = PixelToLatLong;
	this.GetZoomLevel = GetZoomLevel;
	this.GetMapStyle = GetMapStyle;
	this.GetMetersPerPixel = GetMetersPerPixel;
	this.Resize = Resize;
	this.PanMap = PanMap;
	this.ContinuousPan = ContinuousPan;
	this.StopContinuousPan = StopContinuousPan;
	this.StopKeyboardPan = StopKeyboardPan;
	this.PanToLatLong = PanToLatLong;
	this.GetPushpins = GetPushpins;
	this.AddPushpin = AddPushpin;
	this.RemovePushpin = RemovePushpin;
	this.ClearPushpins = ClearPushpins;
	this.SetViewport = SetViewport;
	this.SetBestMapView = SetBestMapView;
	this.IncludePointInViewport = IncludePointInViewport;
	this.SetZoom = SetZoom;
	this.ZoomIn = ZoomIn;
	this.ZoomOut = ZoomOut;
	this.SetCenterAndZoom = SetCenterAndZoom;
	this.AddLine = AddLine;
	this.RemoveLine = RemoveLine;
	this.ClearLines = ClearLines;
	this.AttachEvent = AttachEvent;
	this.DetachEvent = DetachEvent;
	this.IsObliqueAvailable = IsObliqueAvailable;
	this.GetObliqueScene = GetObliqueScene;
	this.SetAnimationEnabled = SetAnimationEnabled;
	this.IsAnimationEnabled = IsAnimationEnabled;
	this.SetObliqueScene = SetObliqueScene;
	this.SetObliqueOrientation = SetObliqueOrientation;
	this.SetView = SetView;
	this.Debug = Debug;
	this.GetResponseRangeCounts = GetResponseRangeCounts;
	this.ResetResponseRangeCounts = ResetResponseRangeCounts;
	this.GetFailureRate = GetFailureRate;
	this.SetTrafficView = SetTrafficView;
	this.GetTrafficAvailability = GetTrafficAvailability;
	this.RemoveTrafficLayer = RemoveTrafficLayer;
	this.GetMarketsFile = GetMarketsFile;
	this.GetIncidentsFile = GetIncidentsFile;
	this.GetSlidingExpirationForAutoRefresh = GetSlidingExpirationForAutoRefresh;
	this.GetAutoRefreshRate = GetAutoRefreshRate;
	this.SetCopyrightOffset = SetCopyrightOffset;
	this.SetFocus = SetFocus;	
};

#include Bounds.js
#include Dashboard/Dashboard.js
#include Dashboard/DashboardSize.js
#include LatLong.js
#include LatLongRectangle.js
#include MapStyle.js
#include MapView.js
#include ObliqueScene.js
#include Orientation.js
#include Pixel.js
#include PixelRectangle.js
#include LineRegion.js
#include Encoding/NumberEncoding.js
#include Encoding/LatLonEncoding.js
