//------------------------------------------------------------------------------
// MapControl/Copyright.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

//VECopyrightTableEntry
//   An entry for the copyright table
//   MinZoomLevel
//   MaxZoomLevel
//   MinLatitude
//   MinLongitude
//   MaxLatitude
//   MaxLongitude
//
//   IsMatch(zoomLevel, latitude, longitude)
//      returns true if the passed in values match within the bounds of the entry
function VECopyrightTableEntry(minZoomLevel, maxZoomLevel, minLat, minLon, maxLat, maxLon)
{
	this.MinZoomLevel = minZoomLevel;
	this.MaxZoomLevel = maxZoomLevel;
	this.MinLatitude = minLat;
	this.MinLongitude = minLon;
	this.MaxLatitude = maxLat;
	this.MaxLongitude = maxLon;
}

VECopyrightTableEntry.prototype.IsMatch = function(zoomLevel, latitude, longitude)
	{
		var isMatch = false;
		if  ( ((zoomLevel >= this.MinZoomLevel) && (zoomLevel <= this.MaxZoomLevel))
				&&
				((latitude >= this.MinLatitude) && (latitude <= this.MaxLatitude))
				&&
				((longitude >= this.MinLongitude) && (longitude <= this.MaxLongitude))
			)
		{
			isMatch = true;
		}
		return isMatch;
	}


//VECopyrightTable
//   Add(veCopyrightTableEntry)
//      Adds VECopyrightTableEntry to the table
//   CreditsForView(view)
//      returns an array of copyright credits for the passed in view
//   CreditsFor(mapStyle, zoomLevel, latitude, longitude)
//      returns an array of copyright credits for the passed in parameters
function VECopyrightTable()
{
	//Create copyright keys for table entries. Each key is a unique copyright credit.
	//The keys are associated with the styles they can apply to.
	//Each time you add a new copyright credit, you need to add it to proper key table
	//for the styles it can apply to.
	var m_tableKeys = new Array();
	m_tableKeys[Msn.VE.MapStyle.Oblique] = [ L_MapControlImageCourtesyOfPictometry_Text ];
	m_tableKeys[Msn.VE.MapStyle.Road] =    [ L_MapControlNavteq_Text, L_MapControlImageCourtesyOfAND_Text ];
	m_tableKeys[Msn.VE.MapStyle.Aerial] =  [ L_MapControlImageCourtesyOfNASA_Text, L_MapControlHarrisCorp_Text, L_MapControlImageCourtesyOfUSGS_Text, L_MapControlImageCourtesyOfEarthData_Text, L_MapControlImageCourtesyOfGetmapping_Text ];
	
	//Create the hierarchical table structure - instantiate array objects so table can be populated.
	//When new map styles are added, or new copyright credits are added, this needs to be updated.
	var m_table = new Array();
	m_table[Msn.VE.MapStyle.Oblique] = new Array();
	m_table[Msn.VE.MapStyle.Oblique][L_MapControlImageCourtesyOfPictometry_Text] = new Array();
	m_table[Msn.VE.MapStyle.Road] = new Array();
	m_table[Msn.VE.MapStyle.Road][L_MapControlNavteq_Text] = new Array();
	m_table[Msn.VE.MapStyle.Road][L_MapControlImageCourtesyOfAND_Text] = new Array();
	m_table[Msn.VE.MapStyle.Aerial] = new Array();
	m_table[Msn.VE.MapStyle.Aerial][L_MapControlImageCourtesyOfNASA_Text] = new Array();
	m_table[Msn.VE.MapStyle.Aerial][L_MapControlHarrisCorp_Text] = new Array();
	m_table[Msn.VE.MapStyle.Aerial][L_MapControlImageCourtesyOfUSGS_Text] = new Array();
	m_table[Msn.VE.MapStyle.Aerial][L_MapControlImageCourtesyOfEarthData_Text] = new Array();
	m_table[Msn.VE.MapStyle.Aerial][L_MapControlImageCourtesyOfGetmapping_Text] = new Array();
	
	//Populate the copyright table.
	//The table is hierarchical in nature to improve performance so we do not need to linearly search through the
	//entire table each time. The first level is the map style. The second level is the Copyright Credit.
	//The leaf nodes are all VECopyrightTableEntries.
	//
    //      Map Style                Credit                                                                        MinZ   MaxZ  MinLat   MinLon  MaxLat  MaxLon 
	m_table[Msn.VE.MapStyle.Oblique][L_MapControlImageCourtesyOfPictometry_Text].push( new VECopyrightTableEntry(    1,    20,    24,    -125,    49,    -66 ) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlNavteq_Text                   ].push( new VECopyrightTableEntry(    1,     9,   -90,    -180,    90,     180) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlNavteq_Text                   ].push( new VECopyrightTableEntry(   10,    19,    16,    -180,    90,     -50) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlNavteq_Text                   ].push( new VECopyrightTableEntry(   10,    19,    27,     -32,    40,     -13) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlNavteq_Text                   ].push( new VECopyrightTableEntry(   10,    19,    35,     -11,    72,      20) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlNavteq_Text                   ].push( new VECopyrightTableEntry(   10,    19,    21,      20,    72,      32) );
	m_table[Msn.VE.MapStyle.Road   ][L_MapControlImageCourtesyOfAND_Text       ].push( new VECopyrightTableEntry(   10,    19,   -90,    -180,    90,     180) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfNASA_Text      ].push( new VECopyrightTableEntry(    1,     8,   -90,    -180,    90,     180) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlHarrisCorp_Text               ].push( new VECopyrightTableEntry(    9,    13,   -90,    -180,    90,     180) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfUSGS_Text      ].push( new VECopyrightTableEntry(   14,    19, 17.99, -150.11, 61.39,  -65.57) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfEarthData_Text ].push( new VECopyrightTableEntry(   14,    19, 21.25, -158.30, 21.72, -157.64) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfEarthData_Text ].push( new VECopyrightTableEntry(   14,    19, 39.99,  -80.53, 40.87,  -79.43) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfEarthData_Text ].push( new VECopyrightTableEntry(   14,    19, 34.86,  -90.27, 35.39,  -89.60) );
	m_table[Msn.VE.MapStyle.Aerial ][L_MapControlImageCourtesyOfGetmapping_Text].push( new VECopyrightTableEntry(   14,    19, 49.94,   -6.35, 58.71,    1.78) );

	//   CreditsFor(mapStyle, zoomLevel, latitude, longitude)
	//      returns an array of copyright credits for the passed in parameters
	this.CreditsFor=function(mapStyle, zoomLevel, latitude, longitude)
	{
		var credits = new Array();
		//Make sure the map style is in the table
		if ( (mapStyle != "undefined") && (mapStyle != null)  && 
			 (typeof(m_tableKeys[mapStyle]) != "undefined") && (m_tableKeys[mapStyle] != null) )
		{
			//Iterate through the copyright keys for this style
			var numKeys = m_tableKeys[mapStyle].length;
			for (var k=0; k < numKeys; ++k)
			{
				//Iterate through the table entries for this copyright key
				var key = m_tableKeys[mapStyle][k];
				var keyArray = m_table[mapStyle][key];
				var numEntries = keyArray.length;
				for (var i=0; i < numEntries; ++i)
				{
					//If this key is a match, put it in the credits and
					//break out of the loop for performance reasons since we already know this key is a match
					if (keyArray[i].IsMatch(zoomLevel, latitude, longitude))
					{
						credits.push(key);
						break;
					}
				}
			}
		}
		
		return credits;
	}

	//   CreditsForView(view)
	//      returns an array of copyright credits for the passed in view
	this.CreditsForView = function(view)
	{
		var copyrightCredits = new Array();
		copyrightCredits.push(L_MapCopyrightMicrosoft);
		
		if (view.mapStyle == Msn.VE.MapStyle.Hybrid)
		{
			//For Hybrid, get the credits for both Road and Aerial
			copyrightCredits = copyrightCredits.concat(this.CreditsFor(Msn.VE.MapStyle.Road, view.zoomLevel, view.latlong.latitude, view.latlong.longitude));
			copyrightCredits = copyrightCredits.concat(this.CreditsFor(Msn.VE.MapStyle.Aerial, view.zoomLevel, view.latlong.latitude, view.latlong.longitude));
		}
		else
		{
			copyrightCredits = copyrightCredits.concat(this.CreditsFor(view.mapStyle, view.zoomLevel, view.latlong.latitude, view.latlong.longitude));
		}
		
		return copyrightCredits;
	}
}

//Static global CopyrightTable
var g_sVECopyrightTable = new VECopyrightTable();

function Copyright(parent)
{
	var bg = document.createElement("div");
	var fg = document.createElement("div");
	var copyrightHeight = 13;
	
	// x offset from right border
	var xOffset = 0;

	// y offset from bottom border
	var yOffset = 0;

	// Create the copyright elements
	this.Init = function()
	{
		//bg.unselectable = "on";
		bg.className = "Copyright CopyrightBackground";
		
		//fg.unselectable = "on";
		fg.className = "Copyright CopyrightForeground";
		
		Reposition();
		parent.appendChild(bg);
		parent.appendChild(fg);
	};
	
	this.Destroy = function()
	{
		parent.removeChild(bg);
		parent.removeChild(fg);
		bg = fg = null;
	};

	// Reposition the copyright
	function Reposition()
	{
		// minus border related offset, copyright offset, and possibly offset to center copyright between
		// the top of footer and the bottom of scale bar.
		bg.style.top = (height - copyrightHeight - 3 - yOffset - (copyrightHeight <= 13 ? 4 : 0)) + "px";

		bg.style.height = copyrightHeight + "px";
		bg.style.right = '9px';
		bg.style.display = "block";
		
		fg.style.top = (height - copyrightHeight - 3 - 1 - yOffset - (copyrightHeight <= 13 ? 4 : 0)) + "px";
		fg.style.height = copyrightHeight + "px";
		fg.style.right = '10px';
		fg.style.display = "block";
	
		
	}

	// Update the contents of the copyright for the content in view
	function Update()
	{
		var credits = g_sVECopyrightTable.CreditsForView(currentView);

		//Currently we go to the next line if there are 2 copyrights already listed.
		//Note: this assumes the copyrights fit on 2 lines and do not wrap 300px. 
		//With the current table this works. However, if we start showing more copyrights, 
		//we will need to adjust the height/width of the copyright on the map control and the scalebar.

		var copyrightString = "";
		for (var i=0; i < credits.length; ++i)
		{
			if (i > 0)
			{
				if (i== 2)
				{
					copyrightString += "<br/>";
				}
				else
				{
					copyrightString += "&nbsp;&nbsp;";
				}
			}
			copyrightString += credits[i];
		}
		
		if (credits.length <= 2)
		{
			copyrightHeight = 13;
		}
		else
		{
			copyrightHeight = 21;
		}
		
		bg.innerHTML = copyrightString;
		fg.innerHTML = copyrightString;
		Reposition();
	}
	
	function SetOffset(_xOffset, _yOffset)
	{
		xOffset = _xOffset;
		yOffset = _yOffset;
		Reposition();
	}
	
	this.Reposition = Reposition;
	this.Update = Update;
	this.SetOffset = SetOffset;
}
