//------------------------------------------------------------------------------
// MapControl/Dashboard/ObliqueSelector.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// The oblique selector is the UI element that displays thumbnails of the neighboring
// scenes to assist the user in oblique navigation.
function ObliqueSelector(parent)
{
	// The primary element is the table.
	var element = document.createElement("table");
	
	// Cells is a 2d array of ObliqueSelectorLink objects
	var cells = new Array(3);
	
	// Initializes the selector
	this.Init = function()
	{
		element.className = "obliqueSelectorTable";
	
		// By default, the selector is hidden.
		Hide();
		
		// Add the table to the parent element.
		parent.appendChild(element);
		
		// Create the TBODY.  This is required for FirFox.
		var tbody = document.createElement("tbody");
		tbody.className = "obliqueSelectorTable";
		element.appendChild(tbody);
		
		// Create the links
		for (var i=0; i<3; i++)
		{
			var row = document.createElement("tr");
			tbody.appendChild(row);
			cells[i] = new Array(3);
			
			for (var j=0; j<3; j++)
			{
				var link = document.createElement("div");
				
				var cell = document.createElement("td");
				row.appendChild(cell);
				
				cell.appendChild(link);
				
				link.className = "Dashboard_thumbnail";
				
				if (i == 1 && j == 1)
				{
					// The center cell has a unique class and is not enabled
					cells[i][j] = new ObliqueSelectorLink(link, false);
					cell.className = "obliqueSelectorCellCenter";
					//link.style.padding = "1px";
				}
				else
				{
					cells[i][j] = new ObliqueSelectorLink(link, true);
					cell.className = "obliqueSelectorCellOff";
				}
				
				cells[i][j].Init();
			}
		}
	};
	
	// Destroys the selector
	this.Destroy = function()
	{
		parent.removeChild(element);
		element = null;
		
		for (var i=0; i<cells.length; i++)
		{
			for (var j=0; j<cells[i].length; j++)
			{
				cells[i][j].Destroy();
				cells[i][j] = null;
			}
		}
		
		cells = null;
	};
	
	// Shows the selector
	function Show()
	{
		Update();
		element.style.display = "block";
	}
	
	// Hides the selector
	function Hide()
	{
		element.style.display = "none";
	}
	
	// Updates all the links with the latest scene in the map control.
	function Update()
	{
		var scene = map.GetObliqueScene();
		if (!scene)
		{
			return;
		}
				
		cells[1][1].SetScene(scene);
		
		switch (scene.GetOrientation())
		{
			case "North":
				cells[0][0].SetScene(scene.GetNeighbor("Northwest"));
				cells[0][1].SetScene(scene.GetNeighbor("North"));
				cells[0][2].SetScene(scene.GetNeighbor("Northeast"));
				
				cells[1][0].SetScene(scene.GetNeighbor("West"));
				cells[1][2].SetScene(scene.GetNeighbor("East"));
				
				cells[2][0].SetScene(scene.GetNeighbor("Southwest"));
				cells[2][1].SetScene(scene.GetNeighbor("South"));
				cells[2][2].SetScene(scene.GetNeighbor("Southeast"));
				break;
				
			case "East":
				cells[0][0].SetScene(scene.GetNeighbor("Northeast"));
				cells[0][1].SetScene(scene.GetNeighbor("East"));
				cells[0][2].SetScene(scene.GetNeighbor("Southeast"));
				
				cells[1][0].SetScene(scene.GetNeighbor("North"));
				cells[1][2].SetScene(scene.GetNeighbor("South"));
				
				cells[2][0].SetScene(scene.GetNeighbor("Northwest"));
				cells[2][1].SetScene(scene.GetNeighbor("West"));
				cells[2][2].SetScene(scene.GetNeighbor("Southwest"));
				break;
				
			case "West":
				cells[0][0].SetScene(scene.GetNeighbor("Southwest"));
				cells[0][1].SetScene(scene.GetNeighbor("West"));
				cells[0][2].SetScene(scene.GetNeighbor("Northwest"));
				
				cells[1][0].SetScene(scene.GetNeighbor("South"));
				cells[1][2].SetScene(scene.GetNeighbor("North"));
				
				cells[2][0].SetScene(scene.GetNeighbor("Southeast"));
				cells[2][1].SetScene(scene.GetNeighbor("East"));
				cells[2][2].SetScene(scene.GetNeighbor("Northeast"));
				break;
				
			case "South":
				cells[0][0].SetScene(scene.GetNeighbor("Southeast"));
				cells[0][1].SetScene(scene.GetNeighbor("South"));
				cells[0][2].SetScene(scene.GetNeighbor("Southwest"));
				
				cells[1][0].SetScene(scene.GetNeighbor("East"));
				cells[1][2].SetScene(scene.GetNeighbor("West"));
				
				cells[2][0].SetScene(scene.GetNeighbor("Northeast"));
				cells[2][1].SetScene(scene.GetNeighbor("North"));
				cells[2][2].SetScene(scene.GetNeighbor("Northwest"));
				break;
		}
	}
	
	// The oblique selector link is a wrapper around the links.
	// This provides a nice clean way of handling the click events for the thumbnails.
	function ObliqueSelectorLink(link, enabled)
	{
		var scene = null;
		
		this.Init = function()
		{
			link.title = L_ObliqueSelectorSelectThumbnail_Text;			
			
			if (enabled)
			{
				link.attachEvent("onclick", OnClick);
				link.attachEvent("onmouseover", OnMouseOver);
				link.attachEvent("onmouseout", OnMouseOut);
			}
		};
		
		this.Destroy = function()
		{
			if (enabled)
			{
				link.detachEvent("onclick", OnClick);
				link.detachEvent("onmouseover", OnMouseOver);
				link.detachEvent("onmouseout", OnMouseOut);
			}
		};
		
		this.SetScene = function(s)
		{
			scene = s;
			
			if (scene)
			{
				link.innerHTML = "<img src=\"" + scene.GetThumbnailFilename() + "\" width=48 height=48 border=0>";
				
				if (enabled)
			    {
					link.style.cursor = "pointer";
			    }
			}
			else
			{
				link.innerHTML = "";
				link.style.cursor = "default";
			}
			
			if (enabled)
			{
				link.parentNode.className = "obliqueSelectorCellOff";
			}
		};
		
		function OnClick()
		{
			if (!scene)
			{
				return;
			}
			
			map.SetObliqueScene(scene.GetID());
		}
		
		function OnMouseOver(e)
		{
			link.parentNode.className = "obliqueSelectorCellOn";
		}
		
		function OnMouseOut(e)
		{
			link.parentNode.className = "obliqueSelectorCellOff";
		}
	}
	
	this.Update = Update;
	this.Show = Show;
	this.Hide = Hide;
}
