//------------------------------------------------------------------------------
// MapControl/Dashboard/OrthoZoomBar.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// OrthoZoomBar constructor, creates the zoom widget at the specified
// location and starting at the specified zoom level
function OrthoZoomBar(parent)
{
	var container = document.createElement("div");
    var plus = document.createElement("div");
    var bar = document.createElement("div");
    var slider = document.createElement("div");
	var minus = document.createElement("div");
	
	var x = 0, y = 0;
	var dragging = false;
	
	this.Init = function()
	{
		minus.className = "ZoomBar_minus";
		minus.unselectable = "on";
		minus.attachEvent("onclick", Minus);
		
		slider.className = "ZoomBar_slider";
		slider.unselectable = "on";
		slider.attachEvent("onmousedown", MouseDown);
		slider.attachEvent("onmousemove", MouseMove);
		slider.attachEvent("onmouseup", MouseUp);
		slider.attachEvent("onclick", IgnoreEvent);

		bar.className = "ZoomBar_bar";
		bar.unselectable = "on";
		bar.appendChild(slider);
		bar.attachEvent("onclick", Jump);
		
		plus.className = "ZoomBar_plus";
		plus.unselectable = "on";
		plus.attachEvent("onclick", Plus);

		container.className = "ZoomBar";
		container.appendChild(minus);
		container.appendChild(bar);
		container.appendChild(plus);
		container.attachEvent("onmousedown", IgnoreEvent);
		container.attachEvent("onmouseup", IgnoreEvent);
		container.attachEvent("onclick", IgnoreEvent);
		container.attachEvent("ondblclick", IgnoreEvent);
				
		parent.appendChild(container);
		
		UpdateFromMap();
	};
	
	this.Destroy = function()
	{
		minus.detachEvent("onclick", Minus);
		
		slider.detachEvent("onmousedown", MouseDown);
		slider.detachEvent("onmousemove", MouseMove);
		slider.detachEvent("onmouseup", MouseUp);
		slider.detachEvent("onclick", IgnoreEvent);
		
		bar.removeChild(slider);
		bar.detachEvent("onclick", Jump);
		
		plus.detachEvent("onclick", Plus);
		
		container.removeChild(plus);
		container.removeChild(bar);
		container.removeChild(minus);
		container.detachEvent("onmousedown", IgnoreEvent);
		container.detachEvent("onmousedown", IgnoreEvent);
		container.detachEvent("onclick", IgnoreEvent);
		container.detachEvent("ondblclick", IgnoreEvent);
		
		parent.removeChild(container);
		
		map.DetachEvent("onendzoom", UpdateFromMap);
		
		minus = slider = bar = plus = container = parent = map = null;
	};
	
	function Show()
	{
		container.style.display = "block";
	}
	
	function Hide()
	{
		container.style.display = "none";
	}
	
	// Updates the position tracking variables so the control can perform
	// calculations based on positions.
	function UpdatePosition()
	{
		x = GetLeftPosition(container);
		y = GetTopPosition(container);
	}
	
	// MouseDown handler for the slider control.
	function MouseDown(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
		
		UpdatePosition();
		
		if (slider.setCapture)
		{
			slider.setCapture();
		}
		
		dragging = true;
		return false;
	}
	
	// MouseMove handler for the slider control.  If dragging, 
	// moves the slider around as appropriate.
	function MouseMove(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
	    
		if (dragging)
		{
			slider.style.left = AdjustMouseXbyOffsets(GetMouseX(e)) + "px";
		}
		
		return false;
	}

	// MouseUp handler for the slider control.  Gets the x
	// position and attempts to set the best zoom level.
	function MouseUp(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
    
		if (slider.releaseCapture)
		{
			slider.releaseCapture();
		}
		
		dragging = false;
		SetPosition(AdjustMouseXbyOffsets(GetMouseX(e)));
		orthoZoom.UpdateFromMap();
		obliqueZoom.UpdateFromMap();
		return false;
	}
	
	// OnClick handler for the "plus" button, which zooms in.  Note that this
	// does not actually adjust the position of the slider.  That happens
	// in response to the zoom level changing.
	function Plus(e)
	{
		map.ZoomIn();
	}
	
	// OnClick handler for the "minus" button, which zooms out.  Note that this
	// does not actually adjust the position of the slider.  That happens
	// in response to the zoom level changing.
	function Minus(e)
	{
		map.ZoomOut();
	}
	
	// OnClick handler for the bar beneath the slider.  If clicked,
	// attempts to jump to the best zoom level for that location.
	function Jump(e)
	{
		e = GetEvent(e);
		CancelEvent(e);
		UpdatePosition();
		SetPosition(AdjustMouseXbyOffsets(GetMouseX(e)));
		return false;
	}
	
	function AdjustMouseXbyOffsets(mouseX)
	{
		mouseX -= x + minus.offsetWidth + slider.offsetWidth;
		var max = bar.offsetWidth - slider.offsetWidth;
	            
		if(mouseX < 0)
		{
			mouseX = 0;
		}
		else if(mouseX > max) 
		{
			mouseX = max;
		}
	    
		return mouseX;
	}
	
	// Finds the best zoom level for an x-location.  Note that this
	// does not actually adjust the position of the slider.  That happens
	// in response to the zoom level changing.
	function SetPosition(pos)
	{
		var max = bar.offsetWidth - slider.offsetWidth;
		var zoom = 1 + MathRound(pos / max * 18);
		map.SetZoom(zoom);
	}
	
	// Updates the slider position based on the map's current zoom level
	function UpdateFromMap()
	{
		var max = bar.offsetWidth - slider.offsetWidth;
		var pos = (map.GetZoomLevel() - 1) / 18 * max;
		slider.style.left = pos + "px";
	}
	
	this.UpdateFromMap = UpdateFromMap;
	this.Show = Show;
	this.Hide = Hide;
}
