//------------------------------------------------------------------------------
// LatLongEncoding.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: keithkin
//
// <header>	$LatLongEncoding$</header>
// Dependencies:
//   NumberEncoding.js
//------------------------------------------------------------------------------

var L_velatlongencodinginvalidstringlength_text = "VELatLongEncoding: Invalid string length";

//VELatLongEncoding
//   Encodes/Decodes latitude/longitude coordinates to a hash string
//   The algorithm is from BBeatty.
//
//   interface:
//      VELatLongEncoding()
//         constructor
//
//      Encode(latitude, longitude)
//         takes a float latitude and float longitude and returns encoded string
//         throws exception if latitude or longitude is out of range
//
//      Decode(str)
//         takes an encoded string and returns decoded latitude and longitude parameters in an array
//         the first element in the array is the latitude.
//         the second element in the array is the longitude.
//         throws exception if string is invalid length
//         throws exception if there is an invalid digit in the encoding
//
function VELatLongEncoding()
{
	var m_minLatitude = -90;
	var m_maxLatitude = 90;
	var m_minLongitude = -180;
	var m_maxLongitude = 180;
	var m_digits = "0123456789bcdfghjkmnpqrstvwxyz";
	var m_valueLength = 6;	//value length of the string encoding
	
	var m_integerEncoding = new VEIntegerEncoding(m_digits, m_valueLength);
	var m_latitudeMap = new VEFloatIntegerMap(m_minLatitude, m_maxLatitude, m_integerEncoding.MaxValue());
	var m_longitudeMap = new VEFloatIntegerMap(m_minLongitude, m_maxLongitude, m_integerEncoding.MaxValue());
	
	//Encode
	this.Encode = function(latitude, longitude)
	{
		var s = m_integerEncoding.Encode(m_latitudeMap.FloatToInt(latitude)) + 
				m_integerEncoding.Encode(m_longitudeMap.FloatToInt(longitude)) ;
		return s;
	}
	
	//Decode
	this.Decode = function(str)
	{
		if (str.length == 2 * m_integerEncoding.ValueLength())
		{
			var valueLength = m_integerEncoding.ValueLength();
			var strLatitude = str.substr(0, valueLength);
			var strLongitude = str.substr(valueLength, valueLength);
			var intLatitude = m_integerEncoding.Decode(strLatitude);
			var intLongitude = m_integerEncoding.Decode(strLongitude);
			var latLongArray  =new Array();
			latLongArray[0]= m_latitudeMap.IntToFloat(intLatitude);
			latLongArray[1]= m_longitudeMap.IntToFloat(intLongitude);
			return latLongArray;
		}
		else
		{
			//invalid string length exception
			throw L_velatlongencodinginvalidstringlength_text;
		}
	}
}
