//------------------------------------------------------------------------------
// MapControl/Pan.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


// Pans the map by pixels
function PanMap(deltaX, deltaY)
{
	if (deltaX == 0 && deltaY == 0)
	{
		return;
	}
	
	// Shift the center point
	ShiftView(currentView, currentBounds, deltaX, deltaY);

	// Shift the offsets
	offsetX = currentView.center.x - originX - width/2;
	offsetY = currentView.center.y - originY - height/2;
	
	// Shift the container element
	map.style.top = -offsetY + "px"; 
	map.style.left = -offsetX + "px";
	
	// Update the copyright
	copyright.Update();
	
	// Shift the tiles
	window.setTimeout(UpdateView, 1);
	window.setTimeout(UpdateLines, 2);
}

// Starts a continuous pan by pixels.
// If count is greater than 0, it represents the number of times the pan executes.
// If count is less than or equal to 0 (or absent), the control will pan indefinitely.
// To stop the continuous pan, call StopContinuousPan.
function ContinuousPan(deltaX, deltaY, count, keyboard)
{
	// Do not continuous pan if zooming
	if (zooming)
	{
		return;
	}

	// If the count parameter was not passed or equal to 0, it's the same as no count at all.
	if (!count)
	{
		count = -1;
	}

	// Set the deltas & the pan counter.
	panningX = deltaX;
	panningY = deltaY;
	panCounter = count;
		
	// If the deltas are both 0, stop panning
	if (!deltaX && !deltaY)
	{
		StopContinuousPan();
		return;
	}
	
	// Note if this is panning caused by a keyboard.
	keyboardPan = keyboard;
		
	// If not already pannning, start the panning thread
	if (!panning)
	{		
		panning = true;
		StepPan();
		Fire("onstartcontinuouspan");
	}
}

// Internal implementation of the continuous pan.  Not designed to be called externally.
function StepPan()
{	
	// If the control is panning
	if (panning)
	{
		// Shift the map
		PanMap(panningX, panningY);
		
		// If a finite number of steps, decrement the counter
		if (panCounter > 0)
			panCounter--;
		
		// If the counter has more steps OR is ignored, continue panning
		if (panCounter != 0)
			window.setTimeout(StepPan, 10);
		else
			StopContinuousPan();
	}
}

// Stops a continuous pan
function StopContinuousPan()
{
	panningX = 0;
	panningY = 0;
	panning = false;
	keyboardPan = false;
	
	if (panLatitude != null && panLongitude != null)
	{		
		// Compute the overall deltas
		var latlong = new Msn.VE.LatLong(panLatitude, panLongitude);
		var pixel = currentMode.LatLongToPixel(latlong, currentView.zoomLevel);
		var dx = pixel.x - (originX + offsetX + width/2);
		var dy = pixel.y - (originY + offsetY + height/2);
		PanMap(dx, dy);
		
		currentView.latlong.latitude = panLatitude;
		currentView.latlong.longitude = panLongitude;
		
		preferredView.Copy(currentView);
		
		panLatitude = null;
		panLongitude = null;
		
		// Update oblique availability.  Do this here, but not in the general case
		// because ComputeCenterPoint automatically calls obliqueMode.UpdateAvailability.
		if (obliqueMode)
		{
			obliqueMode.UpdateAvailability();
		}
	}
	else
	{
		ComputeCenterPoint(true);		
	}
		
	// Notify a view listener
	Fire("onendcontinuouspan");
	Fire("onchangeview");
}

// Scrolls to a lat/long
// Implemented as a simple wrapper around ContinuousPan
function PanToLatLong(latitude, longitude)
{
	panLatitude = latitude;
	panLongitude = longitude;
	PanToPixel(currentMode.LatLongToPixel(new Msn.VE.LatLong(latitude, longitude), currentView.zoomLevel));
}

function PanToView(view)
{
	PanToPixel(view.center);
}

function PanToPixel(pixel)
{
	// If we're in oblique mode, and this point isn't in the current scene
	if (currentView.mapStyle == obliqueStyle)
	{
		var scene = obliqueMode.GetScene();
		if (!scene || !scene.ContainsPixel(pixel, currentView.zoomLevel))
		{
			var view = preferredView.MakeCopy();
			view.sceneId = null;
			view.SetCenterLatLong(obliqueMode.PixelToLatLong(pixel, currentView.zoomLevel));
			SetView(view);
			return;
		}
	}

	// Compute the overall deltas
	var dx = pixel.x - (originX + offsetX + width/2);
	var dy = pixel.y - (originY + offsetY + height/2);

	// Compute the overall distance
	var distance = Math.sqrt(dx * dx + dy * dy);

	// Jump to pan to point if the distance is too far or animated movements are disabled.
	if (!animatedMovementEnabled ||
		MathAbs(dx) > 2 * width || 
		MathAbs(dy) > 2 * height || 
		distance > 1.5 * Math.sqrt(width * width + height * height))
	{
		var view = preferredView.MakeCopy();
		view.SetCenter(pixel);
		SetView(view);
		return;
	}
	
	// Compute the direction of the scroll
	var angle = Math.atan2(dy, dx);
	
	// Compute the necessary number of steps at the "suggested" speed
	var count = MathCeil(distance / panToLatLongSpeed);
	
	// Compute the actual speed for an even number of steps
	var speed = MathRound(distance / count);
	
	// Compute the sized deltas
	dx = MathRound(Math.cos(angle) * speed);
	dy = MathRound(Math.sin(angle) * speed);
	
	// Start the pan
	ContinuousPan(dx, dy, count);
}
