//------------------------------------------------------------------------------
// ScaleBar.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye, sstchur
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

function ScaleBar(parent)
{
	// x offset from right border
	var xOffset = 0;

	// y offset from bottom border
	var yOffset = 0;

	var labelBg = document.createElement("div");
	var labelFg = document.createElement("div");
	var barBg = document.createElement("div");
	var barFg = document.createElement("div");

	var scaleBarWidth = 150; // width of the bar (not the control, just the bar)
	
	this.Init = function()
	{
		labelBg.className = "ScaleBarLabel ScaleBarLabelBackground";
		labelFg.className = "ScaleBarLabel ScaleBarLabelForeground";
		
		barBg.className = "ScaleBar ScaleBarBackground";
		barFg.className = "ScaleBar ScaleBarForeground";
	
		Update();
		Reposition();
		
		parent.appendChild(labelBg);
		parent.appendChild(labelFg);
		parent.appendChild(barBg);
		parent.appendChild(barFg);
	};
	
	this.Destroy = function()
	{
		parent.removeChild(labelBg);
		parent.removeChild(labelFg);
		parent.removeChild(barBg);
		parent.removeChild(barFg);
		labelBg = labelFg = barBg = BarFg = null;
	};

	// Reposition the copyright
	function Reposition()
	{
		labelBg.style.top = (height - 45 - yOffset) + "px";
		labelBg.style.right = '10px';
		labelBg.style.display = "block";
		
		labelFg.style.top = (height - 46 - yOffset) + "px";
		labelFg.style.right = '11px';
		labelFg.style.display = "block";
		
		barBg.style.top = (height - 30 - yOffset) + "px";
		barBg.style.right = '9px';
		barBg.style.display = "block";
		
		barFg.style.top = (height - 31 - yOffset) + "px";
		barFg.style.right = '10px';
		barFg.style.display = "block";

	}

	// Unit conversion methods
	function MetersToKilometers(meters)
	{
		return meters * 0.001;
	}
	
	function MetersToMiles(meters)
	{
		return meters * 0.000621371192;
	}
	
	function MetersToYards(meters)
	{
		return meters * 1.0936133;
	}

	// Updates the map scale bar
	function Update()
	{
		try
		{
		
			var mpp = GetMetersPerPixel();
			var maxMeters = mpp * scaleBarWidth;

			if ( Msn.VE.Marketization.IsEnabled( MapControl.Features.ScaleBarKilometers ) ) 
			{
				// Find and display the most appropriate metric label			
				var units = L_ScaleBarKilometers_Text;
				var dMeasure = MetersToKilometers(maxMeters);
				var niceValue = LargestNiceNumber(dMeasure);
				if(niceValue < 0.5)
				{
				
					units = L_ScaleBarMeters_Text;
					dMeasure = maxMeters;
					niceValue = LargestNiceNumber(dMeasure);					
				}
				//TODO: <Localization> : Need to set the first parameter value based on Locales
				ChangeMapScaleValues("metric", units, niceValue, Math.round(niceValue/dMeasure*scaleBarWidth));			
			}
			else
			{
				// Find and display the most appropriate US label
				var units = L_ScaleBarMiles_Text;
				var dMeasure = MetersToMiles(maxMeters);
				var niceValue = LargestNiceNumber(dMeasure);
				if (niceValue < 0.5)
				{
					units = L_ScaleBarYards_Text;
					dMeasure = MetersToYards(maxMeters);
					niceValue = LargestNiceNumber(dMeasure);
				}
				//TODO: <Localization> : Need to set the first parameter value based on Locales
			ChangeMapScaleValues("us", units, niceValue, Math.round(niceValue/dMeasure*scaleBarWidth));
			}
		}
		catch(ex)
		{
			// Problems can occasionally occur with the scale bar.  Particularly at startup
			// when not all mode values and view values have been populated.
			// If something goes wrong, fail silently.  The scale bar is updated frequently, 
			// and will fix itself at the next update request.
		}
	}


	// Computes the largest nice number smaller than the input number
	function LargestNiceNumber(dIn)
	{
		//Calculate what power of 10 this number is raised to.
		var dLog10 = Math.log(dIn) / Math.log(10);
		var dExponent = Math.floor( dLog10 );
		var dExponentValue = Math.pow(10, dExponent);

		//Get the remainder after removing the exponent part.
		var dRoot = dIn / dExponentValue;

		//If first digit is 3 or more, just use that first digit followed by zeros.
		var dIntegerRoot = Math.floor(dRoot);
		if (dIntegerRoot >= 3.0)
		{
			return dIntegerRoot * dExponentValue;
		}

		//Otherwise reduce to the nearest multiple of 0.5, i.e. you get 1.0, 1.5, 2.0 or 2.5
		var dNiceRoot = Math.floor(dRoot * 2.0) * 0.5;
		return dNiceRoot * dExponentValue;
	}


	//TODO: <Localization> : Need to use the first parameter value (locale specific) in the function 
	// Change the actual values and sizes of the map scale bars
	function ChangeMapScaleValues(type, units, value, width)
	{
		if(value < 1)
		{
			value = value.toFixed(1);
		}
		
		var text = value + " " + units;
		labelBg.innerHTML = text;
		labelFg.innerHTML = text;
		
		barBg.style.width = width;
		barFg.style.width = width;
	}
	
	function SetOffset(_xOffset, _yOffset)
	{
		xOffset = _xOffset;
		yOffset = _yOffset;
		Reposition();
	}
	
	this.Update = Update;
	this.Reposition = Reposition;
	this.SetOffset = SetOffset;
}