//------------------------------------------------------------------------------
// MapControl/View.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, a-jayab, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Starts thet map tiles for the current settings.
function StartView()
{
	// Clear the existing tiles
	ClearTiles(currentTilesList);
	
	// Reposition the map
	map.style.top = "0px";
	map.style.left = "0px";

	// Lock down a new origin point
	// This prevents x/y numbers from getting out-of-control large
	originX = MathRound(currentView.center.x - width/2);
	originY = MathRound(currentView.center.y - height/2);
	
	// Offsets determine how many pixels the users viewport is from the origin
	offsetX = 0;
	offsetY = 0;
	
	tileViewportX1 = MathFloor((originX - buffer) / tileSize);
	tileViewportY1 = MathFloor((originY - buffer) / tileSize);
	tileViewportX2 = MathFloor((originX + width + buffer) / tileSize);
	tileViewportY2 = MathFloor((originY + height + buffer) / tileSize);
	
	tileViewportWidth = tileViewportX2 - tileViewportX1 + 1;
	tileViewportHeight = tileViewportY2 - tileViewportY1 + 1;
	
	for (var y=tileViewportY1; y<=tileViewportY2; y++)
	{
		for (var x=tileViewportX1; x<=tileViewportX2; x++)
		{
			//Original tile
			var tile = RequestTile(x, y, currentView.zoomLevel, currentView.mapStyle, 0);
			currentTilesList.push(tile);
		}
	}
}

// Updates the tiles on the map, trimming off tiles that aren't necessary and adding in new tiles.
// The basic flow is to first figure out the boundaries in tile indexes.  Then extend or retract
// each of the 4 edges accordingly.  
function UpdateView()
{
	// No updates during zoom.  UpdateView is really only applicable during panning.
	if (zooming)
	{
		return;
	}
	
	var ox = originX + offsetX;
	var oy = originY + offsetY;

	var x1 = MathFloor((ox - buffer) / tileSize);
	var y1 = MathFloor((oy - buffer) / tileSize);
	var x2 = MathFloor((ox + width + buffer) / tileSize);
	var y2 = MathFloor((oy + height + buffer) / tileSize);
	
	var X1 = tileViewportX1;
    var X2 = tileViewportX2;
    var Y1 = tileViewportY1;
    var Y2 = tileViewportY2;
    var W1 = tileViewportWidth;
    var H1 = tileViewportHeight;
		
	// While the map is too far to the left
	while (tileViewportX1 < x1)
	{
		// Remove each of the tiles in the left column
		// Start from the bottom and work backwards so the tiles on top retain their correct indexes
		for (var y=tileViewportHeight-1; y>=0; y--)
		{
			var tile = currentTilesList.splice(y * tileViewportWidth, 1)[0];
			try
			{
				tile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}
	
			//Traffic
			if(trafficAvailable && trafficAvailable == true)
			{					
				var ttile = currentTrafficTilesList.splice(y * tileViewportWidth, 1)[0];
				try
				{
					ttile.RemoveFromMap();
				}
				catch(ex)
				{
					//swallow
				}
			}

		}
		
		tileViewportX1++;
		tileViewportWidth--;
	}
	
	// While the map isn't far enough to the left
	while (tileViewportX1 > x1)
	{
		// Add tiles to a new left column
		tileViewportX1--;
		tileViewportWidth++;
		
		for (var y=0; y<tileViewportHeight; y++)
		{
			var tile = RequestTile(tileViewportX1, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 0);
			currentTilesList.splice(y * tileViewportWidth, 0, tile);

			//Traffic stuff			
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = RequestTile(tileViewportX1, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 1);
				currentTrafficTilesList.splice(y * tileViewportWidth, 0, ttile);
			}
		}
	}
	
	// While the map is too far to the top
	while (tileViewportY1 < y1)
	{
		// Remove the top row
		for (var x=0; x<tileViewportWidth; x++)
		{
			var tile = currentTilesList.shift();
			try
			{
				tile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}

			//Traffic stuff						
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = currentTrafficTilesList.shift();
				try
				{
					ttile.RemoveFromMap();
				}
				catch(ex)
				{
					//swallow
				}			
			}
		}		
		
		tileViewportY1++;
		tileViewportHeight--;
	}
	
	// While the map isn't far enough to the top
	while (tileViewportY1 > y1)
	{
		// Add a row to the top
		tileViewportY1--;
		tileViewportHeight++;
		
		// Start from the right side of the row so the tiles can be added to the front each time
		for (var x=tileViewportWidth-1; x>=0; x--)
		{
			var tile = RequestTile(tileViewportX1 + x, tileViewportY1, currentView.zoomLevel, currentView.mapStyle, 0);
			currentTilesList.unshift(tile);			

			//Traffic stuff			
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = RequestTile(tileViewportX1 + x, tileViewportY1, currentView.zoomLevel, currentView.mapStyle, 1);
				currentTrafficTilesList.unshift(ttile);
			}
		}
	}
	
	// While the map is too far to the right
	while (tileViewportX2 > x2)
	{
		// Remove each of the tiles in the right column
		// Start from the bottom and work backwards so the tiles on top retain their correct indexes
		for (var y=tileViewportHeight-1; y>=0; y--)
		{
			var tile = currentTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 1)[0];
			try
			{
				tile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}

			//Traffic			
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = currentTrafficTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 1)[0];
				try
				{
					ttile.RemoveFromMap();
				}
				catch(ex)
				{
					//swallow
				}
			}
			
		}
		
		tileViewportX2--;
		tileViewportWidth--;
	}
	
	// While the map isn't far enough to the right
	while (tileViewportX2 < x2)
	{
		// Add tiles to a new right column
		tileViewportX2++;
		tileViewportWidth++;
		
		for (var y=0; y<tileViewportHeight; y++)
		{
			var tile = RequestTile(tileViewportX2, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 0);					
			currentTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 0, tile);

			//Traffic stuff			
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = RequestTile(tileViewportX2, tileViewportY1 + y, currentView.zoomLevel, currentView.mapStyle, 1);		
				currentTrafficTilesList.splice(y * tileViewportWidth + tileViewportWidth - 1, 0, ttile);
			}			
		}		
	}
	
	// While the map is too far down
	while (tileViewportY2 > y2)
	{
		// Remove the bottom row
		for (var x=0; x<tileViewportWidth; x++)
		{
			var tile = currentTilesList.pop();
			try
			{
				tile.RemoveFromMap();
			}
			catch(ex)
			{
				//swallow
			}

			//Traffic stuff			
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = currentTrafficTilesList.pop();
				try
				{
					ttile.RemoveFromMap();
				}
				catch(ex)
				{
					//swallow
				}
			}
			
		}
		
		tileViewportY2--;
		tileViewportHeight--;
	}
	
	// While the map isn't far down enough
	while (tileViewportY2 < y2)
	{
		// Add a row to the bottom
		tileViewportY2++;
		tileViewportHeight++;
		
		// Start from the right side of the row so the tiles can be added to the front each time
		for (var x=0; x<tileViewportWidth; x++)
		{
			var tile = RequestTile(tileViewportX1 + x, tileViewportY2, currentView.zoomLevel, currentView.mapStyle, 0);
			currentTilesList.push(tile);
			
			//Traffic stuff
			if(trafficAvailable && trafficAvailable == true)
			{								
				var ttile = RequestTile(tileViewportX1 + x, tileViewportY2, currentView.zoomLevel, currentView.mapStyle, 1);
				currentTrafficTilesList.push(ttile);			
			}
		}
	}		
	
	//Cthota
	//Moving to inline for perf reasons
	//BTW, moving to async does not work
	//window.setTimeout(UpdateLayer, 1);
}

// Change the zoom level
// If the requested zoom is within 1 zoom level of the current and animated 
// zoom is enabled, this will start a thread to do the animation.
// Otherwise, this will just swap in new tiles for old tiles
function SetView(requestedView)
{
	// If already zooming or dragging, ignore it.  They can try again when it is done.
	if (zooming || dragging)
	{
		return true;
	}
	
	//If the map is panning, go ahead and stop it. This allows other view changes
	//to be accepted during the pan. i.e. if the user wants to change the zoom level,
	//then we will jump to the end of the pan and zoom in. This also allows consecutive 
	//API calls to work properly if one of the calls starts a pan.
	if(panning)
	{
		StopContinuousPan();
	}
	
	// If the requested view came in as a latlong rectangle, we have to take special care
	// to preserve the true center.  In the process of resolving it to a pixel based view,
	// the true center point can be lost, especially in oblique mode.
	var centerLatLong = requestedView.latlong;
	if (requestedView.GetViewType() == "latlongRect")
	{
		centerLatLong = requestedView.latlongRect.Center();
	}
		
	// Have to handle some special cases for oblique mode requests
	if (requestedView.mapStyle == obliqueStyle)
	{
		// If we don't have oblique capabilities, ignore this request.
		if (obliqueMode == null)
		{
			return true;
		}
		
		// First capture the unresolved requested view.  If we make it through all the oblique
		// checks, we'll capture the resolved view again.
		preferredView.Copy(requestedView);
		
		currentMode = obliqueMode;
		p_elSource.style.backgroundColor = "black";
		requestedView.Resolve(currentMode, width, height);
	
		// If we don't have a scene, then we need to make a server request to 
		// check for availability
		// If the requested scene is not the same as our current scene
		// If the user explicitly requested a scene by id, and this is not that scene,
		// then go back to the server and find the requested scene.
		var scene = obliqueMode.GetScene();
		if ((!scene || scene.GetID() != requestedView.sceneId) && requestedView.sceneId != null)
		{
			obliqueMode.RequestScene(requestedView.sceneId);
			return true;
		}
			
		// If this scene does not actually contain the requested point,
		// then go back to the server and look for a scene at the point
		if (!scene || 
			(!scene.ContainsLatLong(centerLatLong, requestedView.zoomLevel) && scene.GetID() != requestedView.sceneId) ||
			scene.GetOrientation() != requestedView.sceneOrientation)
		{
			obliqueMode.RequestSceneAtLatLong(centerLatLong, requestedView.sceneOrientation, true);
			return true;
		}
	}
	else
	{
		currentMode = orthoMode;
		if (requestedView.mapStyle == 'r')
		{
			p_elSource.style.backgroundColor = "#e9e7d4";
		}
		else
		{
			p_elSource.style.backgroundColor = "black";
		}
		requestedView.Resolve(currentMode, width, height);
	}
	
	// Now grab the resolved center latlong
	centerLatLong = requestedView.latlong;
	
	// Capture the preferred center point
	preferredView.Copy(requestedView);
	
	// Validate the zoom level
	currentMode.ValidateZoomLevel(requestedView);
	
	// Get the current bounds
	currentBounds = currentMode.GetBounds(requestedView);
	
	// Clip the view, make sure it's valid
	ClipView(requestedView, currentBounds);
	
	// Resolve the view again to update the latlong
	requestedView.Resolve(currentMode, width, height);
	
	// If the views are the same thing, ignore the request.
	if (requestedView.Equals(currentView))
	{
		return false;
	}

	// Compute the distance of the transition
	var deltaX = requestedView.GetX(currentView.zoomLevel) - currentView.center.x;
	var deltaY = requestedView.GetY(currentView.zoomLevel) - currentView.center.y;
	var distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
	
	// To determine if animated pan is possible, consider these checks:
	// 1. It is within viewing distance
	// 2. It is at the same zoom level
	// 3. Dynamic movements are enabled
	// 4. Same map style
	// 5. Same oblique scene (or not in oblique at all)
	var animatedPan =
		(distance < width && distance < height) && 
		(requestedView.zoomLevel == currentView.zoomLevel) &&
		animatedMovementEnabled &&
		(requestedView.mapStyle == currentView.mapStyle) &&
		(requestedView.sceneId == null || requestedView.sceneId == currentView.sceneId);
	
	// If it passed the checks, do it.
	if (animatedPan)
	{
		PanToLatLong(centerLatLong.latitude, centerLatLong.longitude);
		return true;
	}
	
	// If the zoom level is changing, capture the current zoom and fire the event.
	previousZoomLevel = currentView.zoomLevel;
	if (currentView.zoomLevel != requestedView.zoomLevel)
	{
		Fire("onstartzoom");
		zooming = true;
	}
	
	// To determine if animated zoom is possible, consider these checks:
	// 1. It is within viewing distance
	// 2. It is within one zoom level away and not the current zoom level
	// 3. Dynamic movements are enabled
	// 4. Same map style
	// 5. Same oblique scene (or not in oblique at all)
	var animatedZoom = 
		(distance < width && distance < height) && 
		(requestedView.zoomLevel == currentView.zoomLevel-1 || requestedView.zoomLevel == currentView.zoomLevel+1) &&
		animatedMovementEnabled &&
		(requestedView.mapStyle == currentView.mapStyle) &&
		(requestedView.sceneId == null || requestedView.sceneId == currentView.sceneId);
	
	// If it passed all the checks, do it.
	if (animatedZoom)
	{
		// Track the old origin.  It must be reset to the top corner of the screen so that the 
		// map container can be set at left=0px, top=0px
		var oldOriginX = originX + offsetX;
		var oldOriginY = originY + offsetY;
		var oldZoom = currentView.zoomLevel;
		var newZoom = requestedView.zoomLevel;
		
		// New origin
		var newOriginX = MathRound(requestedView.center.x - width/2);
		var newOriginY = MathRound(requestedView.center.y - height/2);
		
		// Keep the old tiles in a separate bucket
		// Create a new array for new tiles
		oldTilesList = currentTilesList;
		currentTilesList = new Array();
			
		for (var i=0; i<oldTilesList.length; i++)
		{
			oldTilesList[i].PrepareBaseTile(oldOriginX, oldOriginY, oldZoom, newOriginX, newOriginY, newZoom);
		}
		
		for(var i=0; i<pushpins.length; i++)
		{
			pushpins[i].PrepareForZoom(newOriginX, newOriginY, newZoom);
		}
		
		HideLines();
		currentView.Destroy();
		currentView = requestedView;
		StartView();
		SetLayerView();				
		for (var i=0; i<currentTilesList.length; i++)
		{
			currentTilesList[i].PrepareSwapTile(oldOriginX, oldOriginY, oldZoom, newOriginX, newOriginY, newZoom);
		}

		//cthota: traffic stuff
		//Removing swap layer call w/ inline call due to perf reasons
		if(trafficAvailable && trafficAvailable == true)		
		{
			for(var i=0; i<currentTrafficTilesList.length; i++)
			{
				currentTrafficTilesList[i].PrepareSwapTile(oldOriginX, oldOriginY, oldZoom, newOriginX, newOriginY, newZoom);
			}
		}
				
		zoomCounter = 1;
		StepAnimation();
		return true;
	}

	// Otherwise, if not doing a dynamic zoom, do the standard jump zoom
	oldTilesList = currentTilesList;
	currentTilesList = new Array();
	currentView.Destroy();
	currentView = requestedView;
	HideLines();
	StartView();
	SetLayerView();				
	FinalizeView();
	RepositionPushpins();
	return true;	
}

// Steps the animation
// This is a static member because timer callbacks cannot be member functions.
function StepAnimation()
{
	// If we're not zooming, get out
	if (!zooming)
	{
		return;
	}

	for (var i=0; i<oldTilesList.length; i++)
	{
		oldTilesList[i].SetFactor(zoomCounter);
	}
	
	for (var i=0; i<currentTilesList.length; i++)
	{
		currentTilesList[i].SetFactor(zoomCounter);
	}
	
	for (var i=0; i<pushpins.length; i++)
	{
		pushpins[i].SetFactor(zoomCounter);
	}
		
	if (zoomCounter < zoomTotalSteps)
	{
		zoomCounter++;
		window.setTimeout(StepAnimation, 1);
	}
	else
	{
		zoomCounter = 0;
		FinalizeView();
	}
}

// Finish the zoom.  Clean up the old tiles. 
function FinalizeView()
{
	// Remove the old tiles
	if(oldTilesList!= null)
	{
		ClearTiles(oldTilesList);
		oldTilesList = null;
	}
	
	//Any other layers - including traffic
	ClearLayer();
	
	// End zooming state
	zooming = false;
	
	// Set the final transition to be the starting point
	for(var i=0; i<currentTilesList.length; i++)
	{
		currentTilesList[i].SwapStates();
		currentTilesList[i].ClearSteps();
		currentTilesList[i].SetFactor(0);

		//Traffic stuff
		if(trafficAvailable && trafficAvailable == true)				
		{
			if(currentTrafficTilesList[i] != null)
			{
				currentTrafficTilesList[i].SwapStates();
				currentTrafficTilesList[i].ClearSteps();			
				currentTrafficTilesList[i].SetFactor(0);
			}
		}
	}
	
	//cthota - moved to inline due to perf
	//tried async but some visual wierdness 
	//shows up due to delayed tile swapping
	//SetLayer();
	
	for(var i=0; i<pushpins.length; i++)
	{
		pushpins[i].SwapStates();
		pushpins[i].ClearSteps();
		pushpins[i].SetFactor(0);
	}
	
	currentTilesIndexes = null;
	oldTilesIndexes = null;
	
	// Delay the line by a split second
	// Give the tiles a chance to load first
	// Otherwise, there's an ugly period of time where
	// the tiles are half baked
	window.setTimeout(ShowLines, 250);
	
	// Update the copyright
	if (copyright)
	{
		copyright.Update();
	}
	
	// Notify the map style listeners
	if (previousMapStyle != currentView.mapStyle)
	{
		Fire("onchangemapstyle");
	}
	
	// Notify the zoom level listeners
	if (previousZoomLevel != currentView.zoomLevel)
	{
		Fire("onendzoom");
	}
	
	// Notify a view listener
	Fire("onchangeview");
	
	// Update oblique availability
	if (obliqueMode)
	{
		window.setTimeout(obliqueMode.UpdateAvailability, 100);
	}
	
	// In IE we can force a garbage collect
	try
	{
		CollectGarbage();
	}
	catch (ex)
	{
		// Ignore exception if browsers other than IE complain
	}
}
