//------------------------------------------------------------------------------
// MapControl/Viewport.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


// Finds the best viewport for the specified lat/long rectangle
function SetViewport(lat1, lon1, lat2, lon2)
{
	var view = preferredView.MakeCopy();
	view.sceneId = null;
	
	//Set the LatLong rectangle of the view.
	//Note: The latlongs have to be clipped in case they are outside the acceptable min/max range.
	//If the latlongs are not clipped, the lat/long to pixel conversions in Mercator projections 
	//can mess up calculations. For example, a latitude of 90 degrees that is converted to pixels 
	//approaches infinity, so 85 degrees is the max.
	view.SetLatLongRectangle(
		new Msn.VE.LatLongRectangle(
			new Msn.VE.LatLong(ClipLatitude(lat1), ClipLongitude(lon1)), 
			new Msn.VE.LatLong(ClipLatitude(lat2), ClipLongitude(lon2)) ));
	return SetView(view);
}

// Sets the map view for an array of lat/longs.
// l is an array of objects that have latitude and longitude properties
function SetBestMapView(l)
{
	if(!l || l.constructor != Array)
	{
		return;
	}

	var a = l[0].latitude;
	var b = l[0].longitude;
	var c = a;
	var d = b;
	
	for (var i=1; i<l.length; i++)
	{
		a = MathMin(a, l[i].latitude);
		b = MathMin(b, l[i].longitude);
		c = MathMax(c, l[i].latitude);
		d = MathMax(d, l[i].longitude);
	}
	
	var dLat = (c - a) * 0.1;
	var dLon = (d - b) * 0.1;
	
	a -= dLat;
	b -= dLon;
	c += dLat;
	d += dLon;
		
	SetViewport(
		ClipLatitude(a), 
		ClipLongitude(b), 
		ClipLatitude(c), 
		ClipLongitude(d));
}


// Grows the viewport to a best map view of the current center point and the lat/long parameters
function IncludePointInViewport(latitude, longitude)
{
	var latlong = new Msn.VE.LatLong(latitude, longitude);
	
	// If the user is in oblique and the requested latlong is not in the current scene,
	// then we have to kick back out into ortho mode.
	if (currentView.mapStyle == obliqueStyle)
	{
		var scene = obliqueMode.GetScene();
		if (!scene || !scene.ContainsLatLong(latlong))
		{
			SetMapStyle(lastOrthoMapStyle);
		}
	}
	
	SetBestMapView(new Array(currentView.latlong, latlong));
}


/// Clips a latitude value, which forces it between the minimum and maximum
/// latitudes.
function ClipLatitude(latitude)
{
	return Clip(latitude, minLatitude, maxLatitude);
}


/// Clips a longitude value, which forces it between the minimum and maximum
/// longitudes.
function ClipLongitude(longitude)
{
	return Clip(longitude, minLongitude, maxLongitude);
}


/// Clips a value between two extremes.  If the value is less than the lower bound,
/// the lower bound is returned.  If the value is greater than the upper bound,
/// the upper bound is returned.  Otherwise, the value is returned untouched.
function Clip(n, minValue, maxValue)
{
	if(n < minValue)
	{
		return minValue;
	}
	if(n > maxValue)
	{
		return maxValue;
	}
	return n;
}

