//------------------------------------------------------------------------------
// Search/SearchManager.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, sstchur, a-jayab, a-pratin, cthota, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


// Constructor for the search manager.  Unused, but needed to declare search members.
function VE_SearchManager()
{
}


// The array of search objects
VE_SearchManager.searches = new Array();

// The running counter of search objects.  Each search is assigned an index
VE_SearchManager.searchIndex = 0;

// Represents whether the search manager is actively searching right now
VE_SearchManager.searching = false;

// The queue of requests that have yet to be executed
VE_SearchManager.queue = new Array();

// The one xmlHttp object.  It is shared by all search requests
VE_SearchManager.xmlHttp = null;

// The xmlHttp object used to retrieve combined search.
VE_SearchManager.combinedSearchXmlHttp  = null;

// The number of results to display per search
VE_SearchManager.displayCount = 10;

// The number of results in a batch
VE_SearchManager.requestCount = 20;

// The event handler for paging results
VE_SearchManager.onPageResults = null;

// The maximum number of searches open at one time
VE_SearchManager.maxSearches = 3;

// The colors for the searches
VE_SearchManager.colors = new Array('search3', 'search2', 'search1');

// The search page
VE_SearchManager.searchPage = "search.ashx";

// The ads page
VE_SearchManager.adsPage = "ads.ashx";

// The popup that is used for displaying information about an entity
VE_SearchManager.popup = null;

// The id of the timer used for delayed requests
VE_SearchManager.timerID = -1;

//To detect Search Complete
VE_SearchManager.onSearchComplete = null;


// Execute a new search with optional what and where parameters.
// If a what parameter is provided, a new search is created.
// If a where parameter is provided, all searches are updated.
VE_SearchManager.Search=function(what, where, category)
{
	// Track whether or not we really need to search
	var s = false;
	
	var categoryId = -1;
	if(category != null && category != "undefined" && category > 0)
	{
		categoryId = category;
	}
	
	if(what)
	{
		// Remove pipes, carrots, leading white space and trailing white space
		what = what.replace(/\|/g, ' ').replace(/\^/g, ' ').replace(/^\s+/g, "").replace(/\s+$/g, "");

		if (what != "")
		{
			var i = VE_SearchManager.IndexOf(what);
			if (i == -1 || categoryId != -1)
			{
				VE_SearchManager._AddSearch(what, "WHAT", categoryId);
				s = true;
			}
			else	
			{		
				VE_SearchManager._MoveToFront(i, true);			
			}
		}
	}
	
	if (where)
	{		
		VE_SearchManager._ResetPaging();
		VE_SearchManager._CancelAllRequests();
		s = true;
	}
	
	if (s)
	{
		// if there is a where component, re-execute all searches
		VE_SearchManager._DoSearch(where, where);		
	}
}


// Add an array of searches and execute all at once
VE_SearchManager.BulkSearch=function(terms)
{
	if(!terms || terms.length == 0)
	{
		return;
	}
	
	var nLast = terms.length - 1;
	if (terms[nLast])
	{
		var txtWhat = document.getElementById('what');		
		txtWhat.value = terms[nLast];		
		setActive(txtWhat);		
	}
	
	
	//Check for categories
	/*
	for(var i=0; i<terms.length; i++)
	{
		if(terms[i] == null)
			continue;
		var indx = terms[i].indexOf("^");
		if(indx > 0)
		{
			var arr = terms[i].split("^");
			alert(arr[0]);
			alert(arr[1]);
			VE_SearchManager._AddSearch(arr[0], ,arr[1]);			
		}
		else
		{
			VE_SearchManager._AddSearch(terms[i]);
		}
	}
	*/
	
	for(var i=0; i<terms.length; i++)
	{
		VE_SearchManager._AddSearch(terms[i]);
	}
	
	VE_SearchManager._ResetPaging();
	VE_SearchManager._CancelAllRequests();
	VE_SearchManager._DoSearch("", true);	
}

// Performs a combined query search
VE_SearchManager.CombinedSearch = function(combinedQuery)
{
	if (!combinedQuery || combinedQuery == "")
	{
		return;
	}
	
	// Perform special search here.
	var url = VE_SearchManager.searchPage+"?c=0&d=0&e=0&f=0&g=&i=&r=false&q=" + escape(combinedQuery);
	
	
	// Build the xml http
	//
	// fix for firefox - reset the getXmlHttp connection
	// because firefox can't switch between entities.ashx
	// and search.ashx
	// ( window.XMLHttpRequest => firefox )
	if ( !VE_SearchManager.combinedSearchXmlHttp || window.XMLHttpRequest )
	{
		VE_SearchManager.combinedSearchXmlHttp = null;
		VE_SearchManager.combinedSearchXmlHttp = GetXmlHttp ();
	}
	
	// Send the request
	var x = VE_SearchManager.combinedSearchXmlHttp;
	
	if(x)
	{	
		x.open("POST", url, true);
		x.onreadystatechange = VE_SearchManager._CombinedSearchResponseHandler;
		x.send("");
	}
}



// Refresh all searches
VE_SearchManager.RefreshAll=function()
{
	if(VE_SearchManager.searches.length > 0)
	{
		VE_SearchManager._ResetPaging();
		VE_SearchManager._CancelAllRequests();
		VE_SearchManager._DoSearch("", true, autoRefreshDelay);
	}
}

// Find a search by index
VE_SearchManager.FindByIndex=function(index)
{
	var s = VE_SearchManager.searches;
	for(var i=0; i<s.length; i++)
	{
		if(s[i].index == index)
			return s[i];
	}
	return null;
}

// Find a search by panel
VE_SearchManager.FindByPanel = function(panel)
{
	var s = VE_SearchManager.searches;
	var i;
	for (i = 0; i < s.length; i++)
	{
		if (s[i].panel == panel)
			return s[i];
	}
	
	return null;
}

// Returns the index of the search of the given phrase
// Returns -1 if the phrase is not found
VE_SearchManager.IndexOf=function(phrase)
{
	var s = VE_SearchManager.searches;
	for(var i=0; i<s.length; i++)
	{
		if(s[i].phrase == phrase)
			return s[i].index;
	}
	return -1;
}

// Add a new search to the list
VE_SearchManager._AddSearch=function(what, type, category)
{
	var categoryId = -1;
	if(category != null && category != "undefined" && category > 0)
	{
		categoryId = category;
	}

	var a = VE_SearchManager.searches;

	// If we already have the max, remove the last one first.
	// This must be done before adding a new search to keep
	// the colors in order.
	if(a.length == VE_SearchManager.maxSearches)
	{
		// sstchur:  Housekeeping on the taskArea is needed as well
		var p = a[a.length - 1].panel;		

		/*	true = supressRedraw
				Since this is a non-explicit removal (exceeded max searches) we know that there is another search on its way.
				Therefore, we'll postpone redraw until the new search is added to the TaskArea (redraw will happen automatically then)
		*/
		taskArea.removeItem(p, true);

		
		VE_SearchManager._RemoveSearch(a[a.length-1].index);
	}

	// Create the search and add it to the list.

	var s = new VE_Search(what, type, categoryId);



	a.unshift(s);	
		
	// Execute Post events if any
	if (this.onAddSearch)
	{
		this.onAddSearch();
	}
		
}


// Moves a search from the middle of the array to the front of the array.
VE_SearchManager._MoveToFront=function(index, maximize)
{
	// sstchur:
	var searches = VE_SearchManager.searches;
	for (var i = 0; i < searches.length; i++)	
	{
		var search = searches[i];
		if (search.index == index)
		{
			taskArea.setCurrent(search.panel);			
			if (search.ads == null)
				VE_SearchManager._RefreshAds(search);
			
			break;
		}
	}
}


// Remove a search from the list
VE_SearchManager._RemoveSearch=function(index)
{
	var s = VE_SearchManager.searches;
	
	for(var i=0; i<s.length; i++)
	{
		if(s[i].index == index)
		{
			if (VE_SearchManager.popup && s[i].index == VE_SearchManager.popupID) {
				VE_SearchManager.CloseSearchPopup();
			}			
			VE_SearchManager.colors.push(s[i].color);
			s[i].Destroy();
			s.splice(i, 1);

			// Execute Post events if any
			if (this.onRemoveSearch)
			{
				this.onRemoveSearch();
			}
			return;
		}
	}
}


// Reset all of the searches back to the original page
VE_SearchManager._ResetPaging=function()
{
	var s = VE_SearchManager.searches;
	for(var i=0; i<s.length; i++)
	{
		s[i].startIndex = 0;
		s[i].displayIndex = 0;
	}
}


// Execute a search
// where - the where text to send to the server
// all - determines whether or not to do all searches
// delay - optional delay before executing the request
VE_SearchManager._DoSearch=function(where, all, delay)
{
	var s = VE_SearchManager.searches;
	var a = "";
	var b = IOSec.EncodeUrl(where);
	
	/* sstchur: */
	var topLeft = map.PixelToLatLong(new Msn.VE.Pixel(0, 0));
	var bottomRight = map.PixelToLatLong(new Msn.VE.Pixel(windowWidth, windowHeight));


	/*		
	var x1 = taRoller.isExpanded() ? taskArea_positioner.offsetWidth : 0;
	var y1 = header.offsetHeight + mapActionBar.offsetHeight;
	var topLeft = map.PixelToLatLong(new Msn.VE.Pixel(x1, y1));	
	
	var x2 = windowWidth;	
	var y2 = windowHeight - footer.offsetHeight;
	var bottomRight = map.PixelToLatLong(new Msn.VE.Pixel(x2, y2));
	*/
	
	

	if (topLeft == null || bottomRight == null)
		return;

	var c = topLeft.latitude;
	var d = bottomRight.longitude;
	var e = bottomRight.latitude;
	var f = topLeft.longitude;
	
	// Swap latitudes if they are in the wrong order.
	if (c < e)
	{
		var t = c;
		c = e;
		e = t;
	}
	
	// Swap longitudes if they are in the wrong order.
	if (d < f)
	{
		var t = d;
		d = f;
		f = t;
	}
	
	var g = "";
	var i = "";
	var r = "false";
	var cat = -1;

	
	if(!where || where == "")
	{
		// Always execute at least the *active* search (not necessarily the first)
		if(s.length > 0)
		{
			var activeSearch = VE_SearchManager.FindByPanel(taskArea.getCurrent());
			
			if (!activeSearch)
				activeSearch = s[0];				// fall back to s[0] (search 0) if we couldn't find the activeSearch by Panel		
		
			a = IOSec.EncodeUrl(activeSearch.phrase);
			g = activeSearch.startIndex;
			i = activeSearch.index;	
			//category
			if(activeSearch.categoryId && activeSearch.categoryId > 0)
				cat = activeSearch.categoryId;
				
			r++;
		}
		
		// If all searches, add the values
		if(all)
		{
			for(var j=0; j<s.length; j++)
			{
				/* Skip the active search, as it would have been handled in the previous if conditional */
				if (s[j] == activeSearch)
					continue;			
			
				a += "|" + IOSec.EncodeUrl(s[j].phrase);
				i += "|" + s[j].index;
				//category
				if(s[j].categoryId && s[j].categoryId > 0)
					cat += "|" +  s[j].categoryId;				
				r++;
			}
			
			r = "true";
		}
		
		// Clean the where so "undefined" doesn't get sent up
		b = "";		
	}
	else
	{
		for(var j=0; j<s.length; j++)
		{
			//Refresh ads
			s[j].ads = null;
		}		
	}	

	var url = VE_SearchManager.searchPage+"?a="+a+"&b="+b+"&c="+c+"&d="+d+"&e="+e+"&f="+f+"&g="+g+"&i="+i+"&r="+r+"&mkt="+locale; 
	
	//Append category id	
	if(cat != null && cat != -1)
	{
		url += "&h="+cat;
	}
	
	
	VE_SearchManager._AddRequest(url, delay);
}


// Refreshes the ads for the specified search
VE_SearchManager._RefreshAds = function(s)
{
	if (s == null)
		return;
		
	//Remove element from document
	try
	{
		var adid = "ve_telco_ads" + s.index;
		var adel = document.getElementById(adid);
		if(adel != null)
		{
			s.panel.el.removeChild(adel);
		}
	}
	catch(ex)
	{
	}	
		
	var where = "";
	try
	{
		//Lets see if perf is going to be ok
		where = document.getElementById('where').value;
		if(where == L_SearchUseCurrentViewText_Text)
			where = '';
	}
	catch(err)
	{
	}

	var url = VE_SearchManager.adsPage + 
		"?a=" + IOSec.EncodeUrl(s.phrase) + 
		"&b=" + s.index + 
		"&c=" + map.GetCenterLatitude().toFixed(6) + 
		"&d=" + map.GetCenterLongitude().toFixed(6) +	
		"&e=" + where +		
		"&mkt=" + locale; 				
	VE_SearchManager._AddRequest(url);
	s.requestedAds = true;		
}


// Adds a request to the queue and fires it, if not already searching
VE_SearchManager._AddRequest=function(url, delay)
{
	VE_SearchManager.queue.unshift(url);
	
	if(delay)
	{
		VE_SearchManager.timerID = window.setTimeout("VE_SearchManager._SendNextRequest()", delay);
	}
	else
	{
		VE_SearchManager._SendNextRequest();
		VE_SearchManager.timerID = -1;
	}
}

// Cancels any delayed requests
VE_SearchManager._CancelTimerRequest=function()
{
	if(VE_SearchManager.timerID != -1)
	{
		try
		{
			window.clearTimeout(VE_SearchManager.timerID);
		}
		catch(ex)
		{
			// swallow, the thread may have completed
		}
		VE_SearchManager.timerID = -1;
	}
}

// Cancel all existing search requests
VE_SearchManager._CancelAllRequests=function()
{
	// If already searching, ignore
	if(VE_SearchManager.searching)
	{
		VE_SearchManager.searching = false;
		VE_SearchManager.xmlHttp.abort();
	}
	
	// Cancel the time
	VE_SearchManager._CancelTimerRequest();
	
	// Throw away all of the requests
	var q = VE_SearchManager.queue;
	while(q.length)
	{
		q.pop();
	}	
}

// Fires the next request, if not already searching
VE_SearchManager._SendNextRequest=function()
{
	// If already searching, ignore
	if(VE_SearchManager.searching)
	{
		return;
	}
	
	// Clear the timer
	VE_SearchManager._CancelTimerRequest();

	// If no requests, return
	var q = VE_SearchManager.queue;
	if(!q || q.length == 0)
	{
		return;
	}
	
	var url = q.pop();
	//firfox issue : when search originates from sample searches, xmlhttprequest calls /Help/[market]/search...
	url = GetUrlPrefix() + url;
	
	//
	// Build the xml http
	//
	// fix for firefox - reset the getXmlHttp connection
	// because firefox can't switch between entities.ashx
	// and search.ashx
	// ( window.XMLHttpRequest => firefox )
	if ( !VE_SearchManager.xmlHttp || window.XMLHttpRequest )
	{
		VE_SearchManager.xmlHttp = null;
		VE_SearchManager.xmlHttp = GetXmlHttp ();
	}
	
	// Send the request
	var x = VE_SearchManager.xmlHttp;
	if(x)
	{
		x.open("POST", url, true);
		x.onreadystatechange = VE_SearchManager._ResponseHandler;
		VE_SearchManager.searching = true;
		x.send("");
	}
}


// Handles the response to a search
VE_SearchManager._ResponseHandler=function()
{
	var x = VE_SearchManager.xmlHttp;
	if(x.readyState==4) 
	{
		VE_SearchManager.searching = false;
		var code = x.responseText;
		try
		{
			eval(code);		
			if(VE_SearchManager.onSearchComplete)
			{				
					VE_SearchManager.onSearchComplete();
			}				
		}
		catch(ex)
		{
			// Swallow exceptions.  Don't display JavaScript errors to the user.
		}
		VE_SearchManager._SendNextRequest();
	}
}

VE_SearchManager._CombinedSearchResponseHandler=function()
{
	var x = VE_SearchManager.combinedSearchXmlHttp;
	if(x.readyState==4) 
	{		
		var code = x.responseText;
		
		try
		{				
			eval(code);						
		}
		catch(ex)
		{
			// Swallow exceptions.  Don't display JavaScript errors to the user.
		}
	}
	x = null;
}


// Recover from a failed where search.
VE_SearchManager._HandleFailedWhereSearch = function(message)
{
	// Display the error message if we have one.
	if (message)
	{		
		ShowMessage(message.replace(/%1/g,Msn.VE.Features.ProductName));
		resetWhereBox();
		VE_Analytics.LogFailedSearch();
	}
	
	// Determine if there was a new search that needed to also be executed as part of this where search.
	var s = VE_SearchManager.searches;
	for (var i=0; i < s.length; i++)
	{
		// Results is only null when the search is first created, but hasn't had any results returned to it.
		if (s[i].results == null)
		{
			VE_SearchManager._DoSearch("", true);
			break;
		}
	}
}

// Applies the ads from the server to a search.
VE_SearchManager._ApplyAds = function(index, ads, adsprovider)
{

	var s = VE_SearchManager.FindByIndex(index);
	if (!s)
	{
		return;
	}

	// Only update if the server has returned some ads
	if (ads != null)
	{
		s.ads = ads;
	}
	else
	{
		VE_Analytics.LogAds(s.phrase, "Event7", "Ads", "0", "0");			
		s.requestedAds = false;
	}		
	
	if(adsprovider != null)
	{
		s.adProvider = adsprovider;
	}
	
	s._UpdateAds();
}

// Applies the results from the server to a search.
VE_SearchManager._ApplyResults=function(index, results, ads, hasMore, message, searchCategories, adprovider)
{
	var s = VE_SearchManager.FindByIndex(index);
	if(!s)
	{
		return;
	}
	
	s._ClearPins();
	s.results = results;

	// Only update if the server has returned some ads
	if (ads != null && ads != "undefined" && ads.length > 0)
	{
		s.requestedAds = true;
		VE_SearchManager._ApplyAds(index, ads, adprovider);
	}
	else
	{
		if(s.requestedAds)
			VE_Analytics.LogAds(s.phrase, "Event7", "Ads", "0", "0");		
	}
		
	if(adprovider != null && adprovider != "undefined" && adprovider.length != 0)
	{
		s.adProvider = adprovider;
	}
		
	// This is probably the initial results search for what/where query.
	// We need to manually request ads for it.
	if (s.ads == null)
	{
		VE_SearchManager._RefreshAds(s);
	}
	
	s.hasMore = hasMore;
	s.message = message;
	
	//categories
	s.categories = searchCategories;
	
	s._UpdatePins();
	s._UpdatePanel();
	
	if(VE_ScopingBar)
	{
		VE_ScopingBar.Show(false);
		dragController.setBoundingArea(getVisibleMapRect());
	}
}

// Changes the spelling of a search
VE_SearchManager.ApplySuggestion=function(index, suggestion)
{
	taskArea.removeItem(VE_SearchManager.FindByIndex(index).panel);	
	VE_SearchManager._RemoveSearch(index);	
	VE_SearchManager.Search(suggestion);
	
	var txtWhat = document.getElementById('what');
	txtWhat.value = suggestion;
	setActive(txtWhat);	
	
	if(VE_ScopingBar)
	{
		VE_ScopingBar.Show(false);
		dragController.setBoundingArea(getVisibleMapRect());
	}
}


VE_SearchManager.HandleOnPaste=function(e)
{
	//clipboard data and "onpaste" handler currently
	//only supported in IE and Safari
	if(window.clipboardData)
	{
		var before = window.clipboardData.getData("Text","");
		var after = before.replace(/[\n\r]+/g,", ");		
		window.clipboardData.setData("Text",after);
	}
};

//_RemoveSearchPanel
// Removes a search and its related Search Panel if the Search and its panel exists
// The index passed in is the Search index
VE_SearchManager._RemoveSearchPanel=function(index)
{
	var search = VE_SearchManager.FindByIndex(index);
	if ((search != "undefined") && (search != null))
	{
		if ((search.panel != "undefined") && (search.panel != null))
		{
				var panel = search.panel;
				VE_SearchManager._RemoveSearch(index);
				taskArea.removeItem(panel);
		}
	}
};

VE_SearchManager.ShowSearchResultDetails = function(rId)
{
// Details Actions
	if (Msn.VE.Marketization.IsEnabled(Msn.VE.Features.MenuOption.DetailsOnSearchItem))
	{	
		OpenNewWindow(GetUrlPrefix() + 'details.aspx?dt=yp.' + rId + '&style=' + map.GetMapStyle(), 'Detail' + rId, 'all', 775, 500, 200, 200);
		VE_Analytics.Log('VE | Search Result', 'Get Details');	
	}	
};		
