//------------------------------------------------------------------------------
// JS/Select.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: a-bwrigh
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// Class: VE_SelectItem
// This classes is used to wrap data with a description
// when adding items to a VE_Select class. This way any object
// can be added and retrieved from a VE_Select class in addition
// to displaying an appropriate description for the object.
function VE_SelectItem(data,description)
{
		this.data = data;
		this.description = description;
}

VE_SelectItem.prototype.toString=function()
{
	return this.description;
}


// Class: VE_Select.
// description: A DHTML version of the select object.
// 
function VE_Select(id,aDefaultClass,aSelectedClass,aMouseOverClass)
{
	this.isVisible = false;
	var items = new Array();			
	var element = document.createElement("div");	
	element.setAttribute("id",id);	
	this.id = id;
	var elementId = id;
	var objRef = this;
	var selectedIndex = -1;
	var selectedClass = aSelectedClass ? aSelectedClass : "";
	var defaultClass = aDefaultClass ? aDefaultClass : "";
	var mouseOverClass = aMouseOverClass ? aMouseOverClass : "";	
	var previousDefaultClass = "";
	//Add a new item to the select object.
	//param "item" needs to be of type VE_SelectItem or an exception will be thrown.
	function AddItem(item)
	{				
		if(!(item instanceof VE_SelectItem))
		{
			item = new VE_SelectItem(item,item.toString());
		}
		items.push(item);
		
		var listDiv = document.createElement("div");				
		listDiv.setAttribute("id",elementId+"_"+(items.length-1));				
		listDiv.onclick = ItemClick;
		listDiv.onmouseover = ItemMouseOver;
		listDiv.onmouseout = ItemMouseOut;
		listDiv.innerHTML = item.description;
		element.appendChild(listDiv);
	}
	
	// returns the number of items contained in the select
	function GetCount()
	{
		return items.length;
	}
	
	// returns the DHTMLElement of the select object.
	// This is useful after creating a select and adding it to 
	// the DOM. I.E: document.body.appendChild(mySelectObject.GetElement());
	function GetElement()
	{
		return element;
	}		
	
	// internal / private method called when an item is clicked
	function ItemClick(e)
	{
		var itemDiv = GetTarget(e);		
		var index = FindIndexForElement(itemDiv);		
		SelectItemAtIndex(index);
		if(objRef.OnClick)
		{
			objRef.OnClick(index,items[index]);			
		}
		if(objRef.OnSelect)
		{
			objRef.OnSelect(index,items[index]);			
		}
	}
	
	// internal / private methods called when an item is moused over
	function ItemMouseOver(e)
	{
		var itemDiv = GetTarget(e);	
		var index = FindIndexForElement(itemDiv);
		previousDefaultClass = itemDiv.className;
		itemDiv.className = mouseOverClass;		
		if(objRef.OnMouseOver)
		{
			objRef.OnMouseOver(index,items[index]);
		}
	}
	// internal / private methods called when an item is moused out
	function ItemMouseOut(e)
	{
		var itemDiv = GetTarget(e);	
		var index = FindIndexForElement(itemDiv);
		itemDiv.className = previousDefaultClass;
		if(objRef.OnMouseOut)
		{
			objRef.OnMouseOut(index,items[index]);
		}
	}
	// set the select item at the param "index".
	// if index is no in range then all items are deselected.
	function SelectItemAtIndex(index)
	{
		if(index >= 0 && index < items.length)
		{
			var previousIndex = GetSelectedIndex();
			if(previousIndex >= 0)
			{
				element.childNodes[previousIndex].className = defaultClass;
			}			
			selectedIndex = index;
			element.childNodes[index].className = selectedClass;
			previousDefaultClass = element.childNodes[index].className;
			
		}
		else 
		{			
			var previousIndex = GetSelectedIndex();
			if(previousIndex >= 0)
			{
				element.childNodes[previousIndex].className = defaultClass;
			}	
			selectedIndex = -1;
		}
		
	}
	
	// return the selected items index
	function GetSelectedIndex()
	{
		return selectedIndex;
	}
	
	// return the currently selcted VE_SelectItem 	
	function GetSelectedItem()
	{
		if(selectedIndex >= 0 && selectedIndex < items.length)
		{
			return items[selectedIndex];
		}
		return null;
	}
	// return the VE_SelectItem at the specific param "index"
	function ItemAtIndex(index)
	{
		if(index >= 0 && index < items.length)
		{
			return items[index];
		}
		return null;
	}
	
	function RemoveItemAtIndex(index)
	{
		if(index < 0 || index >= items.length)
			return;
		if(index < selectedIndex)
		{ //need to update selectedIndex
			selectedIndex-=1;
		}
		else if(index == selectedIndex)
		{
			selectedIndex = -1;
		}
		items.splice(index,1);
		element.removeChild(element.childNodes[index]);
	}
	
	function FindIndexForElement(elm)
	{
		for(var i =0; i < element.childNodes.length ; ++i)
		{
			if(elm == element.childNodes[i])
			{
				return i;
			}
		}
		return -1;
	}

	// remove all items
	function ClearItems()
	{				
		while(items.length > 0)
		{
			items.pop();					
		}
		while(element.childNodes.length > 0)
		{
			element.removeChild(element.lastChild);
		}
		selectedIndex = -1;
	}
	
	function Show()
	{
		element.style.display = "block";
		this.isVisible = true;
	}
	
	function Hide()
	{
		element.style.display = "none";
		this.isVisible = false;
	}	
	
	// Setup public methods
	this.OnClick = null;
	this.GetCount = GetCount;
	this.GetElement = GetElement;
	this.GetSelectedIndex = GetSelectedIndex;
	this.GetSelectedItem = GetSelectedItem;
	this.SelectItemAtIndex = SelectItemAtIndex;
	this.OnSelect = null;
	this.AddItem = AddItem;
	this.OnMouseOver  = null;
	this.OnSelect = null;
	this.OnMouseOut = null;
	this.ClearItems = ClearItems;
	this.RemoveItemAtIndex = RemoveItemAtIndex;
	this.Show = Show;
	this.Hide = Hide;
	
}