//------------------------------------------------------------------------------
// StateUpdateEvents.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Dependencies
//
// State.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Referred to by
//
// Many many files
//------------------------------------------------------------------------------


// Updates the permalink based on the current VE state
VE_State.prototype.GetPermalink = function()
{
	var searchesString = this._config.ToString(VE_State.CategorySearches, true);
	var viewportString = this._config.ToString(VE_State.CategoryViewport, true);
	var scratchpadString = this._config.ToString(VE_State.CategoryScratchpad, true);
	var drivingDirectionsString = this._config.ToString(VE_State.CategoryDirections, true);
	var trafficString = this._config.ToString(VE_State.CategoryTraffic, true);

	var urlParameterString = "";
	if (searchesString)
	{
		if (urlParameterString)
		{
			urlParameterString += "&";
		}
		urlParameterString += searchesString;
	}
	if (viewportString)
	{
		if (urlParameterString)
		{
			urlParameterString += "&";
		}
		urlParameterString += viewportString;
	}
	
	if (trafficString)
	{
		//add only if trfc is ON
		if( this.GetTraffic() != 0 )
		{
			if (urlParameterString)
			{
				urlParameterString += "&";
			}
			urlParameterString += trafficString;
		}
	}
	
	if (drivingDirectionsString)
	{
		if (urlParameterString)
		{
			urlParameterString += "&";
		}
		urlParameterString += drivingDirectionsString;
	}
	
	//Only Store the scratchpad string if it is not empty
	if (scratchpadString && scratchpadString.length > 0)
	{
		if (urlParameterString)
		{
			urlParameterString += "&";
		}
		urlParameterString += scratchpadString;
	}
	
	var plink = GetUrlPrefix() + "default.aspx?" + "v=" + this.GetCurrentUrlApiVersion() + "&" + urlParameterString;
	return IOSec.TruncateUrlSafe (plink, 2020);
}

// Updates VE state based on the current state of the map
VE_State.prototype.UpdateFromMap = function()
{
	//If the map style is Birdseye, then store the CenterPoint LatLong state as an encoded String to hide 
	//the actual lat/long coordinates
	if (map.GetMapStyle() == Msn.VE.MapStyle.Oblique)
	{
		var encoder = new VELatLongEncoding();
		this.SetCenterPoint(encoder.Encode(map.GetCenterLatitude().toFixed(6), map.GetCenterLongitude().toFixed(6)));
	}
	else
	{
		//Otherwise store the CenterPoint LatLong state normally as an Array
		this.SetCenterPoint([map.GetCenterLatitude().toFixed(6), map.GetCenterLongitude().toFixed(6)]);
	}
	
	this.SetZoom(map.GetZoomLevel());
	this.SetMapStyle(map.GetMapStyle());
	
	var scene = map.GetObliqueScene();
	if (scene)
	{
		this.SetScene(scene.GetID());
	}
}

// Updates VE state based on the current state of searches
VE_State.prototype.UpdateFromSearches = function()
{
	var searchKeywords = new Array(VE_SearchManager.searches.length);
	for (var i = VE_SearchManager.searches.length - 1; i > -1; i--)
	{
		if(!VE_SearchManager.searches[i] || VE_SearchManager.searches[i].IsWhereAdsSearch())
		{
			continue;
		}
		
		// Need to do this because it will be unescaped down the line.		
		searchKeywords.push(IOSec.EncodeUrl(IOSec.EncodeUrl(VE_SearchManager.searches[i].phrase)));
		//searchKeywords.push(IOSec.EncodeUrl(IOSec.EncodeUrl(VE_SearchManager.searches[i].phrase + '^' + VE_SearchManager.searches[i].categoryId)));
	}
	this.SetSearchKeywords(searchKeywords);
}

// Updates VE state based on the current state of the scratchpad
VE_State.prototype.UpdateFromScratchpad = function()
{	
	var scratchpadEntities = new Array(VE_Scratchpad.entities.length);
	//modified to use collection manager for all scratchpad items
	scratchpadEntities.push(VE_Scratchpad.ScratchpadSerializer(false));
	this.SetScratchpadEntityIds(scratchpadEntities);
}

// Updates VE state based on the current state of driving directions.
VE_State.prototype.UpdateFromDrivingDirections = function()
{
	if (VE_Directions.displayedRoute == null)
	{
		this.SetRoutePoints([]);
	}
	else
	{
		this.SetRoutePoints(VE_Directions.displayedRoute.GetSerializedForm());
	}
}

// VE specific VE_SearchManager post add search event
VE_SearchManager.onAddSearch = function()
{
	if (state)
	{
		state.UpdateFromSearches();
	}
}

// VE specific VE_SearchManager post remove search event
VE_SearchManager.onRemoveSearch = function()
{
	if (state)
	{
		state.UpdateFromSearches();
	}
}

// Post event for when a SearchResult is added.
VE_Scratchpad.onAddSearchResult = function()
{
	if (state)
	{
		state.UpdateFromScratchpad();
	}
}

// Post event for when a Location is added.
VE_Scratchpad.onAddLocation = function()
{
	if (state)
	{
		state.UpdateFromScratchpad();
	}
}

// Post event for when many SearchResults are added.
VE_Scratchpad.onAddResults = function()
{
	if (state)
	{
		state.UpdateFromScratchpad();
	}
}

// Post event for when an entity is removed.
VE_Scratchpad.onRemoveEntity = function()
{
	if (state)
	{
		state.UpdateFromScratchpad();
	}
}

// VE specific VE_SearchManager post add search event
VE_SearchManager.onAddSearch = function()
{
	if (state)
	{
		state.UpdateFromSearches();
	}
}

// VE specific VE_SearchManager post remove search event
VE_SearchManager.onRemoveSearch = function()
{
	if (state)
	{
		state.UpdateFromSearches();
	}
}

