//------------------------------------------------------------------------------
// ./animation/animation.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur
//
//  Dependencies:
//  1.  Microsoft.Web.Geometry (microsoft.web.geometry.js)
//	2.  Micrososft.Web.Css (microsoft.web.css.js)
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};

Microsoft.Web.Animation =
{
	AccelerationFunction: function(pFn)
	{
		// Maintain a reference to 'this'
		var self = this;
		
		// private variables
		var m_nTotal = -1;
		var m_nSteps = -1;		
		var m_pFn = pFn;
		
		this.setSteps = function(_steps)
		{
			m_nSteps = _steps;
			CalculateTotal();
		}
		
		this.getValue = function(_percentDone)
		{
			return m_pFn(_percentDone) / m_nTotal;
		}
		
		function CalculateTotal()
		{
			m_nTotal = 0;
			for (var i = 1; i <= m_nSteps; i++)
				m_nTotal += m_pFn(i / m_nSteps);
		}
	},
		
	Movie: function(container, interval)
	{
		var self = this;
		this.Repeat = true;
		this.AppendContent = true;
		
		var m_frames = new Array();
		var m_counter = -1;
		var m_intervalId = null;
		
		this.addFrame = function(contents, addToPrev)
		{
			if (addToPrev == null)
				addToPrev = true;
				
			var f = { data: contents, append: addToPrev }
			m_frames.push(f);
		}
		
		this.start = function()
		{
			self.stop();
			self.show();
			self.clear();
			m_counter = -1;		
			m_intervalId = setInterval(Play, interval);		
		}
		
		this.stop = function()
		{
			if (m_intervalId)		
				clearInterval(m_intervalId);
		}
		
		this.show = function()
		{
			container.style.visibility = 'visible';	
		}
		
		this.hide = function()
		{
			container.style.visibility = 'hidden';
		}
		
		this.clear = function()
		{
			container.innerHTML = '';
		}
		
		
		function Play()
		{
			m_counter++;
			if (m_counter > m_frames.length - 1)
			{
				m_counter = 0;			
				if (!self.Repeat)
				{
					clearInterval(m_intervalId);
					return;
				}
				else
					self.clear();
			}
			
			var f = m_frames[m_counter];
			if (f.append)
				container.innerHTML += f.data;
			else
				container.innerHTML = f.data;
		}	
	},
	
	
	// Roller Enumerations
	RollDirection:
	{
		TopDown: 0,
		RightLeft: 1,
		BottomUp: 2,
		LeftRight: 3,
		Count: 4
	},

	RollStyle:
	{
		In: 0,
		Out: 1
	},
	
	// Roller Class
	Roller: function(_elem, _leaveAmount)
	{
		// Maintain a reference to 'this'
		var self = this;
		
		// Shortcut the Microsoft.Web.Css Namespace Chain
		var css = Microsoft.Web.Css;
		// Shortcut the Microsoft.Web.Animation Namespace Chaind
		var ani = Microsoft.Web.Animation;
		
		// Private variables
		var m_elem = _elem;
		var m_pals = null;
		var m_isExpanded = true;
		var m_isRolling = false;
		
		// Public properties
		this.LeaveAmount = (typeof _leaveAmount == 'number') ? _leaveAmount : 10;
		this.AccFunction = null;
		this.Factor = null;
		
		// Events
		this.onroll = null;
		this.onbeforerollin = null;
		this.onafterrollin = null;
		this.onbeforerollout = null;
		this.onafterrollout = null;
		
		this.isExpanded = function()
		{
			return m_isExpanded;
		};
		
		this.isRolling = function()
		{
			return m_isRolling;
		};
		
		this.associate = function(_pals)
		{
			m_pals = _pals;
		};	
		
		this.rollIn = function(direction)
		{
			if (m_isRolling)
				{ return; }
				
			OnBeforeRollIn(m_elem);
			Roll(ani.RollStyle.In, direction);
			m_isExpanded = false;			
		};
		
		this.rollOut = function(direction)
		{
			if (m_isRolling)
				{ return; }
				
			OnBeforeRollOut(m_elem);
			Roll(ani.RollStyle.Out, direction);
			m_isExpanded = true;			
		};
		
		function Roll(style, direction)
		{
			m_isRolling = true;
			
			if (!self.AccFunction)
				self.AccFunction = AccelerationFunctions.CrazyElevator;
				
			if (!self.Factor)
				self.Factor = 15;
			
			var size;
			if (direction == ani.RollDirection.TopDown || direction == ani.RollDirection.BottomUp)
				size = m_elem.offsetHeight;
			else
				size = m_elem.offsetWidth;
		
			var rollDistance = size - self.LeaveAmount;
			
			if (direction == ani.RollDirection.TopDown || direction == ani.RollDirection.LeftRight)
			{
				size = 0;
				if (style == ani.RollStyle.Out)
					size = self.LeaveAmount;
			}
			else
			{
				if (style == ani.RollStyle.Out)
					size -= self.LeaveAmount;
			}
			
			var steps = Math.ceil(rollDistance / 10);
			if (steps < 50) steps = 50;											// 50 is arbitrary (and there are better ways to handle this.... it's a TODO)
			self.AccFunction.setSteps(steps);
			
			for (var i = 1; i <= steps; i++)
			{
				var percentComplete = i / steps;
				setTimeout(doRoll(percentComplete), i * self.Factor);
			}
		
			function doRoll(pc)
			{
				return function()
				{
					var prevSize = size;
					var fnX = self.AccFunction.getValue(pc) * rollDistance;
					if (direction == ani.RollDirection.TopDown || direction == ani.RollDirection.LeftRight)
						size += fnX;
					else
						size -= fnX;
						
					var dir = direction;
					if (style == ani.RollStyle.Out)
						dir = (direction + ani.RollDirection.Count / 2 ) % ani.RollDirection.Count;			// reverse the clipping direction for RollOut
					var clip = getClip(dir, size);					
					m_elem.style.clip = clip;
					
					if (m_pals)
					{
						var delta = Math.abs(Math.round(prevSize) - Math.round(size));
						for (var i = 0; i < m_pals.length; i++)						
							{ associationMove(m_pals[i], delta, direction); }
					}

					if (pc == 1)				// make sure it's completely done (pc = percentComplete)
					{
						m_isRolling = false;
						if (style == ani.RollStyle.In)
							{ OnAfterRollIn(m_elem); }
						else	// must be RollStyle.Out						
						{
							m_elem.style.clip = 'rect(auto auto auto auto)';
							OnAfterRollOut(m_elem);
						}
					}
					
					OnRoll(m_elem);
				}
			}
		}
		
		function associationMove(pal, delta, direction)
		{
			var pos = { x: 0, y: 0 };
			
			var left = parseInt(css.Functions.getComputedStyle(pal, 'left'));
			pos.x = isNaN(left) ? 0 : left;
			
			var top = parseInt(css.Functions.getComputedStyle(pal, 'top'));
			pos.y = isNaN(top) ? 0 : top;
			
			if (direction == ani.RollDirection.TopDown)
				{ pal.style.top = (pos.y + delta) + css.Units.Pixels; }
			else if (direction == ani.RollDirection.LeftRight)
				{ pal.style.left = (pos.x + delta) + css.Units.Pixels; }
			else if (direction == ani.RollDirection.BottomUp)
				{ pal.style.top = (pos.y - delta) + css.Units.Pixels; }
			else
				{ pal.style.left = (pos.x - delta) + css.Units.Pixels; }			
		}
	
		function getClip(direction, size)
		{
			var topClip, rightClip, bottomClip, leftClip;
			topClip = rightClip = bottomClip = leftClip = css.Units.Auto;
			
			var sizeClip = parseInt(Math.round(size)) + css.Units.Pixels;
			switch (direction)
			{
				case ani.RollDirection.TopDown:
					topClip = sizeClip;
					break;
				case ani.RollDirection.RightLeft:
					rightClip = sizeClip;
					break;
				case ani.RollDirection.BottomUp:
					bottomClip = sizeClip;
					break;
				case ani.RollDirection.LeftRight:
					leftClip = sizeClip;
					break;
			}
			
			var clip = 'rect(' + topClip + ' ' + rightClip + ' ' + bottomClip + ' ' + leftClip + ')';
			return clip;
		}	
		
		
		// -- EVENT FUNCTIONS -- //
		function OnRoll(_elem)
		{
			if (typeof self.onroll == 'function')
				{ self.onroll(_elem); }
		}
		
		function OnBeforeRollIn(_elem)
		{
			if (typeof self.onbeforerollin == 'function')
				{ self.onbeforerollin(_elem); }
		}
		
		function OnAfterRollIn(_elem)
		{
			if (typeof self.onafterrollin == 'function')
				{ self.onafterrollin(_elem); }
		}
		
		function OnBeforeRollOut(_elem)
		{
			if (typeof self.onbeforerollout == 'function')
				{ self.onbeforerollout(_elem); }
		}
		
		function OnAfterRollOut(_elem)
		{
			if (typeof self.onafterrollout == 'function')
				{ self.onafterrollout(_elem); }
		}		
	},

	Slider: function(_elem)
	{
		// Maintain a reference to 'this'
		var self = this;
	
		// Shortcut the Microsoft.Web.Geometry Namespace Chain
		var geo = Microsoft.Web.Geometry;
		
		var m_elem = _elem;
		var m_displacement = 0;
		var m_currentX;
		
		this.EasingFunction = null;
		this.Factor = null;

		this.slideToPoint = function(_endPoint)
		{
			if (!self.AccFunction)
				self.AccFunction = AccelerationFunctions.Linear;
				
			if (!self.Factor)
				self.Factor = 10;
				
			var startPoint = geo.Functions.getElementPosition(m_elem);
			var currentX = startPoint.x;
			var currentY = startPoint.y;
			
			var m = geo.Functions.getSlope(startPoint, _endPoint);		
			var b = geo.Functions.getYIntercept(m, _endPoint);

			var distance = Math.abs(startPoint.x - _endPoint.x);		
			var steps = Math.ceil(distance / 10);
			if (steps < 50) steps = 50;
			self.AccFunction.setSteps(steps);
			
			for (var i = 0; i <= steps; i++)
			{
				var percentComplete = i / steps;
				setTimeout(move(percentComplete), i * self.Factor);
			}		
		
			function move(pc)
			{
				return function()
				{	
					if (_endPoint.x > startPoint.x)
						currentX += self.AccFunction.getValue(pc) * distance;
					else
						currentX -= self.AccFunction.getValue(pc) * distance;

					var currentY = m * currentX + b;

					m_elem.style.left = parseFloat(Math.round(currentX)) + 'px';
					m_elem.style.top = parseFloat(Math.round(currentY)) + 'px';
				}
			}
		}	
	}
}

var AccelerationFunctions =
{		
	Linear: new Microsoft.Web.Animation.AccelerationFunction(
		function(percentDone)
		{
			return 1;
		}
	),
	
	ExponentialAcc: new Microsoft.Web.Animation.AccelerationFunction(
		function(percentDone)
		{
			var lower = 0;
			var upper = 1;
			var range = upper - lower;
			var x = lower + (percentDone * range);
			var fnX = Math.pow(x, 2);
			return fnX;
		}
	),
		
	ExponentialDec: new Microsoft.Web.Animation.AccelerationFunction(
		function(percentDone)
		{
			var lower = -1;
			var upper = 0;
			var range = upper - lower;
			var x = lower + (percentDone * range);
			var fnX = Math.pow(x, 2);
			return fnX;
		}
	),
	
	CosineWave: new Microsoft.Web.Animation.AccelerationFunction(
		function(percentDone)
		{
			var lower = -Math.PI;
			var upper = Math.PI;
			var range = upper - lower;
			var x = lower + (percentDone * range);
			var fnX = Math.cos(x) + 1;		
			return fnX;		
		}
	),
	
	CrazyElevator: new Microsoft.Web.Animation.AccelerationFunction(
		function(percentDone)
		{
			var lower = -5;
			var upper = 5;
			var range = upper - lower;
			var x = lower + (percentDone * range);
			var fnX = 2 / (Math.pow(Math.abs(x), 3) + 1);
			return fnX;
		}
	)	
}