//------------------------------------------------------------------------------
// ./util/microsoft.web.ui.dragdrop.js 
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur
// Dependencies:
// 1. Microsoft.Web.Geometry (microsoft.web.geometry.js)
// 2. Microsoft.Web.ADT.Hashtable (microsoft.web.adt.hashtable.js)
//
// <header>	$Header$</header>
//--

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};
if (!Microsoft.Web.UI) Microsoft.Web.UI = {};

Microsoft.Web.UI.DragDrop =
{
	DragDirection:
	{
		Horizontal: 1,
		Vertical: 2,
		Both: 3
	},
	
	DragController: function()
	{
		// Shortcut the Microsoft.Web.UI.DragDrop Namespace Chain
		var dd = Microsoft.Web.UI.DragDrop;
		// Shortcut the Microsoft.Web.Geometry Namespace Chain
		var geo = Microsoft.Web.Geometry;
		
		// Account for IE 'this' bug
		var self = this;
		
		// -- PRIVATE VARIABLES --//
		var m_htDrobs;
		var m_DBD;
		var m_boundingArea;
		Construct();
		
		// -- "Constructor" -- //
		function Construct()
		{
			m_htDrobs = new Microsoft.Web.ADT.Hashtable();
			m_DBD = null;
			m_boundingArea = new geo.Rectangle(new geo.Point(0, 0), new geo.Point(document.body.offsetWidth, document.body.offsetHeight));
			
			AddListeners();			
		}

		// -- PUBLIC FUNCTIONS -- //
		this.makeDraggableObject = function(_elem, _dragArea, _dragDirection, _useClone, _opacity)
		{
			var guid = '{DROB:' + m_htDrobs.getCount() + '}';
			var drob = new DraggableObject(_elem, _dragArea, _dragDirection, _useClone, _opacity, guid);
			m_htDrobs.addItem(guid, drob);
			return drob;
		}
		
		this.setBoundingArea = function(_rect, _refresh)
		{
			m_boundingArea = _rect;
			
			/*	By default, the dragController will refresh all of its children.  This behavior can be overridden
					by explicitely passin in false */

			if (_refresh == null)
				_refresh = true;
				
			if (!_refresh)
				return;			
			
			var i;
			for (i = 0; i < m_htDrobs.getCount(); i++)
			{
				var drob = m_htDrobs.retrieveByIndex(i);
				var elem = drob.getElement();
				
				var currentPos = drob.getLastKnownPosition();
				if (!currentPos)
					return;
				
				var p = geo.Functions.getBestBoundingPoint(elem, currentPos, m_boundingArea);				
				elem.style.top = p.y + 'px';
				elem.style.left = p.x + 'px';
				
				// Update the DROB's last known position
				drob.setLastKnownPosition(p);
			}
		}
		
		
		this.destroy = function()
		{
			RemoveListeners();
			if (m_DBD != null)
				m_DBD.destroy();
			m_boundingArea = null;
			
			var keys = m_htDrobs.getKeys();
			var i, drob;
			for (i = 0; i < keys.length; i++)
			{
				drob = m_htDrobs.retrieveByKey(keys[i]);
				if (drob)
					drob.destroy();
			}
			
			m_htDrobs.destroy();
		}
		
		
		// -- PRIVATE FUNCTIONS --//
		function AddListeners()
		{
			if (document.addEventListener)
			{
				document.addEventListener('mousemove', Drag, true);
				document.addEventListener('mouseup', EndDrag, true);		
			}
		}
		
		function RemoveListeners()
		{
			if (document.removeEventListener)
			{
				document.removeEventListener('mousemove', Drag, true);
				document.removeEventListener('mouseup', EndDrag, true);		
			}
		}
		
		function BeginDrag(_evt)
		{
			/* Only honor drag/drop in conjunction with the primary mouse button */
			var whichButton = _evt.which || _evt.button;
			if (whichButton != 1)
				return;
				
			if (typeof document.body.style.MozUserSelect != 'undefined')			
				document.body.style.MozUserSelect = 'none';

			var target = _evt.srcElement || _evt.target;
			
			while (typeof target.drobGUID == 'undefined' && target.parentNode != null)			
				target = target.parentNode;				
			
			var drob = m_htDrobs.retrieveByKey(target.drobGUID);
			if (drob)
			{
				m_DBD = drob;
				if (m_DBD.UseClone)
					m_DBD.createClone();
					
				var elem = m_DBD.getElement();		
				var p = geo.Functions.getElementPosition(elem);				
				m_DBD.setXOffset(_evt.clientX - p.x);
				m_DBD.setYOffset(_evt.clientY - p.y);
				
				// Account for the fact that IE does not correctly implement W3C recommendations
				if (!document.addEventListener)
				{
					var dragArea = m_DBD.getDragArea();
					dragArea.setCapture();
				}
				
				OnBeginDrag(m_DBD);
			}
		}
		
		function Drag(_evt)
		{
			_evt = _evt || window.event;
			
			if (m_DBD)
			{
				// W3C standard way of stopping event propagation
				if (_evt && _evt.stopPropagation)
					_evt.stopPropagation();
					
				var elem = m_DBD.getElement();
				var dragDirection = m_DBD.getDragDirection();				
			
				var left = (_evt.clientX - m_DBD.getXOffset());
				var top = (_evt.clientY - m_DBD.getYOffset());
				var drobPos = new geo.Point(left, top);
				
				var bestPoint = geo.Functions.getBestBoundingPoint(m_DBD.getElement(), drobPos, m_boundingArea);
				
				m_DBD.setLastKnownPosition(bestPoint);
				
				var hs = m_DBD.isInHotSpot();
				if (hs && typeof hs != 'undefined')
					{ m_DBD.threatenSnap(hs); }
				
				// Position the element horizontally
				if ((dragDirection & dd.DragDirection.Horizontal) == dd.DragDirection.Horizontal)				
						elem.style.left = bestPoint.x + 'px';				
				
				// Position the element vertically
				if ((dragDirection & dd.DragDirection.Vertical) == dd.DragDirection.Vertical)				
						elem.style.top = bestPoint.y + 'px';
				
				OnDrag(m_DBD);
			}
		}
		
		function EndDrag()
		{
			if (typeof document.body.style.MozUserSelect != 'undefined')			
				document.body.style.MozUserSelect = '';			
				
			if (m_DBD)
			{
				// Account for the fact that IE does not correctly implement W3C recommendations
				if (!document.addEventListener)
				{
					var dragArea = m_DBD.getDragArea();
					dragArea.releaseCapture();
				}
				
				var hs = m_DBD.isInHotSpot();
				if (hs)
				{
					m_DBD.destroyClone();
					m_DBD.snap(hs);					
				}
				else if (m_DBD.UseClone)
				{
					m_DBD.syncWithClone();
					m_DBD.destroyClone();
				}
				
				OnEndDrag(m_DBD);
			}
			
			m_DBD = null;
		}
				
		
		// -- EVENT FUNCTIONS -- //
		function OnBeginDrag(_drob)
		{
			if (self.onbegindrag)
			{
				var dea = new DragEventArgs(_drob);
				self.onbegindrag(dea);
			}
		}
		
		function OnDrag(_drob)
		{
			if (self.ondrag)
			{
				var dea = new DragEventArgs(_drob);
				self.ondrag(dea);
			}
		}
		
		function OnEndDrag(_drob)
		{
			if (self.onenddrag)
			{
				var dea = new DragEventArgs(_drob)
				self.onenddrag(dea);
			}
		}
				
		
		// -- Private Class:  DragEventArgs -- //
		function DragEventArgs(_drob)
		{
			// More arguments to come later
			this.Element = _drob.getElement();
			this.CurrentPoint = geo.Functions.getElementPosition(this.Element);
		}		
		
		
		// -- Private Class: DraggableObject -- //
		function DraggableObject(_elem, _dragArea, _dragDirection, _useClone, _opacity, _guid)
		{
			// Keep a reference to 'this' DraggableObject
			var self = this;
			
			// -- PRIVATE VARIABLES -- //
			var m_elem;
			var m_clone; 
			var m_dragArea;
			var m_dragDirection;
			var m_opacity;
			var m_xOffset;
			var m_yOffset;
			var m_lastKnownPosition;
			var m_hotSpotDiv;
			var m_hotSpots;
			
			// -- PUBLIC PROPERTIES -- //
			this.UseClone;
			Construct();
			
			// -- "Constructor" --//
			function Construct()
			{
				m_elem = _elem;
				m_elem.drobGUID = _guid;
				m_clone = null;
				m_dragArea = _dragArea || _elem;
				m_dragDirection = _dragDirection || dd.DragDirection.Both;
				m_opacity = _opacity || .6;
				m_xOffset = 0;
				m_yOffset = 0;
				m_lastKnownPosition = null;
				m_hotSpotDiv = document.createElement('div');
				m_hotSpotDiv.id = 'theOneAndOnlyHotSpotIndicator';
				m_hotSpotDiv.style.position = 'absolute';
				m_hotSpotDiv.style.visibility = 'hidden';
				m_hotSpotDiv.style.zIndex = '20';
				document.body.appendChild(m_hotSpotDiv);
				m_hotSpots = new Array();
				
				self.UseClone = (_useClone != false) ? true : false;
				
				if (document.addEventListener)
					m_dragArea.addEventListener('mousedown', BeginDrag, true);
				else if (document.attachEvent)
				{
					m_dragArea.attachEvent('onmousedown', BeginDrag);
					m_dragArea.attachEvent('onmousemove', Drag);
					m_dragArea.attachEvent('onmouseup', EndDrag);
				}
			}
			
			// -- "Destructor" -- //
			this.destroy = function()
			{
				if (document.removeEventListener)
					m_dragArea.removeEventListener('mousedown', BeginDrag, true);
				else if (document.detachEvent)
				{
					m_dragArea.detachEvent('onmousedown', BeginDrag);
					m_dragArea.detachEvent('onmousemove', Drag);
					m_dragArea.detachEvent('onmouseup', EndDrag);
				}
				
				m_elem = null;
				m_dragArea = null;
				m_dragDirection = null
				m_xOffset = null;
				m_yOffset = null;
			};	
			
			
			// -- PUBLIC FUNCTIONS -- //
			this.getElement = function() { return (self.UseClone && m_clone) ? m_clone : m_elem; };
			this.getClone = function() { return m_clone; };
			this.getXOffset = function() { return m_xOffset; };
			this.setXOffset = function(_xOffset) { m_xOffset = _xOffset; };
			this.getYOffset = function() { return m_yOffset; };
			this.setYOffset = function(_yOffset) { m_yOffset = _yOffset; };
			this.getLastKnownPosition = function() { return m_lastKnownPosition; };
			this.setLastKnownPosition = function(_point) { m_lastKnownPosition = _point; };
			this.getDragDirection = function() { return m_dragDirection; };
			this.getDragArea = function() { return m_dragArea; };
			
			this.createClone = function()
			{
				m_clone = m_elem.cloneNode(true);
				
				if (m_clone.style && typeof m_clone.style.opacity != 'undefined')
					m_clone.style.opacity = m_opacity;
				else if (m_clone.style && typeof m_clone.style.filter != 'undefined')
					m_clone.style.filter = 'alpha(opacity=' + m_opacity * 100 + ')';
				
				m_clone.style.opacity = m_opacity;				
				var cloneParent = m_elem.parentNode || document.body;
				cloneParent.appendChild(m_clone);
			};
			
			this.destroyClone = function()
			{
				if (!m_clone)
					return;
					
				var cloneParent = m_clone.parentNode;
				cloneParent.removeChild(m_clone);
				m_clone = null;
			};
			
			this.syncWithClone = function()
			{
				if (!m_clone)
					self.createClone();
					
				m_elem.style.top = m_clone.style.top;
				m_elem.style.left = m_clone.style.left;
			};
			
			this.createHotSpot = function(_rect)
			{
				if (_rect instanceof geo.Rectangle)
					m_hotSpots.push(_rect);
			};
			
			this.isInHotSpot = function()
			{
				var i;				
				for (i = 0; i < m_hotSpots.length; i++)
				{
					var hs = m_hotSpots[i];					
					var p1 = m_lastKnownPosition;
					var p2 = new geo.Point(m_lastKnownPosition.x + m_elem.offsetWidth, m_lastKnownPosition.y + m_elem.offsetHeight);
					var re = new geo.Rectangle(p1, p2);					
					document.getElementById('debug').value = hs.getP1().x + '<=' + re.getP1().x + ' : ' + re.getP2().x + '<=' + hs.getP2().x;
					var ol = hs.getOverlap(re);
					
					if (ol.collides())					
						{ return hs; }
				}
				
				m_hotSpotDiv.style.visibility = 'hidden';
				return null;
			}
			
			this.snap = function(_hotSpot)
			{
				m_elem.style.top = _hotSpot.getP1().y + 'px';
				m_elem.style.left = _hotSpot.getP1().x + 'px';				
				m_hotSpotDiv.style.visibility = 'hidden';
			}
			
			this.threatenSnap = function(_hotSpot)
			{
				if (m_hotSpotDiv.style.visibility == 'visible')
					{ return; }
				
				m_hotSpotDiv.style.top = _hotSpot.getP1().y + 'px';
				m_hotSpotDiv.style.left = _hotSpot.getP1().x + 'px';
				m_hotSpotDiv.style.height = m_elem.offsetHeight + 'px';
				m_hotSpotDiv.style.width = m_elem.offsetWidth + 'px';
				m_hotSpotDiv.style.padding = '2px';
				m_hotSpotDiv.style.backgroundColor = 'lemonchiffon';
				m_hotSpotDiv.style.border = 'dashed 2px black';
				if (m_hotSpotDiv.style.opacity)
					m_hotSpotDiv.style.opacity = '.8';
				else
					m_hotSpotDiv.style.filter = 'alpha(opacity=80)';
				
				m_hotSpotDiv.style.visibility = 'visible';
			};
		}	
	}
}