//------------------------------------------------------------------------------
// Microsoft.Web.Geometry Library
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: sstchur
// Dependencies: none
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

var Microsoft;
if (!Microsoft) Microsoft = {};
if (!Microsoft.Web) Microsoft.Web = {};
if (!Microsoft.Web.Geometry) Microsoft.Web.Geometry = {};

Microsoft.Web.Geometry.Point = function(_x, _y)
{
	// account for IE bug
	var self = this;
	
	// shortcut for Microsoft.Web.Geometry Namespace chain
	var geo = Microsoft.Web.Geometry;
	
	/// PUBLIC VARIABLES ///
	this.x = _x;
	this.y = _y;


	// -- PUBLIC FUNCTIONS -- //
	
	// add the values {_x} and {_y} to the current point and returns a new point
	this.add = function(_x, _y)
	{
		var pt = new geo.Point(self.x + _x, self.y + _y);
		return pt;
	}
	
	// determines the distance between two points and returns the result
	this.getDistanceFrom = function(_pt)
	{
		var tmp = Math.pow(_pt.x - self.x, 2) + Math.pow(_pt.y - self.y, 2);
		var s = Math.sqrt(tmp);
		return s;
	}
				
}
			
Microsoft.Web.Geometry.Overlap =
{
	Range:
	{
		GreaterThanX: 1,
		LessThanX: 2,
		GreaterThanY: 4,
		LessThanY: 8,
		InXRange: 16,
		InYRange: 32,
		InRange: 48
	},
				
	getInstance: function(_r1, _r2)
	{
		// shortcut the Microsoft.Web.Geometry.Overlap Namespace chain
		var ol = Microsoft.Web.Geometry.Overlap;
		
		/// PRIVATE VARIABLES ///
		var m_r1 = _r1;
		var m_r2 = _r2;
		var m_range = 0;
		Construct();
		
		function Construct()
		{					
			if (m_r2.getP2().x > m_r1.getP2().x)
				m_range += ol.Range.GreaterThanX;
		
			if (m_r2.getP1().x < m_r1.getP1().x)
				m_range += ol.Range.LessThanX;
			
			if (m_r2.getP2().y > m_r1.getP2().y)
				m_range += ol.Range.GreaterThanY;
			
			if (m_r2.getP1().y < m_r1.getP1().y)
				m_range += ol.Range.LessThanY;
			
			if (m_r1.getP1().x <= m_r2.getP1().x && m_r2.getP2().x <= m_r1.getP2().x)
				m_range += ol.Range.InXRange;
				
			if (m_r1.getP1().y <= m_r2.getP1().y && m_r2.getP2().y <= m_r1.getP2().y)
				m_range += ol.Range.InYRange;
		}
		
		
		// -- PUBLIC FUNCTIONS -- //
		
		// return the range (relative position) of the overlap
		this.getRange = function()
		{
			return m_range;
		}
		
		// returns the left x distance by which r2 "bleeds over" r1
		this.getLeftXBleed = function()
		{
			if (m_range & ol.Range.LessThanX)
				return Math.abs(m_r1.getP1().x - m_r2.getP1().x);
			else
				return 0;					// no bleed
		}
		
		// return the right x distance by which r2 "bleeds over" r1
		this.getRightXBleed = function()
		{
			if (m_range & ol.Range.GreaterThanX)
				return m_r2.getP2().x - m_r1.getP2().x;
			else
				return 0;					// no bleed
		}
		
		// returns the top y distance by which r2 "bleeds over" r1
		this.getTopYBleed = function()
		{
			if (m_range & ol.Range.LessThanY)
				return Math.abs(m_r1.getP1().y - m_r2.getP1().y);
			else
				return 0;					// no bleed
		}
		
		// returns the bottom y distance by which r2 "bleeds over" r1
		this.getBottomYBleed = function()
		{
			if (m_range & ol.Range.GreaterThanY)
				return m_r2.getP2().y - m_r1.getP2().y;
			else
				return 0;					// no bleed
		}
	}
}
			
Microsoft.Web.Geometry.Rectangle = function(_p1, _p2)
{
	// account for IE bug
	var self = this;
	
	/// PRIVATE VARIABLES ///
	var m_p1 = _p1;
	var m_p2 = _p2;
	var m_nHeight;
	var m_nWidth;
	Construct();
	
	function Construct()
	{
		CalculateSize();
	}				
	
	
	// -- PUBLIC FUNCTIONS -- //
	
	// moves the rectangle's to the new (x, y) coordinate specified by point {_p}
	this.move = function(_p)
	{
		m_p1.x = _p.x;
		m_p1.y = _p.y;
		m_p2.x = _p.x + m_nWidth;
		m_p2.y = _p.y + m_nHeight;
	}
	
	// returns the upper-left point of the rectangle
	this.getP1 = function()
	{
		return m_p1;
	}
	
	// returns the lower-right point of the rectangle
	this.getP2 = function()
	{
		return m_p2;
	}
	
	// sets the upper-left point of the rectangle
	this.setP1 = function(_p)
	{
		m_p1 = _p;
		CalculateSize();
	}
	
	// sets the lower-right point of the rectangle
	this.setP2 = function(_p)
	{
		m_p2 = _p;
		CalculateSize();
	}
	
	// returns the width of the rectangle
	this.getWidth = function()
	{
		return m_nWidth;
	}
	
	// returns the height of the rectangle
	this.getHeight = function()
	{
		return m_nHeight;
	}
	
	// return true of {_p} is in the rectangle's area, false otherwise
	this.containsPoint = function(_p)
	{
		return (_p.x >= m_p1.x && _p.x <= m_p2.x && _p.y >= m_p1.y && _p.y <= m_p2.y);
	}
	
	// determine the overlap of rectangle, {_rect2} with this rectangle
	this.getOverlap = function(_rect2)
	{
		// shortcut the Microsoft.Web.Geometry Namespace chain
		var geo = Microsoft.Web.Geometry;
		
		return new geo.Overlap.getInstance(self, _rect2);
	}
		
	
	// -- PRIVATE FUNCTIONS -- //
	
	// calculates the size of the rectangle
	function CalculateSize()
	{
		m_nHeight = m_p2.y - m_p1.y;
		m_nWidth = m_p2.x - m_p1.x;
	}				
}

Microsoft.Web.Geometry.Functions = 
{
	// calculcates the slope of line, given any two points on that line
	getSlope: function(_p1, _p2)
	{
		return (_p2.y - _p1.y) / (_p2.x - _p1.x);
	},
			
	// calculates the y-intercept of a line, given a slope and a point on the line
	getYIntercept: function(_m, _p)
	{
		return _p.y - (_m * _p.x);
	},	
	
	// returns a point whose x, y coordinates are the upper-left position of some given DOM element
	getElementPosition: function(_elem)
	{
		// shortcut theMicrosoft.Web.Geometry Namespace chain
		var geo = Microsoft.Web.Geometry;
		
		var xPos = 0;
		var yPos = 0;
		while (_elem && _elem.offsetParent)
		{
			xPos += _elem.offsetLeft - _elem.scrollLeft;
			yPos += _elem.offsetTop - _elem.scrollTop;
			_elem = _elem.offsetParent;		
		}
		
		return new geo.Point(xPos, yPos);
	},
	
	getBestBoundingPoint: function(_elem, _currentPoint, _boundingRect)
	{
		// shortcut theMicrosoft.Web.Geometry Namespace chain
		var geo = Microsoft.Web.Geometry;
		
		if (!_currentPoint)
			_currentPoint = geo.Functions.getElementPosition(_elem);
		
		var elemRect = new geo.Rectangle(_currentPoint, new geo.Point(_currentPoint.x + _elem.offsetWidth, _currentPoint.y + _elem.offsetHeight));
		var ol = _boundingRect.getOverlap(elemRect);
		var rng = ol.getRange();
		
		// If the _currentPoint falls within the boundingArea, no need to do any further checks or calculations
		if ((rng & geo.Overlap.Range.InRange) == geo.Overlap.Range.InRange)
			return _currentPoint;
			
		// Determine the best point within the boundingArea to place the drob
		var left = _currentPoint.x;
		var top = _currentPoint.y;
		
		if (rng & geo.Overlap.Range.GreaterThanX)
			left = _boundingRect.getP2().x - elemRect.getWidth();
			
		if (rng & geo.Overlap.Range.LessThanX)
			left = _boundingRect.getP1().x;
			
		if (rng & geo.Overlap.Range.GreaterThanY)
			top = _boundingRect.getP2().y - elemRect.getHeight();
			
		if (rng & geo.Overlap.Range.LessThanY)
			top = _boundingRect.getP1().y;
			
		return new geo.Point(left, top);	
	}
}

